C     SUBROUTINE AXIS1
C
C     PURPOSE
C        TO DRAW AN ANNOTATED AXIS
C
C     USAGE
C        CALL AXIS1(X0,Y0,ITITLE,N,S,THETA,AMIN,DA,IDEC,DT,K)
C           X0     - X-COORDINATE OF THE START OF THE AXIS (INCHES)
C           Y0     - Y-COORDINATE OF THE START OF THE AXIS (INCHES)
C           ITITLE - LOCATION OF THE AXIS TITLE (PASSED BY REFERENCE)
C                    OR A HOLLERITH STRING CONTAINING THE TITLE
C           N      - NUMBER OF CHARCTERS IN THE TITLE
C                     N<0 - ANNOTATION ON CCW SIDE OF AXIS
C                     N>0 - ANNOTATION ON CW  SIDE OF AXIS
C           S      - LENGHT OF THE AXIS (INCHES)
C           THETA  - ANGLE AT WHICH THE AXIS IS TO BE PLOTTED
C           AMIN   - THE FIRST SCALE FACTOR
C           DA     - THE SECOND SCALE FACTOR
C           IDEC   - NUMBER OF DECIMAL PLACES TO PLOT IN ANNOTATION
C                    -9 <= IDEC <=9 NUMBER OF DECIMAL PLACES TO ANNOTATE
C           DT     - DISTANCE BETWEEN TIC MARKS
C           K      - EVERY KTH TIC MARK IS ANNOTATED
C                    K=0 - INDICATES LOGARITMIC SCALING
C
C     SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C        NUMBER - SUBROUTINE TO PLOT A NUMBER
C        SYMBOL - SUBROUTINE TO PLOT A SYMBOL OR CHARACTER STRING
C        PLOT   - SUBROUTINE TO POSITION PEN
C
C     COMMENTS
C        THIS ROUTINE WAS MODIFIED BY ED ANDERSON (NOAO), JULY, 1985
C
CALCOMP  PRODUCT NUMBER  411010*2-G              EBCDIC PUNCH
CALCOMP HCBS FOR CDC-6000     906/EM RS-232 ONLINE          JUL, 1979 *
CALCOMP  COPYRIGHT 1979 CALIFORNIA COMPUTER PRODUCTS INC.
C
      SUBROUTINE AXIS1(X0,Y0,ITITLE,N,S,THETA,AMIN,DA,IDEC,DT,K)
      DIMENSION ITITLE(2)
      PI=4.*ATAN(1.)
      STH=SIN(THETA*PI/180.)
      CTH=COS(THETA*PI/180.)
      IF (N.LT.0) THEN
         SIDE=-1.
         KN=-N
      ELSE
         SIDE=1.
         KN=N
      ENDIF
C
C..............................................DETERMINE IF ANNOTATION IS WANTED
      IF (DT.EQ.0.) GOTO 10
C
C.................................DETERMINE THE ANNOTATION MULTIPLICATION FACTOR
      IF (K.GT.0) THEN
         EX=0.0
         DVAL=ABS(DA)
         IF (DVAL.NE.0.) THEN
            IF (DVAL.GE.99.0) THEN
    1          IF (DVAL.GE.99.0) THEN
                  DVAL=DVAL/10.0
                  EX=EX+1.0
                  GOTO 1
               ENDIF
            ELSE IF (DVAL.LE.0.01) THEN
    2          IF (DVAL.LE.0.01) THEN
                  DVAL=DVAL*10.0
                  EX=EX-1.0
                  GOTO 2
               ENDIF
            ENDIF
         ENDIF
         VAL=AMIN*10.0**(-EX)
         DVAL=(DA*10.0**(-EX))*DT
      ELSE IF (K.EQ.0) THEN
         EX=0.
         VAL=AMIN
         XDA=DA
      ELSE
         CALL ERR(' AXIS1 ERROR:  ILLEGAL VALUE OF K')
         RETURN
      ENDIF
C
C......................................PLOT THE ANNOTATION (HEIGHT=0.105 INCHES)
      DY=0.15*SIDE-0.05
      XPOS=X0-DY*STH
      YPOS=Y0+DY*CTH
      IF (K.GT.0) THEN
         NTIC=INT(S/DT)+1
         NANN=IFIX(ANINT(FLOAT(NTIC)/FLOAT(K)))
         DO 3 I=1, NANN
            CALL NUMBER1(XPOS,YPOS,0.105,VAL,THETA,IDEC)
            VAL=VAL+(DVAL*FLOAT(K))
            XPOS=XPOS+(FLOAT(K)*DT*CTH)
            YPOS=YPOS+(FLOAT(K)*DT*STH)
    3    CONTINUE
      ELSE IF (K.EQ.0) THEN
         NTIC=INT(S/XDA)+1
         DO 4 I=1, NTIC
            CALL NUMBER1(XPOS,YPOS,0.105,VAL,THETA,IDEC)
            VAL=VAL+1.
            XPOS=XPOS+(XDA*CTH)
            YPOS=YPOS+(XDA*STH)
    4    CONTINUE
      ENDIF
C
C....................................................PLOT THE AXIS AND TIC MARKS
   10 CALL PLOT(X0+S*CTH,Y0+S*STH,3)
      IF (DT.EQ.0.) THEN
         CALL PLOT(X0,Y0,2)
      ELSE
         CALL WHERE(XPOS,YPOS,FCTR)
         DX=+0.07*SIDE*STH
         DY=-0.07*SIDE*CTH
         CALL PLOT(XPOS+DX,YPOS+DY,2)
         CALL PLOT(XPOS,YPOS,2)
         DO 5 I=1, NTIC
            CALL PLOT(XPOS,YPOS,2)
            CALL PLOT(XPOS+DX,YPOS+DY,2)
            CALL PLOT(XPOS,YPOS,2)
            IF (K.GT.0) THEN
               XPOS=XPOS-(DT*CTH)
               YPOS=YPOS-(DT*STH)
            ELSE
               XPOS=XPOS-(XDA*CTH)
               YPOS=YPOS-(XDA*STH)
            ENDIF
    5    CONTINUE
      ENDIF
C
C..........................................PLOT THE TITLE (HEIGHT = 0.14 INCHES)
      Z=FLOAT(KN)
      IF (EX.NE.0.) Z=Z+7.0
      DX=-.07*Z+S/2.
      DY=0.325*SIDE-0.075
      XPOS=X0+DX*CTH-DY*STH
      YPOS=Y0+DY*CTH+DX*STH
      CALL SYMBOL(XPOS,YPOS,0.14,ITITLE(1),THETA,KN)
      IF (EX.NE.0.) THEN
         Z=KN+2
         XPOS=XPOS+Z*CTH*0.14
         YPOS=YPOS+Z*STH*0.14
         CALL SYMBOL(XPOS,YPOS,0.14,3H*10,THETA,3)
         XPOS=XPOS+(3.0*CTH-0.8*STH)*0.14
         YPOS=YPOS+(3.0*STH+0.8*CTH)*0.14
         CALL NUMBER(XPOS,YPOS,0.07,EX,THETA,-1)
      ENDIF
C
C.......................................................................FINISHED
      RETURN
      END
