C     SUBROUTINE AXIS2
C
C     PURPOSE
C        TO DRAW AN ANNOTATED AXIS
C
C     USAGE
C        CALL AXIS2(X0,Y0,ITITLE,N,THT,AHT,S,THETA,AMIN,DA,IDEC,DT,K)
C           X0     - X-COORDINATE OF THE START OF THE AXIS (INCHES)
C           Y0     - Y-COORDINATE OF THE START OF THE AXIS (INCHES)
C           ITITLE - LOCATION OF THE AXIS TITLE (PASSED BY REFERENCE)
C                    OR A HOLLERITH STRING CONTAINING THE TITLE
C           N      - NUMBER OF CHARCTERS IN THE TITLE
C                       N = 0 -> NO TITLE PLOTTED
C           THT    - HEIGHT OF CHARACTERS IN TITLE
C                       THT = 0. -> NO TITLE PLOTTED
C                       THT > 0. -> TITLE ON COUNTER-CLOCKWISE SIDE OF AXIS
C                       THT < 0. -> TITLE ON CLOCKWISE SIDE OF AXIS
C           AHT    - HEIGHT OF ANNOTATION
C                       AHT = 0. -> NO ANNOTATION PLOTTED
C                       AHT > 0. -> TITLE ON COUNTER-CLOCKWISE SIDE OF AXIS
C                       AHT < 0. -> TITLE ON CLOCKWISE SIDE OF AXIS
C           S      - LENGHT OF THE AXIS (INCHES)
C           THETA  - ANGLE AT WHICH THE AXIS IS TO BE PLOTTED
C           AMIN   - THE FIRST SCALE FACTOR
C           DA     - THE SECOND SCALE FACTOR
C           IDEC   - NUMBER OF DECIMAL PLACES TO PLOT IN ANNOTATION
C                    -9 <= IDEC <=9 NUMBER OF DECIMAL PLACES TO ANNOTATE
C           DT     - DISTANCE BETWEEN TIC MARKS
C                    DT=0. NO TIC MARKS PLOTTED
C                    
C           K      - EVERY KTH TIC MARK IS ANNOTATED
C                    K=0 - INDICATES LOGARITHMIC SCALING
C
C     SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C        NUMBER - SUBROUTINE TO PLOT A NUMBER
C        SYMBOL - SUBROUTINE TO PLOT A SYMBOL OR CHARACTER STRING
C        PLOT   - SUBROUTINE TO POSITION PEN
C
C     COMMENTS
C        THIS ROUTINE WAS MODIFIED BY ED ANDERSON (NOAO), JULY, 1985
C
CALCOMP  PRODUCT NUMBER  411010*2-G              EBCDIC PUNCH
CALCOMP HCBS FOR CDC-6000     906/EM RS-232 ONLINE          JUL, 1979 *
CALCOMP  COPYRIGHT 1979 CALIFORNIA COMPUTER PRODUCTS INC.
C
      SUBROUTINE AXIS2(X0,Y0,ITITLE,N,THT,AHT,S,THETA,AMIN,DA,IDEC,DT,K)
      DIMENSION ITITLE(2)
      PI=4.*ATAN(1.)
      STH=SIN(THETA*PI/180.)
      CTH=COS(THETA*PI/180.)
      TICHT=0.8*ABS(AHT)
      DY=0.
      DX=0.
C
C..................................DETERMINE WHICH SIDE OF AXIS TITLE WILL BE ON
      IF (THT.GT.0) THEN
         TSIDE=1.
      ELSE IF (THT.LT.0) THEN
         TSIDE=-1.
      ELSE
         TSIDE=0.
      ENDIF
C
C.................................DETERMINE THE ANNOTATION MULTIPLICATION FACTOR
      IF (AHT.EQ.0.) THEN
         GOTO 1
      ELSE IF (AHT.GT.0.) THEN
         ASIDE=1.
      ELSE IF (AHT.LT.0.) THEN
         ASIDE=-1.
      ELSE
         ASIDE=0.
      ENDIF

      IF (K.GT.0) THEN
         EX=0.0
         DVAL=ABS(DA)
         IF (DVAL.NE.0.) THEN
            IF (DVAL.GE.99.0) THEN
   10          IF (DVAL.GE.99.0) THEN
                  DVAL=DVAL/10.0
                  EX=EX+1.0
                  GOTO 10
               ENDIF
            ELSE IF (DVAL.LE.0.01) THEN
   20          IF (DVAL.LE.0.01) THEN
                  DVAL=DVAL*10.0
                  EX=EX-1.0
                  GOTO 20
               ENDIF
            ENDIF
         ENDIF
         VAL=AMIN*10.0**(-EX)
         DVAL=(DA*10.0**(-EX))*DT
      ELSE IF (K.EQ.0) THEN
         EX=0.
         VAL=AMIN
         XDA=DA
      ELSE
         CALL ERR(' AXIS2 ERROR:  ILLEGAL VALUE OF K')
         RETURN
      ENDIF
C
C............................................................PLOT THE ANNOTATION
      IF (ASIDE.EQ.1.) THEN
         IF (TSIDE.GE.0) THEN
            DYA=-AHT-0.1
         ELSE
            DYA=-AHT-0.1-TICHT
         ENDIF
      ELSE
         IF (TSIDE.LE.-1) THEN
            DYA=0.1
         ELSE
            DYA=0.1+TICHT
         ENDIF
      ENDIF
      XPOS=X0-DYA*STH
      YPOS=Y0+DYA*CTH
      IF (K.GT.0) THEN
         NTIC=INT(S/DT)+1
         NANN=IFIX(ANINT(FLOAT(NTIC)/FLOAT(K)))
         DO 30 I=1, NANN
            CALL NUMBER1(XPOS,YPOS,ABS(AHT),VAL,THETA,IDEC)
            VAL=VAL+(DVAL*FLOAT(K))
            XPOS=XPOS+(FLOAT(K)*DT*CTH)
            YPOS=YPOS+(FLOAT(K)*DT*STH)
   30    CONTINUE
      ELSE
         NTIC=INT(S/XDA)+1
         DO 40 I=1, NTIC
            CALL NUMBER1(XPOS,YPOS,ABS(AHT),VAL,THETA,2)
            VAL=VAL+1.
            XPOS=XPOS+(XDA*CTH)
            YPOS=YPOS+(XDA*STH)
   40    CONTINUE
      ENDIF
C
C....................................................PLOT THE AXIS AND TIC MARKS
    1 CALL PLOT(X0+S*CTH,Y0+S*STH,3)
      IF (DT.EQ.0.) THEN
         CALL PLOT(X0,Y0,2)
      ELSE
         CALL WHERE(XPOS,YPOS,FCTR)
         IF (TSIDE.EQ.0.) TSIDE=ASIDE
         DX=-TICHT*TSIDE*STH
         DY=+TICHT*TSIDE*CTH
         CALL PLOT(XPOS+DX,YPOS+DY,2)
         CALL PLOT(XPOS,YPOS,2)
         DO 50 I=1, NTIC
            CALL PLOT(XPOS,YPOS,2)
            CALL PLOT(XPOS+DX,YPOS+DY,2)
            CALL PLOT(XPOS,YPOS,2)
            IF (K.GT.0) THEN
               XPOS=XPOS-(DT*CTH)
               YPOS=YPOS-(DT*STH)
            ELSE
               XPOS=XPOS-(XDA*CTH)
               YPOS=YPOS-(XDA*STH)
            ENDIF
   50    CONTINUE
      ENDIF
C
C.................................................................PLOT THE TITLE
      IF (THT.EQ.0.) GOTO 2
      IF (TSIDE.EQ.1.) THEN
         IF (ASIDE.GE.0.) THEN
             DY=DYA-0.1-THT
         ELSE
             DY=-0.1-THT
         ENDIF
      ELSE
         IF (ASIDE.LE.0.) THEN
            DY=DYA+ABS(AHT)+0.1
         ELSE
            DY=0.1
         ENDIF
      ENDIF
      Z=FLOAT(N)
      IF (EX.NE.0.) Z=Z+7.
      DX=(S/2.)-(6./14.)*ABS(THT)*Z
      XPOS=X0+DX*CTH-DY*STH
      YPOS=Y0+DY*CTH+DX*STH
      CALL SYMBOL(XPOS,YPOS,ABS(THT),ITITLE(1),THETA,N)
      IF (EX.NE.0.) THEN
         Z=N+2
         XPOS=XPOS+Z*CTH*0.14
         YPOS=YPOS+Z*STH*0.14
         CALL SYMBOL(XPOS,YPOS,0.14,3H*10,THETA,3)
         XPOS=XPOS+(3.0*CTH-0.8*STH)*0.14
         YPOS=YPOS+(3.0*STH+0.8*CTH)*0.14
         CALL NUMBER(XPOS,YPOS,ABS(THT),EX,THETA,-1)
      ENDIF
C
C.......................................................................FINISHED
    2 RETURN
      END
