
C
C     PURPOSE
C        PLOT LINEAR AXES WHICH CROSS AT THE DATA VALUE (0.,0.) WITH ONLY
C        ONE ANNOTATION FOR THE CROSSING POINT.
C
C     USAGE
C        CALL CRAXES(SX,SY,XMIN,XDA,YMIN,YDA,IDEC,AHT)
C          SX     - LENGTH OF THE X-AXIS (INCHES)
C          SY     - LENGTH OF THE Y-AXIS (INCHES)
C          XMIN   - THE MINIMUM X DATA VALUE (FIRST X SCALE FACTOR)
C          XDA    - THE INCREMENT/INCH IN THE X DIRECTION (2ND X SCALE FACTOR)
C          YMIN   - THE MINIMUM Y DATA VALUE (FIRST Y SCALE FACTOR)
C          YDA    - THE INCREMENT/INCH IN THE Y DIRECTION (2ND Y SCALE FACTOR)
C          IDEC   - NUMBER OF DIGITS TO BE PLOTTED TO THE RIGHT OF THE DECIMAL
C                     POINT IN THE ANNOTATION OF BOTH AXES
C          AHT    - HEIGHT OF ANNOTATION
C
C     SUBROUTINES AND FUNCTION SUBPROGRAMS USED
C        PLOT   - SUBROUTINE TO MOVE THE PEN
C        NUMBER - SUBROUTINE TO PLOT A NUMBER
C
C     COMMENTS
C        THIS PROGRAM CREATED BY ED ANDERSON (NOAO), JULY, 1985.
C
      SUBROUTINE CRAXES(SX,SY,XMIN,XDA,YMIN,YDA,IDEC,AHT)
      XMAX=XMIN+(SX*XDA)
      YMAX=YMIN+(SY*YDA)
C
C.........................................DETERMINE WHERE ALONG THE X-AXIS 0. IS
      IF ((XMIN*XMAX).GT.0.) THEN
         CALL ERR(' CRAXES ERROR: ORIGIN OUTSIDE X-RANGE')
         RETURN
      ENDIF
      ZPOSX=-XMIN/XDA
C
C.........................................DETERMINE WHERE ALONG THE Y-AXIS 0. IS
      IF ((YMIN*YMAX).GT.0.) THEN
         CALL ERR(' CRAXES ERROR:  ORIGIN OUTSIDE Y-RANGE')
         RETURN
      ENDIF
      ZPOSY=-YMIN/YDA
C
C....................................................DETERMINE SIZE OF TIC MARKS
      TICHT=0.8*AHT
      DEL1=-TICHT-.1-AHT
      DEL2=TICHT+.1
C
C...................................................DRAW AND ANNOTATE THE X-AXIS
      DO 4 IAXIS=1, 2
         IF (IAXIS.EQ.1) THEN
            POS=ZPOSX-AINT(ZPOSX)
            ZPOS=ZPOSX
            S=SX
            DA=XDA
            AMIN=XMIN
            NTIC=0
            CALL PLOT (0.,ZPOSY,3)
         ELSE
            POS=ZPOSY-AINT(ZPOSY)
            ZPOS=ZPOSY
            S=SY
            DA=YDA
            AMIN=YMIN
            NTIC=0
            CALL PLOT (ZPOSX,0.,3)
         ENDIF
    1    IF (POS.GT.S) THEN
            IF (IAXIS.EQ.1) THEN
               CALL PLOT(S,ZPOSY,1)
               GOTO 2
            ELSE
               CALL PLOT(ZPOSX,S,1)
               GOTO 2
            ENDIF
         ENDIF
         IF (IAXIS.EQ.1) THEN
            CALL PLOT (POS,ZPOSY,2)
            IF (ABS(POS-ZPOS).GT.0.002) THEN
               CALL PLOT (POS,ZPOSY+TICHT,1)
               CALL PLOT (POS,ZPOSY-TICHT,1)
               CALL PLOT (POS,ZPOSY,1)
               NTIC=NTIC+1
            ENDIF
         ELSE
            CALL PLOT (ZPOSX,POS,2)
            IF (ABS(POS-ZPOS).GT.0.002) THEN
               CALL PLOT (ZPOSX+TICHT,POS,1)
               CALL PLOT (ZPOSX-TICHT,POS,1)
               CALL PLOT (ZPOSX,POS,1)
               NTIC=NTIC+1
            ENDIF
         ENDIF
         POS=POS+1.
         GOTO 1
C
C.................................DETERMINE THE ANNOTATION MULTIPLICATION FACTOR
    2    POS=POS-1.
         EX=0.0
         DVAL=ABS(DA)
         IF (DVAL.NE.0.) THEN
            IF (DVAL.GE.99.0) THEN
   10          IF (DVAL.GE.99.0) THEN
                  DVAL=DVAL/10.0
                  EX=EX+1.0
                  GOTO 10
               ENDIF
            ELSE IF (DVAL.LE.0.01) THEN
   11          IF (DVAL.LE.0.01) THEN
                  DVAL=DVAL*10.0
                  EX=EX-1.0
                  GOTO 11
               ENDIF
            ENDIF
         ENDIF
         VAL=(AMIN+POS*DA)*10.0**(-EX)
         DVAL=DA*10.0**(-EX)
C
C................................................................PLOT ANNOTATION
         DO 3 I=1, NTIC+1
            IF (ABS(POS-ZPOS).GT.0.002) THEN
               IF (IAXIS.EQ.1) THEN
                  CALL NUMBER1(POS,ZPOSY+DEL1,AHT,VAL,0.,IDEC)
               ELSE
                  CALL NUMBER1(ZPOSX-DEL2,POS,AHT,VAL,90.,IDEC)
               ENDIF
            ELSE
C............................................................ANNOTATE THE ORIGIN
               IF (IAXIS.EQ.1) THEN
                  ZVAL=0.
                  CALL NUMBER(ZPOSX+.05,ZPOSY+DEL1,AHT,ZVAL,0.,0)
               ENDIF
            ENDIF
            POS=POS-1.
            VAL=VAL-DVAL
    3    CONTINUE
         IF (EX.NE.0.) THEN
            IF (IAXIS.EQ.1) THEN
               CALL SYMBOL(S+.2,ZPOSY-.5*AHT,AHT,3H*10,0.,3)
               CALL WHERE(XPOS,YPOS,FCTR)
               CALL NUMBER(XPOS,YPOS+.75*AHT,.8*AHT,EX,0.,-1)
            ELSE
               CALL SYMBOL(ZPOSX+.5*AHT,S+.2,AHT,3H*10,90.,3)
               CALL WHERE(XPOS,YPOS,FCTR)
               CALL NUMBER(XPOS-.75*AHT,YPOS,.8*AHT,EX,90.,-1)
            ENDIF
         ENDIF
    4 CONTINUE

      RETURN
      END
