c************************************************************************
c
c                           IO_IMFORT.F 
c
c           Package of IMFORT i/o routines for use with DoPhot
c
c           Written by Mark Stevens, NOAO Tucson, November 1990
c************************************************************************

      subroutine rdiraf (lhead,head,ncols,nrows,maxdim,imdata,file,im)

      integer       lhead, ncols, nrows, im, dtype, naxis, maxdim, ier
      integer       axlen(7)
      integer*2     imdata (maxdim) 
      character*80  head(lhead), errmsg
      character*(*) file
c-----------------------------------------------------------------------
c Open the IRAF image file, get the number of rows and columns, then
c read the header and pixel data. IMFORT routines appear in uppercase.

      call IMOPEN (file, 1, im, ier)
      if (ier .ne. 0) goto 999

      call IMGSIZ (im, axlen, naxis, dtype, ier)
      if (ier .ne. 0) goto 999
      ncols = axlen(1)
      nrows = axlen(2)

      if (dtype .ne. 3) then
         write (6,10)
10       format(' IRAF input image is not type SHORT INTEGER')  
         goto 1000 
      else if (naxis .ne. 2) then
         write (6, 20)
20       format (' IRAF input image is not two dimensional')
         goto 1000 
      end if

      call gethead (lhead, head, im, ier)
      if (ier .ne. 0) goto 999

      call getdata (im, maxdim, imdata, ncols, nrows, ier)
      if (ier .ne. 0) goto 999

      return

999   call IMEMSG (ier, errmsg)
      write (6,'('' Error : '',a80)') errmsg
1000  call IMCLOS (im, ier)
      stop 'Error from rdiraf'
      end
 
c************************************************************************

      subroutine gethead (lhead, head, im, ier)

      integer      lhead, im, dtype, kwl, ier, line, nchr
      character*80 head(lhead), patstr
      character*8  kwname
      character*20 valstr
      character*20 chrstr
      character*47 comstr
      logical      sortit
c--------------------------------------------------------------------------
c Reads the header from the IRAF image file.

      patstr = '*'
      sortit = .false.
      line = 1
 
c  Open the image header keyword list
      call IMOKWL (im, patstr, sortit, kwl, ier)
      if (ier .ne. 0) return

c  Get the next keyword from the list. When at the end of the list ier
c  becomes nonzero. Get the data type of the keyword and comment string, if any.
c  Get the value of the keyword. Automatically converted to a string.

      do while ((ier .eq. 0) .and. (line .le. lhead))

         call IMGNKW (kwl, kwname, ier)
         if (ier .eq. 0) then
            call IMTYPK (im, kwname, dtype, comstr, ier)
            if (ier .ne. 0) return

            call IMGKWC (im, kwname, valstr, ier)
            if (ier .ne. 0) return

            if (dtype .ne. 2) then
               head (line) = kwname//'= '//valstr//' / '//comstr 
            else
               nchr = 20
               do while ((nchr.gt.0) .and. (valstr(nchr:nchr).eq.' '))
                  nchr = nchr - 1
               end do 
               chrstr(1:1) = ''''
               chrstr(2:nchr+1) = valstr(1:nchr)
               chrstr(nchr+2:nchr+2) = ''''
               head (line) = kwname//'= '//chrstr//' / '//comstr 
            end if
            line = line + 1
         end if

      end do 

c  Close the keyword list 
      call IMCKWL (kwl, ier) 
      if (ier .ne. 0) return

      ier = 0
      return
      end 

c*********************************************************************

      subroutine getdata (im, maxdim, imdata, ncols, nrows, ier)

      integer   im, maxdim, nrows, ncols, ier, i, j, n
      integer*2 imdata (maxdim)
      integer*2 sbuf (4096)
c---------------------------------------------------------------------
c Read the pixel data from the IRAF image file.

      do i = 1, nrows
        call IMGL2S (im, sbuf, i, ier)
        if (ier .ne. 0) return
        n = ncols*(i - 1)
        do j = 1, ncols
          imdata(n + j) = sbuf(j)
        end do
      end do

      ier = 0
      return
      end 

c**********************************************************************

      subroutine wriraf (filenm,imgin,imgout,ncols,nrows,maxdim,imdata)

      integer       maxdim, ier, nrows, ncols, imgin, imgout 
      integer*2     imdata (maxdim)
      character*(*) filenm
      character*80  errmsg
c------------------------------------------------------------------------
c  Make a copy of the input IRAF image with the output file name. The
c  header is copied from the input image.

      call IMOPNC (filenm, imgin, imgout, ier)
      if (ier .ne. 0) goto 999 

c  Write the pixel data to the new image file.

      call putdata (imgout, ncols, nrows, maxdim, imdata, ier)
      if (ier .ne. 0) goto 999

c  Close the input image file.
      call IMCLOS (imgin, ier)
      if (ier .ne. 0) goto 1000

c  Close the output image file.
      call IMCLOS (imgout, ier)
      if (ier .ne. 0) goto 1000

      return

999   call IMCLOS (imgin, ier)
      call IMCLOS (imgout, ier)
1000  call IMEMSG (ier, errmsg)
      write (6,'('' Error : '',a80)') errmsg
      stop 'Error from wriraf'
      end

c************************************************************************

      subroutine putdata (im, ncols, nrows, maxdim, imdata, ier)

      integer       maxdim, im, ier, nrows, ncols, i, j, n  
      integer*2     imdata (maxdim)
      integer*2     shortp (4096)
c------------------------------------------------------------------------
c  Write one row of pixels at a time to the new IRAF image file.

      do i = 1, nrows
        n = ncols*(i - 1)
        do j = 1, ncols
          shortp (j) = imdata(n + j)
        end do
        call IMPL2S (im, shortp, i, ier) 
        if (ier .ne. 0) return
      end do

      ier = 0
      return
      end
