include <gset.h>
include "lpl.h"
include "lpx.h"


# Note dependency of multiple markertypes on values of NUM_GIO_LTYPE and
# NUM_GIO_MARKER in lpl.h; note that if GIO marker codes are modified, code may
# need to be modified in markercode loop below due to GM_FILL.

# LPL_GLABAX -- Draw axes, ticks, labels, and titles.  Do gsets for line and
# markertypes to be drawn in subsequent lpl_drawdata.

procedure lpl_glabax (lpl, lpx, gd, curve, ncurves)

pointer	lpl			# longplot descriptor
pointer	lpx			# curve arrays to aid in stack plotting
pointer	gd			# graphics descriptor
int	curve			# current curve number
int	ncurves			# total curves

bool	overtrace, stack, series
int	ttl_offset
real	orig_ticlen, ticlen

int	strlen()
real	ggetr()

begin
	# These are used heavily; convert to logicals:
	overtrace = (LPL_MULTIPLE(lpl) == MULT_OVERTRACE)
	stack     = (LPL_MULTIPLE(lpl) == MULT_STACK || LPL_MULTIPLE(lpl) ==
	    MULT_STACK_DY)
	series    = (LPL_MULTIPLE(lpl) == MULT_SERIES)

	# Rounding.  Note that these were NOT set for gd in lpl_calc_coords.
	if (LPL_XROUND(lpl) == YES)
	    call gseti (gd, G_XROUND, YES)
	if (LPL_YROUND(lpl) == YES)
	    call gseti (gd, G_YROUND, YES)

	# Titles; stack and overtrace plots do not get an image title even
	# if "title=imtitle".  Instead, a multi-line title will appear above
	# the topmost plot containing only the input specifications, one per
	# line.

	ttl_offset = Memi[LPL_TITLENDX(lpl) + curve - 1] - 1
	if (!series && curve == 1) {

	    # Draw the first curve's title only, above the topmost curve.
	    # This title may have been loaded with individual inputspecs,
	    # one per line, by lpl_rdcrvs or lpl_indirection.

	    call gsview (gd, LPX_VXMIN(lpx,0),LPX_VXMAX(lpx,0),
		LPX_VYMIN(lpx,0), LPX_VYMAX(lpx,ncurves-1))
	    call gseti (gd, G_DRAWAXES, 0)
	    call gseti (gd, G_DRAWGRID, NO)
	    call gseti (gd, G_DRAWTICKS, NO)
	    call gseti (gd, G_LABELTICKS, NO)
	    call gseti (gd, G_DRAWTITLE, YES)
	    call glabax (gd, TITLE(LPL_TITLE(lpl), 0), "", "")
	    call gsview (gd, LPX_VXMIN(lpx,0), LPX_VXMAX(lpx,0),
		LPX_VYMIN(lpx,0), LPX_VYMAX(lpx,0))
	    call gseti (gd, G_DRAWTITLE, NO)
	    call amovkc (EOS, TITLE(LPL_TITLE(lpl), 0), 
		strlen (TITLE(LPL_TITLE(lpl), 0)))
	    call amovki (1, Memi[LPL_TITLENDX(lpl)], ncurves)

	} else if (!series)
	    call gseti (gd, G_DRAWTITLE, NO)

	else
	    call gseti (gd, G_DRAWTITLE, YES)

	# Axis and tick drawing and labelling:
	if (overtrace && curve > 1) {
	    call gseti (gd, G_DRAWAXES, 0)
	    call gseti (gd, G_XDRAWTICKS, NO)
	    call gseti (gd, G_YDRAWTICKS, NO)
	    call gseti (gd, G_XLABELTICKS, NO)
	    call gseti (gd, G_YLABELTICKS, NO)
	    call gseti (gd, G_XLABELAXIS, NO)
	    call gseti (gd, G_YLABELAXIS, NO)
	} else if (stack) {
	    if (curve == 1 && ncurves == 1) {
		call gseti (gd, G_DRAWAXES, 3)
		call gseti (gd, G_XDRAWTICKS, YES)
		call gseti (gd, G_YDRAWTICKS, YES)
		call gseti (gd, G_XLABELTICKS, LPL_TICKLABEL(lpl))
		call gseti (gd, G_YLABELTICKS, LPL_TICKLABEL(lpl))
		call gseti (gd, G_XLABELAXIS, YES)
		call gseti (gd, G_YLABELAXIS, YES)
	    } else if (curve == 1) {
		call gseti (gd, G_XDRAWAXES, 1)
		call gseti (gd, G_YDRAWAXES, 3)
		call gseti (gd, G_XDRAWTICKS, YES)
		call gseti (gd, G_YDRAWTICKS, YES)
		call gseti (gd, G_XLABELTICKS, LPL_TICKLABEL(lpl))
		call gseti (gd, G_YLABELTICKS, NO)
		call gseti (gd, G_XLABELAXIS, YES)
		call gseti (gd, G_YLABELAXIS, NO)
	    } else if (curve > 1 && curve == ncurves) {
		call gseti (gd, G_XDRAWAXES, 2)
		call gseti (gd, G_YDRAWAXES, 3)
		call gseti (gd, G_XDRAWTICKS, YES)
		call gseti (gd, G_YDRAWTICKS, YES)
		call gseti (gd, G_XLABELTICKS, NO)
		call gseti (gd, G_YLABELTICKS, NO)
		call gseti (gd, G_XLABELAXIS, NO)
		call gseti (gd, G_YLABELAXIS, NO)
	    } else if (curve > 1) {
		call gseti (gd, G_XDRAWAXES, 0)
		call gseti (gd, G_YDRAWAXES, 3)
		call gseti (gd, G_XDRAWTICKS, NO)
		call gseti (gd, G_YDRAWTICKS, YES)
		call gseti (gd, G_XLABELTICKS, NO)
		call gseti (gd, G_YLABELTICKS, NO)
		call gseti (gd, G_XLABELAXIS, NO)
		call gseti (gd, G_YLABELAXIS, NO)
	    }

	} else {

	    call gseti (gd, G_DRAWAXES, 3)
	    call gseti (gd, G_XDRAWTICKS, YES)
	    call gseti (gd, G_YDRAWTICKS, YES)
	    call gseti (gd, G_XLABELTICKS, LPL_TICKLABEL(lpl))
	    call gseti (gd, G_YLABELTICKS, LPL_TICKLABEL(lpl))
	    call gseti (gd, G_XLABELAXIS, YES)
	    call gseti (gd, G_YLABELAXIS, YES)
	}

	# X-axis tick spacing:  if negative, determine from scale.
	if (LPL_MAJRX(lpl) < 0)
	    LPL_MAJRX(lpl) = nint ((LPX_WXMAX(lpx,curve-1) -
		LPX_WXMIN(lpx,curve-1)) / LPX_WXSCALE(lpx,curve-1))
	call gseti (gd, G_XNMAJOR, LPL_MAJRX(lpl))
	call gseti (gd, G_XNMINOR, LPL_MINRX(lpl))

	# Y-axis tick spacing:  if negative, determine from scale.
	if (LPL_MAJRY(lpl) < 0)
	    LPL_MAJRY(lpl) = nint ((LPX_WYMAX(lpx,curve-1) -
		LPX_WYMIN(lpx,curve-1)) / LPX_WYSCALE(lpx,curve-1))
	call gseti (gd, G_YNMAJOR, LPL_MAJRY(lpl))
	call gseti (gd, G_YNMINOR, LPL_MINRY(lpl))

	# Override bold text and tick generation unless requested.
	if (LPL_BOLD(lpl) == NO) {
	    call gseti (gd, G_TXFONT, GT_ROMAN)
	    call gsetr (gd, G_MAJORWIDTH, 1.0)
	    call gsetr (gd, G_MINORWIDTH, 1.0)
	    call gsetr (gd, G_AXISWIDTH, 1.0)
	} else {
	    call gseti (gd, G_TXFONT, GT_BOLD)
	    call gsetr (gd, G_MAJORWIDTH, 2.0)
	    call gsetr (gd, G_MINORWIDTH, 2.0)
	    call gsetr (gd, G_AXISWIDTH, 2.0)
	}

	# Line, markertypes:  if specified in indirection file for each curve,
	# use them; else if we really need to distinguish them, use sequentially
	# increasing ones from GIO repertoire (must watch out for GM_FILL == 1).

	call gseti (gd, G_PLTYPE, GL_SOLID)
	if (LPL_PLOTMODE(lpl) != PLOT_MARK)
	    if (LPX_LINETYPE(lpx,curve-1) != NULL)
		call gseti (gd, G_PLTYPE, LPX_LINETYPE(lpx,curve-1))
	    else if ((stack && LPL_LEGEND(lpl) == YES) || (overtrace &&
	        curve > 1))
		call gseti (gd, G_PLTYPE, mod (curve-1, NUM_GIO_LTYPE) + 1)

	# Scale the y-axis tick marks to those of the x-axis to avoid greatly
	# stretched ones in plots with x-dimension >> y-dimension.

	orig_ticlen = 0.6 * ggetr (gd, "ch")
	ticlen = orig_ticlen * LPX_PYSCALE(lpx,curve-1) / 
	    LPX_PXSCALE(lpx,curve-1)
	call gsetr (gd, G_YMAJORLENGTH, ticlen)
	call gsetr (gd, G_YMINORLENGTH, ticlen * 0.5)
	call gsetr (gd, G_XMAJORLENGTH, orig_ticlen)
	call gsetr (gd, G_XMINORLENGTH, orig_ticlen * 0.5)
	
	# Grid?
	if (LPL_GRID(lpl) == YES)
	    call gseti (gd, G_DRAWGRID, YES)
	else
	    call gseti (gd, G_DRAWGRID, NO)

	# Finally, draw it all.
	call glabax (gd, TITLE(LPL_TITLE(lpl),ttl_offset),
	    XLABEL(LPL_XLABEL(lpl)), YLABEL(LPL_YLABEL(lpl)))
end
