include	<mach.h>
include	<gio.h>
include <gset.h>
include "lpl.h"
include	"lblglabax.h"

# LBL_FIND_TICKS -- Find the optimal positions for the tick marks on an axis.
# Taken almost entirely from gio$glabax/glbfind.x with exceptions as noted.
# Does not modify the WCS; this has already been done for each curve.
# It is perhaps questionable to `gset' all those parameters above this
# level, then `gstat' them here, rather than bring the LPL, LPX datastructures
# down into here, but it seemed easier to modify the existing glabax
# code rather than restructure the whole thing to do something so similar.
# Besides, about 2/3 of those parameters are GIO defaults not available above.

procedure lbl_find_ticks (gp, coord, ax1, ax2)

pointer	gp			# graphics descriptor
int	coord			# x or y coordinate
pointer	ax1, ax2		# axis descriptors (output)

int	logflag, nminor, tmax, scaling, t1, t2
int	nmajor, round, gnminor, setaxispos, drawticks, drawaxes
int	labelticks
real	axispos1, axispos2, ticklabelsize, axislabelsize, axiswidth
real	wmin, wmax, junk, minorwidth, majorwidth
real	char_height, char_width, wctick, tval
real	p1, p2, tp1, tp2, wcp1, tick1, step, minor_step, length

int	gt_ndigits(), gstati()
bool	fp_equalr()
real	ggetr(), elogr(), aelogr(), gstatr()

#debug+
int	debug
common /deb/ debug
#debug-
begin
	if (coord == XCOORD) {
	    nmajor = gstati (gp, G_XNMAJOR)
	    round = gstati (gp, G_XROUND)
	    gnminor = gstati (gp, G_XNMINOR)
	    setaxispos = gstati (gp, G_XSETAXISPOS)
	    axispos1 = gstatr (gp, G_XAXISPOS1)
	    axispos2 = gstatr (gp, G_XAXISPOS2)
	    ticklabelsize = gstatr (gp, G_XTICKLABELSIZE)
	    drawticks = gstati (gp, G_XDRAWTICKS)
	    axislabelsize = gstatr (gp, G_XAXISLABELSIZE)
	    axiswidth = gstatr (gp, G_XAXISWIDTH)
	    minorwidth = gstatr (gp, G_XMINORWIDTH)
	    majorwidth = gstatr (gp, G_XMAJORWIDTH)
	    drawaxes = gstati (gp, G_XDRAWAXES)
	    labelticks = gstati (gp, G_XLABELTICKS)
	    call ggwind (gp, wmin,wmax, junk,junk)
	} else {
	    nmajor = gstati (gp, G_YNMAJOR)
	    round = gstati (gp, G_YROUND)
	    gnminor = gstati (gp, G_YNMINOR)
	    setaxispos = gstati (gp, G_YSETAXISPOS)
	    axispos1 = gstatr (gp, G_YAXISPOS1)
	    axispos2 = gstatr (gp, G_YAXISPOS2)
	    ticklabelsize = gstatr (gp, G_YTICKLABELSIZE)
	    drawticks = gstati (gp, G_YDRAWTICKS)
	    axislabelsize = gstatr (gp, G_YAXISLABELSIZE)
	    axiswidth = gstatr (gp, G_YAXISWIDTH)
	    minorwidth = gstatr (gp, G_YMINORWIDTH)
	    majorwidth = gstatr (gp, G_YMAJORWIDTH)
	    drawaxes = gstati (gp, G_YDRAWAXES)
	    labelticks = gstati (gp, G_YLABELTICKS)
	    call ggwind (gp, junk,junk, wmin,wmax)
	}

	# Start by zeroing the AX structures so that we do not have to zero
	# fields explicitly.  This is a bit tricky because we are implicitly
	# setting fields that are not named, but it saves time and space.

	call aclri (Memi[ax1], LEN_AX)
	call aclri (Memi[ax2], LEN_AX)

	# If ticks are not to be drawn or if there are fewer than 2 ticks then
	# we are done.

	if (nmajor <= 2)
	    return

	# Call the tick placement algorithm to determine where to put the major
	# tick marks.  The output of this block are the variables P1 and P2,
	# the world coords of the ends of the axis, and TICK1 and STEP, the
	# world coords of the first tick and separation in world coords between
	# major ticks.

	p1 = wmin
	p2 = wmax
	scaling = gstati (gp, G_XTRAN)

	AX_SCALING(ax1) = scaling
	AX_SCALING(ax2) = scaling

	if (scaling == LOG) {
	    p1 = log10 (p1)
	    p2 = log10 (p2)
	    logflag = YES
	} else if (scaling == ELOG) {
	    p1 = elogr (p1)
	    p2 = elogr (p2)
	    logflag = YES
	} else
	    logflag = NO

	# Call the tick placement algorithm.
	call gtickr (p1, p2, nmajor, logflag, tick1, step)
#debug+
if (debug == YES) {
call eprintf ("lbl_find:	after gtickr, nmaj %d, step %g\n")
call pargi (nmajor); call pargr (step)
}
#debug-

	# If rounding is enabled, calculate ticking parameters for extension.
	if (round == YES || scaling != LINEAR) {
	    if (!fp_equalr (p1, tick1)) {
		tick1 = tick1 - step
		p1 = tick1
	    }

	    length = (p2 - p1) / step
	    if (!fp_equalr (p1 + int(length) * step, p2))
		p2 = p1 + (int(length) + 1) * step

	    if (scaling == ELOG) {
		tp1 = aelogr (p1)
		tp2 = aelogr (p2)
	    } else if (scaling == LOG) {
		tp1 = 10.0 ** p1
		tp2 = 10.0 ** p2
	    } else {
		tp1 = p1
		tp2 = p2
	    }

	}

	# Compute the coords of the axis endpoint and of the first tick in world
	# coords.

	if (scaling == LINEAR) {
	    wctick = tick1
	    wcp1 = p1
	} else if (scaling == LOG) {
	    wctick = 10.0 ** tick1
	    wcp1 = 10.0 ** p1
	} else {
	    wctick = aelogr (tick1)
	    wcp1 = aelogr (p1)
	}

#debug+
if (debug == YES) {
call eprintf ("lbl_find: p1=%g, p2=%g, log=%d, tick1=%g, wctick=%g, step=%g\n")
call pargr(p1); call pargr(p2); call pargi(logflag); call pargr(tick1)
call pargr(wctick); call pargr(step)
}
#debug-

	# Compute the number of minor ticks.  If we are log scaling there
	# are either no minor ticks or 8 minor ticks.  If the scaling is
	# linear the tick placement algorithm is used to compute the best
	# number of minor ticks, using G_NMINOR as a close estimate.

	if (gnminor <= 0)
	    nminor = 0
	else if (logflag == YES)
	    nminor = 8
	else {
	    nminor = gnminor
	    tmax = nminor * 2
	    repeat {
#debug+
if (debug == YES) {
call eprintf ("		before gtickr, tick1 %g, nm %d, tval %g, m_s %g\n")
call pargr (tick1); call pargi (nminor); call pargr (tval); call pargr (minor_step)
}
#debug-
		call gtickr (tick1, tick1+step, nminor, NO, tval, minor_step)
		nminor = nminor + 1
	    } until (abs ((tval - tick1) / step) < TOL || nminor > tmax)
	    nminor = nint (abs (step / minor_step)) - 1
	}
#debug+
if (debug == YES) {
call eprintf ("	MINOR: final nminor %d, minor_step %g, tick1 %g, step %g\n")
call pargi (nminor); call pargr (minor_step); call pargr (tick1)
call pargr (step)
}
#debug-

	AX_NMINOR(ax1) = nminor
	AX_NMINOR(ax2) = nminor

	# Compute the step size in world coords between minor ticks and the
	# number of minor ticks to be drawn initially until the first major
	# tick (tick1) is reached.  Note that for ELOG scaling the minor
	# step size and number of minor ticks are different in the range
	# +-10 (which is linear) than elsewhere, but we ignore that here.

	if (scaling == LINEAR) {
	    minor_step = step / (nminor + 1)
	    AX_INLEFT(ax1) = int ((wctick - wcp1) / minor_step)
	} else {
	    t1 = nint (tick1)
	    t2 = nint (tick1 + step)
	    if (scaling == LOG)
		minor_step = (10.0 ** t2 - 10.0 ** t1) / 9.
	    else
		minor_step = (aelogr(real(t2)) - aelogr(real(t1))) / 9.
	    if (nminor == 0)
		minor_step = minor_step * 9.
	    AX_INLEFT(ax1) = 0
	}

	AX_INLEFT(ax2) = AX_INLEFT(ax1)

	# Set KSTEP, the adjustment to the step size at each major tick.  This
	# is always 1.0 if the scale is linear.  Set KSTEP to negative if ELOG
	# scaling, to tell the drawing code to invert kstep (.1->10 or 10->.1)
	# when passing through the origin (necessary for ELOG scaling).  The
	# sign is not otherwise significant.  If heading toward the origin
	# initially then KSTEP is inverted for ELOG scaling vs LOG scaling.

	if (scaling == LINEAR) {
	    AX_IKSTEP(ax1) = 1.0
	} else if (scaling == ELOG) {
	    tval = p1
	    if (abs (tval + step) > abs(t1))
		AX_IKSTEP(ax1) = -10.0
	    else
		AX_IKSTEP(ax1) = -0.1
	} else
	    AX_IKSTEP(ax1) = 10.0 ** step
	AX_IKSTEP(ax2) = AX_IKSTEP(ax1)

	# Set those parameters which differ depending on whether the axis is
	# horizontal or vertical.

#debug+
if (debug == YES) {
call eprintf ("____wctick=%g, inleft=%d, minorstep=%g\n")
call pargr(wctick);call pargi(AX_INLEFT(ax1)); call pargr(minor_step)
}
#debug-

	if (coord == XCOORD) {
	    AX_TICK1(ax1,1) = wctick - (AX_INLEFT(ax1) * minor_step)
	    AX_TICK1(ax2,1) = wctick - (AX_INLEFT(ax2) * minor_step)

	    if (setaxispos == YES) {
		AX_TICK1(ax1,2) = axispos1
		AX_TICK1(ax2,2) = axispos2
	    } else {
		AX_TICK1(ax1,2) = wmin
		AX_TICK1(ax2,2) = wmax
	    }

	    AX_ISTEP(ax1,1) = minor_step
	    AX_ISTEP(ax2,1) = minor_step

	    char_height = ggetr (gp, "ch")
	    if (char_height < EPSILON)
		char_height = DEF_CHARHEIGHT
	    char_height = char_height * ticklabelsize

	    AX_TICKLABELOFFSET(ax2,2) = 0.5 * char_height
	    AX_TICKLABELOFFSET(ax1,2) = -AX_TICKLABELOFFSET(ax2,2)

	    # Set gtext format for tick labels.
	    call strcpy ("hj=c,vj=t", AX_TICKLABELPOS(ax1), SZ_FORMAT)
	    call strcpy ("hj=c,vj=b", AX_TICKLABELPOS(ax2), SZ_FORMAT)

	} else {
	    if (setaxispos == YES) {
		AX_TICK1(ax1,1) = axispos1
		AX_TICK1(ax2,1) = axispos2
	    } else {
		AX_TICK1(ax1,1) = wmin
		AX_TICK1(ax2,1) = wmax
	    }

	    AX_TICK1(ax1,2) = wctick - (AX_INLEFT(ax1) * minor_step)
	    AX_TICK1(ax2,2) = wctick - (AX_INLEFT(ax2) * minor_step)

	    AX_ISTEP(ax1,2) = minor_step
	    AX_ISTEP(ax2,2) = minor_step

	    char_width = ggetr (gp, "cw")
	    if (char_width < EPSILON)
		char_width = DEF_CHARWIDTH
	    char_width = char_width * ticklabelsize

	    AX_TICKLABELOFFSET(ax2,1) = 0.5 * char_width
	    AX_TICKLABELOFFSET(ax1,1) = -AX_TICKLABELOFFSET(ax2,1)

	    call strcpy ("hj=r,vj=c", AX_TICKLABELPOS(ax1), SZ_FORMAT)
	    call strcpy ("hj=l,vj=c", AX_TICKLABELPOS(ax2), SZ_FORMAT)
	}

	# Set the tick parameters that are identical for the two axes and
	# which do not depend on whether the axis is horizontal or vertical.

	AX_DRAWTICKS(ax1)	= drawticks
	AX_DRAWTICKS(ax2)	= drawticks
	AX_TICKLABELSIZE(ax1)	= ticklabelsize
	AX_TICKLABELSIZE(ax2)	= ticklabelsize
	AX_AXISLABELSIZE(ax1)	= axislabelsize
	AX_AXISLABELSIZE(ax2)	= axislabelsize
	AX_AXISWIDTH(ax1)	= axiswidth
	AX_AXISWIDTH(ax2)	= axiswidth
	AX_MINORWIDTH(ax1)	= minorwidth
	AX_MINORWIDTH(ax2)	= minorwidth
	AX_MAJORWIDTH(ax1)	= majorwidth
	AX_MAJORWIDTH(ax2)	= majorwidth

#debug+
if (debug == YES) {
call eprintf ("		ticklabelsize = %7.4f, axislabelsize = %7.4f\n")
call pargr (ticklabelsize); call pargr (axislabelsize)
call eprintf ("		nmajor %d, axispos1 %7.4f, axispos2 %7.4f\n")
call pargi (nmajor); call pargr (axispos1); call pargr (axispos2)
call eprintf ("		axiswidth %7.4f, labelticks %d\n")
call pargr (axiswidth); call pargi (labelticks)
}
#debug-
	# Compute the number of digits of precision needed for the tick labels.
	AX_NDIGITS(ax1) = max (1, gt_ndigits (p1, p2, step))
	AX_NDIGITS(ax2) = AX_NDIGITS(ax1)

	# If both axes are to be drawn label ticks if enabled.  If only one
	# axis is to be drawn that is the axis that must be labelled.

	if (drawaxes > 0) {
	    AX_LABELTICKS(ax1) = labelticks
	    AX_LABELTICKS(ax2) = labelticks
	}
	if (drawaxes == 1 || drawaxes == 3)
	    AX_LABELTICKS(ax2) = NO
	else if (drawaxes == 2)
	    AX_LABELTICKS(ax1) = NO

	# Tick label format.
	call sprintf (AX_TICKFORMAT(ax1), SZ_FORMAT, "%%0.%dg")
	    call pargi (AX_NDIGITS(ax1) + 1)
	call strcpy (AX_TICKFORMAT(ax1), AX_TICKFORMAT(ax2), SZ_FORMAT)
end
