include	<mach.h>
include	<gset.h>
include	<gio.h>
include	"lblglabax.h"

# LBL_GLABAX -- Draw and label the axes of the plot; this is very similar to
# GIO's GLABAX (which see), but it allows more control of label placement, 
# it knows about 90-degree rotation of the whole plot, and it tries to 
# optimize pen movement for such plotters.  Furthermore, it does not use those
# GSET parameters used by GIO's GLABAX that are not controlled by LONGPLOT.

procedure lbl_glabax (gp, title, xlabel, ylabel)

pointer	gp			# graphics descriptor
char	title[ARB]		# plot title (may be more than one line)
char	xlabel[ARB]		# X axis label
char	ylabel[ARB]		# Y axis label

char	label[SZ_LABEL]
int	axis, wcs, ntitlelines, ip, major_tick, nchars
int	save_pltype, save_clip, save_txfont
real	save_plwidth, save_txsize
real	dx, dy, x, y, sx, sy, scalar, wc, wstep
pointer	sp, axes[4], ax

real	gstatr()
int	gstati(), lbl_gettick(), lbl_encode(), lbl_loglab()
errchk	lbl_setup, gadraw, grdraw, gamove, gtext
errchk	lbl_label_axis, lbl_plot_title, lbl_gettick

#debug+
int	debug
common /deb/ debug
#debug-
begin
	call smark (sp)
	call salloc (axes[1], LEN_AX, TY_STRUCT)
	call salloc (axes[2], LEN_AX, TY_STRUCT)
	call salloc (axes[3], LEN_AX, TY_STRUCT)
	call salloc (axes[4], LEN_AX, TY_STRUCT)

	wcs = gstati (gp, G_WCS)

	# Count the number of lines in the title block.
	ntitlelines = 0
	if (title[1] != EOS) {
	    for (ip=1;  title[ip] != EOS;  ip=ip+1)
		if (title[ip] == '\n' && title[ip+1] != EOS)
		    ntitlelines = ntitlelines + 1
	    ntitlelines = ntitlelines + 1
	}
	ntitlelines = max (ntitlelines, gstati (gp, G_NTITLELINES))

	# Fix the coordinate systems and set the axis drawing parameters.
	# The number of lines in the title block is needed to determine how
	# much space to allow at the top of the screen.

	call lbl_setup (gp, axes, ntitlelines, xlabel, ylabel)

	# Save the values of any user parameters we must change while drawing
	# the axes.

	save_pltype  = gstati (gp, G_PLTYPE)
	save_txfont  = gstati (gp, G_TXFONT)
	save_plwidth = gstatr (gp, G_PLWIDTH)
	save_txsize  = gstatr (gp, G_TXSIZE)
	save_clip    = gstati (gp, G_CLIP)

	# Draw and label the four axes.  First set the linetype to be used
	# to draw the axes and ticks; this may be different than
	# that used to plot the data.  Draws are preferred to moves to minimize
	# the number of polylines needed to draw the axis.  An axis is drawn
	# by moving to the start of the axis, drawing each tick in sequence,
	# and then moving to the end of the axis.  Tick labels are drawn at
	# the major ticks if required.  The axes and ticks must be drawn in
	# world coords to get the proper scaling.  Clipping is turned off while
	# drawing the axes to avoid clipping portions of the axes due to small
	# floating point errors.

	call gseti (gp, G_PLTYPE, 1)
	call gseti (gp, G_CLIP, NO)

	do axis = 1, 4 {
	    ax = axes[axis]
	    if (AX_DRAWME(ax) == NO)
		next

#debug+
if (debug == YES) {
call eprintf ("\nlbl_glabax:	axis %d tick1=(%g,%g) istep=(%g,%g) kstep=%g\n")
call pargi (axis)
call pargr (AX_TICK1(ax,1)); call pargr (AX_TICK1(ax,2))
call pargr (AX_ISTEP(ax,1)); call pargr (AX_ISTEP(ax,2))
call pargr (AX_IKSTEP(ax))
call eprintf ("\tstart=(%g,%g) end=(%g,%g)\n")
call pargr (AX_START(ax,1)); call pargr (AX_START(ax,2))
call pargr (AX_END(ax,1)); call pargr (AX_END(ax,2))
call eprintf ("nminor=%d, inleft=%d, minortick=(%g,%g), majortick=(%g,%g)\n")
call pargi (AX_NMINOR(ax)); call pargi (AX_INLEFT(ax))
call pargr (AX_MINORTICK(ax,1)); call pargr (AX_MINORTICK(ax,2))
call pargr (AX_MAJORTICK(ax,1)); call pargr (AX_MAJORTICK(ax,2))
call eprintf ("--> AX_STEP(ax,1) %g, AX_STEP(ax,2) %g\n")
call pargr (AX_STEP(ax,1)); call pargr (AX_STEP(ax,2))
}
#debug-

	    # Set the axis linewidth and move to the start of the axis.
	    call gsetr (gp, G_PLWIDTH, AX_AXISWIDTH(ax))
	    call gamove (gp, AX_START(ax,1), AX_START(ax,2))

	    # Draw the axis and label the major ticks if so indicated.
	    # First set flag to initialize lbl_gettick.

#debug+
if (debug == YES) {
call eprintf ("	before lbl_gettick loop, AX_DRAWTICKS(ax) = %d\n")
call pargi (AX_DRAWTICKS(ax))
}
#debug-
	if (AX_DRAWTICKS(ax) == NO)
	    AX_NLEFT(ax) = -1
	    while (lbl_gettick (gp, ax, x, y, major_tick) != EOF) {

		# Advance to the next tick.
		call gsetr (gp, G_PLWIDTH, AX_AXISWIDTH(ax))
		call gadraw (gp, x, y)

#debug+
if (debug == YES && axis == 2) {
call eprintf ("	in lbl_gettick loop, major_tick %d, x %9.4f, y %9.4f\n")
call pargi (major_tick); call pargr (x); call pargr (y)
}
#debug-
		if (major_tick == YES) {
		    # Draw a major tick.

		    call gsetr (gp, G_PLWIDTH, AX_MAJORWIDTH(ax))
		    dx = AX_MAJORTICK(ax,1)
		    dy = AX_MAJORTICK(ax,2)
		    call grdraw (gp, dx, dy)
		    call grdraw (gp, -dx, -dy)
#debug+
if (debug == YES && axis == 2) {
call eprintf ("	>>>> majtk YES, dx %9.4f, dy %9.4f, majwid %9.4f, ltk %d\n")
call pargr (dx); call pargr (dy); call pargr (AX_MAJORWIDTH(ax))
call pargi (AX_LABELTICKS(ax))
}
#debug-

		    if (AX_LABELTICKS(ax) == YES) {
			# Get the tick label position in NDC coords.  World
			# coords cannot be used for an offset outside the
			# viewport as the coords might be indefinite if log
			# scaling.

			call gseti (gp, G_WCS, 0)
			call gcurpos (gp, sx, sy)
			dx = AX_TICKLABELOFFSET(ax,1)
			dy = AX_TICKLABELOFFSET(ax,2)

			# Format the numeric tick label string.  The scalar
			# multiplier is used to compute the step size between
			# major ticks.

			scalar = AX_NMINOR(ax) + 1.0
			if (AX_HORIZONTAL(ax) == YES) {
			    wc = x
			    wstep = AX_STEP(ax,1) * scalar
			} else {
			    wc = y
			    wstep = AX_STEP(ax,2) * scalar
			}

			# Draw the label string.
			call gsetr (gp, G_TXSIZE, AX_TICKLABELSIZE(ax))

			# If log scaling, label the ticks in log units.
			if (AX_SCALING(ax) == LINEAR) {
			    nchars = lbl_encode (wc, label, SZ_LABEL,
				AX_TICKFORMAT(ax), wstep)
			    call gtext (gp, sx + dx, sy + dy, label,
				AX_TICKLABELPOS(ax))
			} else {
			    nchars = lbl_loglab (gp, sx+dx, sy+dy, wc,
				AX_TICKLABELPOS(ax), AX_SCALING(ax))
			}

			# Store the maximum number of characters in tick label.
			AX_TICKLABELCHARS(ax) = max (AX_TICKLABELCHARS(ax),
			    nchars)

			# Leave the pen back at the base of the tick.
			call gamove (gp, sx, sy)
			call gseti (gp, G_WCS, wcs)
		    } 

		} else {
		    # Draw a minor tick.

		    dx = AX_MINORTICK(ax,1)
		    dy = AX_MINORTICK(ax,2)

		    call gsetr (gp, G_PLWIDTH, AX_MINORWIDTH(ax))
		    call grdraw (gp, dx, dy)
		    call grdraw (gp, -dx, -dy)
		}
	    }

	    # Draw line segment from last tick to the end of the axis.
	    call gadraw (gp, AX_END(ax,1), AX_END(ax,2))

	    # Flush the graphics output.  When working interactively, this
	    # gives the user something to watch while we generate the rest
	    # of the plot.

	    if (AX_NMINOR(ax) > 0)
		call gflush (gp)
	}

	# Draw grid between major ticks.
	if (gstati (gp, G_XDRAWGRID) == YES)
	    call lbl_drawgrid (gp, axes[3], axes[2])
	if (gstati (gp, G_YDRAWGRID) == YES)
	    call lbl_drawgrid (gp, axes[1], axes[4])

	# Label the X and Y axes.
	do axis = 1, 4 {
	    ax = axes[axis]
	    if (AX_DRAWME(ax) == YES && AX_LABELTICKS(ax) == YES)
		call lbl_label_axis (gp, ax, xlabel, ylabel)
	}

	# Draw plot title block.
	call lbl_plot_title (gp, title, ntitlelines)

	# Restore the parameters we were originally called with.
	call gseti (gp, G_WCS, wcs)
	call gseti (gp, G_PLTYPE,  save_pltype)
	call gsetr (gp, G_PLWIDTH, save_plwidth)
	call gsetr (gp, G_TXSIZE,  save_txsize)
	call gseti (gp, G_TXFONT,  save_txfont)
	call gseti (gp, G_CLIP,    save_clip)

	call gflush (gp)
	call sfree (sp)
end
