include	<mach.h>
include	<gio.h>
include <gset.h>
include	"lblglabax.h"

# LBL_GETTICK -- Get the position and type of the next tick on an axis.
# Taken almost entirely from gio$glabax/glbgtick.x with only minor 
# modifications.  See the extensive comments at the front of that file.

int procedure lbl_gettick (gp, ax, x, y, major_tick)

pointer	gp			# graphics descriptor
pointer	ax			# axis descriptor
real	x, y			# coordinates of next tick (output)
int	major_tick		# YES if next tick is a major tick

int	i, axis, wcs, scaling, nminor, expon
real	kstep, step, astep, ten, sx, sy, tolerance, pos, norm_pos
real	vxmax, vymax, junk

int	gstati()
bool	lbl_eq()
define	logscale_ 91

#debug+
int	debug
common	/deb/ debug
#debug-
begin
	if (AX_DRAWTICKS(ax) == NO)
	    return (EOF)

	tolerance = TOL
	scaling = AX_SCALING(ax)
	nminor  = AX_NMINOR(ax)
	kstep   = AX_KSTEP(ax)

	if (AX_HORIZONTAL(ax) == YES)
	    axis = 1
	else
	    axis = 2

	# Count down a minor tick.  If nleft is negative then we are being
	# called for the first time for this axis.

	if (AX_NLEFT(ax) < 0) {

	    # Initialize everything and return coords of the first tick.
	    AX_KSTEP(ax) = AX_IKSTEP(ax)
	    AX_NLEFT(ax) = AX_INLEFT(ax)
	    do i = 1, 2 {
		AX_POS(ax,i)  = AX_TICK1(ax,i)
		AX_STEP(ax,i) = AX_ISTEP(ax,i)
	    }

	    step  = AX_STEP(ax,axis)
	    astep = abs (step)

	    if (AX_NLEFT(ax) == 0) {
		# Note that there may not be any minor ticks.
		major_tick = YES
		AX_NLEFT(ax) = nminor
		if (nminor > 0)
		    if (scaling == ELOG && (astep >= .99 && astep < 2.0)) {
			# Elog scaling in linear region.
			AX_NLEFT(ax) = 4
			if (step < 0)
			    step = -2.0
			else
			    step =  2.0
			AX_STEP(ax,axis) = step
		    }
	    } else {
		AX_NLEFT(ax) = AX_NLEFT(ax) - 1
		major_tick = NO
	    }

	    # Elog scaling in linear region.  KSTEP must be inverted as we
	    # pass through the origin.  This normally occurs upon entry to the
	    # linear region, but if we start out at +/- 10 we must set KSTEP
	    # to its linear value during setup.

	    if (scaling == ELOG && lbl_eq(step,2.0))
		AX_KSTEP(ax) = -10.0

	} else {
	    # All ticks after the first tick.
	    do i = 1, 2
		AX_POS(ax,i) = AX_POS(ax,i) + AX_STEP(ax,i)
	    AX_NLEFT(ax) = AX_NLEFT(ax) - 1

	    # If we are log scaling the ticks will never have more than 2
	    # digits of precision.  Try to correct for the accumulation of
	    # error by rounding.  When log scaling the error increases by
	    # a factor of ten in each decade and can get quite large if
	    # the log scale covers a large range.

	    if (scaling != LINEAR) {
		pos = AX_POS(ax,axis)
		call fp_normr (pos, norm_pos, expon)
		pos = nint (norm_pos * 10.0) / 10.0
		pos = pos * (10.0 ** expon)
		AX_POS(ax,axis) = pos
	    }

	    if (AX_NLEFT(ax) < 0) {
		# Next tick is a major tick.  If log scaling we must reset
		# the tick parameters for the next decade.

		major_tick = YES
		AX_NLEFT(ax) = nminor

		# The following handles the special case of ELOG scaling in
		# the linear regime when the number of minor ticks is zero.
		# The step size in such a case is 9 to some power in the log
		# region and +/- 10 in the linear region.

		if (scaling == ELOG && nminor == 0) {
		    pos = AX_POS(ax,axis)
		    if (step < 0)
			ten = -10.
		    else
			ten = 10.

		    if (lbl_eq (pos, 10.0)) {
			if (lbl_eq (step, 10.0)) {
			    if (step < 0)
				AX_STEP(ax,axis) = -9.
			    else
				AX_STEP(ax,axis) = 9.
			    goto logscale_
			} else
			    step = ten
		    } else if (lbl_eq (pos, 0.0)) {
			step = ten
			if (pos / step < 0)
			    AX_KSTEP(ax) = -0.1
			else
			    AX_KSTEP(ax) = -10.0
		    } else
			goto logscale_
		    AX_STEP(ax,axis) = step

		} else if (scaling != LINEAR) {
		    # Adjust the tick step by the kstep factor, provided we
		    # are not at the origin in ELOG scaling (the step is 1
		    # on either side of the origin for ELOG scaling).  Reset
		    # the step size to 1.0 if ELOG scaling and just coming out
		    # of the linear regime.
logscale_
		    step = AX_STEP(ax,axis)
		    if (scaling != ELOG || abs(AX_POS(ax,axis)) > 0.1) {
			if (scaling == ELOG && lbl_eq (step, 2.0))
			    AX_STEP(ax,axis) = step / 2.0

			do i = 1, 2
			    AX_STEP(ax,i) = AX_STEP(ax,i) * abs (AX_KSTEP(ax))
		    }

		    # Adjust the step size to 2.0 if ELOG scaling and in the
		    # linear regime (initial step size of 1).

		    step = AX_STEP(ax,axis)
		    if (scaling == ELOG && lbl_eq(step,1.0)) {
			if (step < 0)
			    step = -2.0
			else
			    step =  2.0
			AX_STEP(ax,axis) = step
		    }

		    # If elog scaling and we have just entered the linear
		    # regime, adjust the number of ticks and the KSTEP factor.

		    if (scaling == ELOG && lbl_eq(step,2.0)) {
			# Elog scaling in linear region.  KSTEP must be
			# inverted as we pass through the origin.

			if (abs(AX_POS(ax,axis)) > 0.1)
			    AX_KSTEP(ax) = -10.0

			if (nminor > 0)
			    AX_NLEFT(ax) = 4
		    }
		}
	    } else
		major_tick = NO
	}

	x = AX_POS(ax,1)
	y = AX_POS(ax,2)

	# Return EOF if tick falls beyond end of axis.  The comparison is made
	# in NDC coords to avoid having to check if the WCS is increasing or
	# decreasing and to avoid the problems of comparing unnormalized
	# floating point numbers.

	wcs = gstati (gp, G_WCS)
	call ggview (gp, junk,vxmax, junk,vymax)

	call gctran (gp, x,y, sx,sy, wcs, 0)
	if (sx - vxmax > tolerance || sy - vymax > tolerance) {
#debug+
if (debug == YES) {
call eprintf (
"\nin lblgtick, to rtn EOF, sx%9.4f, sy%9.4f, vx%9.4f, vy%9.4f, tol %13.7f\n")
call pargr (sx); call pargr (sy); call pargr (vxmax); call pargr (vymax)
call pargr (tolerance)
}
#debug-
	    return (EOF)
	} else
	    return (OK)
end


# LBL_EQ -- Compare two (near normalized) floating point numbers for
# equality, using the absolute value of the first argument.

bool procedure lbl_eq (a, b)

real	a			# compare absolute value of this number
real	b			# to this positive number

begin
	return (abs (abs(a) - b) < 0.1)
end
