include	<mach.h>
include	<gio.h>
include <gset.h>
include "lpl.h"
include	"lblglabax.h"

# LBL_SET_AXES -- Set all axis descriptor parameters not pertaining to the
# ticks.  The WCS has already been fixed by the time we get here.
# Taken almost entirely from gio$glabax/glbsetax.x with exceptions as noted.

procedure lbl_set_axes (gp, coord, ax1, ax2)

pointer	gp			# graphics descriptor
int	coord			# x or y coordinate in this pass
pointer	ax1, ax2		# axis descriptors (output)

int	axis
int	setaxispos, drawaxes
real	majorlength, minorlength, axispos1, axispos2
real	p1, p2, x1, x2, y1, y2, wxmin,wxmax, wymin,wymax

int	gstati()
real	lbl_ticklen(), gstatr()

#debug+
int	debug
common /deb/ debug
#debug-

begin

	# Rather than bring LPL, LPX datastructures down here or pass a lot
	# of extra arguments, we query GIO for current settings in order to
	# avoid using GIO datastructures directly.

	call ggwind (gp, wxmin,wxmax, wymin,wymax)

	if (coord == XCOORD) {
	    setaxispos = gstati (gp, G_XSETAXISPOS)
	    majorlength = gstatr (gp, G_XMAJORLENGTH)
	    minorlength = gstatr (gp, G_XMINORLENGTH)
	    drawaxes = gstati (gp, G_XDRAWAXES)
	    axispos1 = gstatr (gp, G_XAXISPOS1)
	    axispos2 = gstatr (gp, G_XAXISPOS2)
	} else {
	    setaxispos = gstati (gp, G_YSETAXISPOS)
	    majorlength = gstatr (gp, G_YMAJORLENGTH)
	    minorlength = gstatr (gp, G_YMINORLENGTH)
	    drawaxes = gstati (gp, G_YDRAWAXES)
	    axispos1 = gstatr (gp, G_YAXISPOS1)
	    axispos2 = gstatr (gp, G_YAXISPOS2)
	}
#debug+
if (debug == YES) {
call eprintf ("_____should be NDC:  MAJORLENGTH %g, MINORLENGTH %g\n")
call pargr (majorlength); call pargr (minorlength)
}
#debug-

	# If the window was rounded in Y in the second call to find_ticks,
	# then the Y positions of the first ticks set in the first call will
	# be in error and must be corrected.  If the user has elected to set
	# the axis position explicitly, however, then we must leave it alone.

	if (coord == XCOORD && setaxispos == NO) {
	    AX_TICK1(ax1,2) = wymin
	    AX_TICK1(ax2,2) = wymax
	}

	# Set the tick lengths.  This is done here rather than in findticks due
	# to rounding, as noted above.  The tick offsets in world coordinates.
	# The m[aj|in]orlength values are given in NDC coordinates.

	if (coord == XCOORD) {
	    axis = 2
	    AX_HORIZONTAL(ax1) = YES
	    AX_HORIZONTAL(ax2) = YES
	} else {
	    axis = 1
	    AX_HORIZONTAL(ax1) = NO
	    AX_HORIZONTAL(ax2) = NO
	}

	AX_MAJORTICK(ax1,axis) = lbl_ticklen (gp, ax1,  majorlength)
	AX_MINORTICK(ax1,axis) = lbl_ticklen (gp, ax1,  minorlength)
	AX_MAJORTICK(ax2,axis) = lbl_ticklen (gp, ax2, -majorlength)
	AX_MINORTICK(ax2,axis) = lbl_ticklen (gp, ax2, -minorlength)

	# Initialize maximum-chars-used-in-tick-label for later accumulation.
	AX_TICKLABELCHARS(ax1) = 0
	AX_TICKLABELCHARS(ax2) = 0

	# Select none, either, or both axes to be drawn.  If only the second
	# axis is drawn then that is the side we must draw the tick and axis
	# labels on.

	switch (drawaxes) {
	case 0:
	    AX_DRAWME(ax1) = NO
	    AX_DRAWME(ax2) = NO
	    return
	case 1:
	    AX_DRAWME(ax1) = YES
	    AX_DRAWME(ax2) = NO
	case 2:
	    AX_DRAWME(ax1) = NO
	    AX_DRAWME(ax2) = YES
	default:
	    AX_DRAWME(ax1) = YES
	    AX_DRAWME(ax2) = YES
	}

	# Determine the endpoints of the axis.  These default to the corners of
	# the viewport (in world coordinates), but the positions may be
	# overriden by the user if desired.

	# First get the positions of the two axes.
	if (setaxispos == YES) {
	    p1 = axispos1
	    p2 = axispos2
	} else if (coord == XCOORD) {
	    p1 = wymin
	    p2 = wymax
	} else {
	    p1 = wxmin
	    p2 = wxmax
	}

	# Convert these positions into the world coordinates of the endpoints.
	if (coord == XCOORD) {
	    x1 = wxmin
	    x2 = wxmax
	    y1 = p1
	    y2 = p2
	} else {
	    x1 = p1
	    x2 = p2
	    y1 = wymin
	    y2 = wymax
	}

	if (coord == XCOORD) {
	    # Set the left and right endpoints of the axes.

	    AX_START(ax1,1) = x1
	    AX_START(ax1,2) = y1
	    AX_END(ax1,1)   = x2
	    AX_END(ax1,2)   = y1

	    AX_START(ax2,1) = x1
	    AX_START(ax2,2) = y2
	    AX_END(ax2,1)   = x2
	    AX_END(ax2,2)   = y2

	} else {
	    # Set the lower and upper endpoints of the axes.

	    AX_START(ax1,1) = x1
	    AX_START(ax1,2) = y1
	    AX_END(ax1,1)   = x1
	    AX_END(ax1,2)   = y2

	    AX_START(ax2,1) = x2
	    AX_START(ax2,2) = y1
	    AX_END(ax2,1)   = x2
	    AX_END(ax2,2)   = y2
	}
end
