include <gio.h>
include "lpl.h"

# LPL_APPEARANCE -- Get appearance-related parameters.
# Only those appearance parameters not embedded in input indirection are
# handled here; the others, specific to each individual plot, are acquired
# at lpl_indirection() time.

procedure lpl_appearance (lpl)

pointer	lpl		# Longplot descriptor

pointer sp, stringbuf
int	ip

int	clgeti(), btoi(), strncmp(), ctoi(), ctor(), lpl_cnv_marker()
real	clgetr()
bool	clgetb()

begin
	call smark (sp)
	call salloc (stringbuf, SZ_LINE, TY_CHAR)

	call malloc (LPL_TITLE(lpl), SZ_LINE*4, TY_CHAR)
	call malloc (LPL_XLABEL(lpl), SZ_LINE, TY_CHAR)
	call malloc (LPL_YLABEL(lpl), SZ_LINE, TY_CHAR)

	# Axis labels
	call clgstr ("xlabel", XLABEL(LPL_XLABEL(lpl)), SZ_LINE)
	call clgstr ("ylabel", YLABEL(LPL_YLABEL(lpl)), SZ_LINE)

	# Titles; ## for now, titlepos turned off (no legend yet)
#	call clgstr ("titlepos", Memc[stringbuf], SZ_LINE)
#	if (strncmp (Memc[stringbuf], "normal", 1) == 0 ||
#	    strncmp (Memc[stringbuf], "above", 1) == 0)
#	    LPL_TITLEPOS(lpl) = POS_ABOVE
#	else if (strncmp (Memc[stringbuf], "below", 1) == 0)
#	    LPL_TITLEPOS(lpl) = POS_BELOW
#	else if (strncmp (Memc[stringbuf], "legend", 1) == 0)
#	    LPL_TITLEPOS(lpl) = POS_LEGEND
#	else {
#	    call eprintf ("Warning (LPL_APPEAR):  bad titlepos = %s\n")
#		call pargstr (Memc[stringbuf])
#	    call eprintf ("    titlepos reset to 'above'\n")
#	    LPL_TITLEPOS(lpl) = POS_ABOVE
#	}
#
	call clgstr ("title", TITLE(LPL_TITLE(lpl), 0), SZ_LINE*4)

	# Plot positioning; fill out words for standalone (non-cl) execution.
	call clgstr ("multiple", Memc[stringbuf], SZ_LINE)
	ip = 1
	if (ctor (Memc[stringbuf], ip, LPL_YDELTA(lpl)) <= 0)
	    if (strncmp (Memc[stringbuf], "normal", 1) == 0 ||
	        strncmp (Memc[stringbuf], "series", 2) == 0)
		LPL_MULTIPLE(lpl) = MULT_SERIES
	    else if (strncmp (Memc[stringbuf], "overtrace", 1) == 0) {
		LPL_MULTIPLE(lpl) = MULT_OVERTRACE
		LPL_YDELTA(lpl) = 0.0
	    } else if (strncmp (Memc[stringbuf], "stacked", 2) == 0) {
		LPL_MULTIPLE(lpl) = MULT_STACK
		LPL_YDELTA(lpl) = INDEFR
	    } else {
		LPL_MULTIPLE(lpl) = MULT_SERIES
		call eprintf ("Warning (LPL_APPEAR):  bad 'multiple' = %s\n")
		    call pargstr (Memc[stringbuf])
		call eprintf ("    Multiple plot placement set to 'series'\n")
	    }
	else
	    LPL_MULTIPLE(lpl) = MULT_STACK_DY

	# What gets drawn -- normal=line; marker; both
	call clgstr ("plotmode", Memc[stringbuf], MAXWORDSIZE)
	if (strncmp (Memc[stringbuf], "normal", 1) == 0 ||
	    strncmp (Memc[stringbuf], "line", 1) == 0)
	    LPL_PLOTMODE(lpl) = PLOT_LINE
	else if (strncmp (Memc[stringbuf], "marker", 1) == 0)
	    LPL_PLOTMODE(lpl) = PLOT_MARK
	else if (strncmp (Memc[stringbuf], "both", 1) == 0)
	    LPL_PLOTMODE(lpl) = PLOT_BOTH
	else {
	    call eprintf ("Warning (LPL_APPEAR):  bad 'plotmode' = %s\n")
		call pargstr (Memc[stringbuf])
	    call eprintf ("    Plotmode reset to 'line'\n")
	    LPL_PLOTMODE(lpl) = PLOT_LINE
	}

	# Marker type and size; these may also be specified in indirection file.
	if (LPL_PLOTMODE(lpl) != PLOT_LINE) {
	    call clgstr ("mtype", Memc[stringbuf], SZ_LINE)
	    LPL_MARKERTYPE(lpl) = lpl_cnv_marker (Memc[stringbuf])
	    LPL_MARKERSIZE(lpl) = clgetr ("msize")
	}

	# X-axis tick marks or spacing
	LPL_TICKLABEL(lpl) = btoi (clgetb ("labelticks"))
	call clgstr ("majrx", Memc[stringbuf], MAXWORDSIZE)
	ip = 1
	if (strncmp (Memc[stringbuf], "scale", 1) == 0)
	    if (IS_INDEF(LPL_XSCALE(lpl))) {
		call eprintf (
		    "Warning (LPL_APPEAR):  No XSCALE to set MAJRX = scale\n")
		call eprintf ("	Resetting MAJRX to %d\n")
		    call pargi (DEF_MAJRX)
		LPL_MAJRX(lpl) = DEF_MAJRX
	    } else
		LPL_MAJRX(lpl) = -1
	else
	    if (ctoi (Memc[stringbuf], ip, LPL_MAJRX(lpl)) <= 0) {
		LPL_MAJRX(lpl) = DEF_MAJRX
		call eprintf ("Warning (LPL_APPEAR): bad MAJRX %s reset-> %d\n")
		    call pargstr (Memc[stringbuf])
		    call pargi (DEF_MAJRX)
	    }
	LPL_MINRX(lpl) = clgeti ("minrx")

	# Y-axis tick marks and spacing
	call clgstr ("majry", Memc[stringbuf], MAXWORDSIZE)
	ip = 1
	if (strncmp (Memc[stringbuf], "scale", 1) == 0)
	    if (IS_INDEF(LPL_YSCALE(lpl))) {
		call eprintf (
		    "Warning (LPL_APPEAR):  No YSCALE to set MAJRY = scale\n")
		call eprintf ("	Resetting MAJRY to %d\n")
		    call pargi (DEF_MAJRY)
		LPL_MAJRY(lpl) = DEF_MAJRY
	    } else
		LPL_MAJRY(lpl) = -1
	else
	    if (ctoi (Memc[stringbuf], ip, LPL_MAJRY(lpl)) <= 0) {
		LPL_MAJRY(lpl) = DEF_MAJRY
		call eprintf ("Warning (LPL_APPEAR): bad MAJRY %s reset-> %d\n")
		    call pargstr (Memc[stringbuf])
		    call pargi (DEF_MAJRY)
	    }
	LPL_MINRY(lpl) = clgeti ("minry")
	
	# Log[/elog] post-`transform' of axes; this has absolutely nothing to
	# do with the `xt'-`yt' real coordinate transformations.  It is mainly
	# intended for evening out data plots with wide ranges and skewed
	# distributions.  If a coordinate transform (`xt') was applied AND this
	# is specified, this one takes place after the coord transform.
	LPL_XLOGLIN(lpl) = LINEAR
	if (btoi (clgetb ("logx")) == YES)
	    if (IS_INDEF(LPL_XSCALE(lpl)))
		LPL_XLOGLIN(lpl) = LOG
	    else {
		call eprintf (
		    "Warning (LPL_APPEAR): Cannot take `logx' with `xscale'\n")
		call eprintf ("	Resetting `logx' to `no'.\n")
		call eprintf (
		    "	Use `px' instead of `xscale' in order to use `logx'\n")
	    }

	LPL_YLOGLIN(lpl) = LINEAR
	if (btoi (clgetb ("logy")) == YES)
	    if (IS_INDEF(LPL_YSCALE(lpl)))
		LPL_YLOGLIN(lpl) = LOG
	    else {
		call eprintf (
		    "Warning (LPL_APPEAR): Cannot take `logy' with `yscale'\n")
		call eprintf ("	Resetting `logy' to `no'.\n")
		call eprintf (
		    "	Use `py' instead of `yscale' in order to use `logy'\n")
	    }

	# Bold border, axes, ticks, labels, and titles?
	LPL_LBOLD(lpl) = btoi (clgetb ("lbold"))

	# Bold data (lines, markers)?
	LPL_DBOLD(lpl) = btoi (clgetb ("dbold"))

	# Float character sizes with plot dimensions in y?
	LPL_FLOATCHARSIZE(lpl) = btoi (clgetb ("floatchar"))

	# Draw grid lines at major tick marks?
	LPL_GRID(lpl) = btoi (clgetb ("grid"))

	# Rotation
	LPL_ROTATE(lpl) = btoi (clgetb ("rotate"))

	# Legend; for now, no legend
#	LPL_LEGEND(lpl)	   = NO			#btoi (clgetb ("legend"))

	# Scale bar; for now, no scale bar
#	LPL_SCALEBAR(lpl)  = NO			#btoi (clgetb ("scalebar"))

	# Dummy init for now; need new parameter to actually set.
	LPL_ONE_X_ONLY(lpl) = NO
	
	call sfree (sp)
end
