include <ctype.h>
include <gio.h>
include <gset.h>
include <mach.h>
include "lpl.h"

# LPL_COORDS -- Get data/coordinate related parameters for task LONGPLOT and
# set in longplot descriptor LPL.

procedure lpl_coords (lpl)

pointer lpl	# LONGPLOT descriptor

pointer	sp, stringbuf
int	ip
real	xscale,yscale
real	wxmin,wxmax, wymin,wymax, pxmin,pxmax, pymin,pymax

bool	clgetb()
int	clgeti(), btoi(), strncmp()
real	clgetr()

errchk	lpl_parse_coord()

define	xweedout_	91
define	yweedout_	92

# We could save some memory here, but this is clearer.
string	wx0 "WARNING (LPL_COORDS): both wxmin,wxmax and xautoexpand specified\n"
string	wx1 		     "%22t xautoexpand overridden to NO.\n"
string	wx2 "WARNING (LPL_COORDS): both pxmin,pxmax and xscale specified\n"
string	wx3 		     "%22t pxmax overridden (will depend on data).\n"
string	wx4 "WARNING (LPL_COORDS): one of wxmin,wxmax not specified;\n"
string	wx5 		     "%22t xautoexpand overridden to YES.\n"
string	wx6 "WARNING (LPL_COORDS): pxmin,pxmax do not agree with wxmin,wxmax\n"
string	wx7 		     "%22t and xscale; pxmax overridden.\n"

string	wy0 "WARNING (LPL_COORDS): both wymin,wymax and yautoexpand specified\n"
string	wy1 		     "%22t yautoexpand overridden to NO.\n"
string	wy2 "WARNING (LPL_COORDS): both pymin,pymax and yscale specified\n"
string	wy3 		     "%22t pymax overridden (will depend on data).\n"
string	wy4 "WARNING (LPL_COORDS): one of wymin,wymax not specified;\n"
string	wy5 		     "%22t yautoexpand overridden to YES.\n"
string	wy6 "WARNING (LPL_COORDS): pymin,pymax do not agree with wymin,wymax\n"
string	wy7 		     "%22t and yscale; pymax overridden.\n"

#debug+
int	debug
common /deb/ debug
#debug-
begin
	call smark (sp)
	call salloc (stringbuf, SZ_LINE, TY_CHAR)
	call malloc (LPL_XTRSTR(lpl), SZ_LINE, TY_CHAR)
	call malloc (LPL_YTRSTR(lpl), SZ_LINE, TY_CHAR)

	# Get autoexpand flags.
	LPL_XAUTOEXPAND(lpl) = btoi (clgetb ("xautoexpand"))
	LPL_YAUTOEXPAND(lpl) = btoi (clgetb ("yautoexpand"))

	# WCS, plotter range, scale parameters; initialize working variables
	# to INDEF so we can tell what has been set.

	wxmin = INDEFR
	wxmax = INDEFR
	wymin = INDEFR
	wymax = INDEFR
	pxmin = INDEFR
	pxmax = INDEFR
	pymin = INDEFR
	pymax = INDEFR

	# World min and max:
	call clgstr ("wx", Memc[stringbuf], SZ_LINE)
	if (Memc[stringbuf] != EOS)
	    call lpl_parse_coord (Memc[stringbuf], wxmin, wxmax)

	call clgstr ("wy", Memc[stringbuf], SZ_LINE)
	if (Memc[stringbuf] != EOS)
	    call lpl_parse_coord (Memc[stringbuf], wymin, wymax)

	# Plotter min and max; these are actual plotter units, not a viewport.
	call clgstr ("px", Memc[stringbuf], SZ_LINE)
	if (Memc[stringbuf] != EOS)
	    call lpl_parse_coord (Memc[stringbuf], pxmin, pxmax)
	
	call clgstr ("py", Memc[stringbuf], SZ_LINE)
	if (Memc[stringbuf] != EOS)
	    call lpl_parse_coord (Memc[stringbuf], pymin, pymax)

	# Scales.
	xscale = clgetr ("xscale")
	yscale = clgetr ("yscale")

	# Coordinate transformation strings.
	call clgstr ("xt", Memc[stringbuf], SZ_LINE)
	for (ip = 1;  IS_WHITE(Memc[stringbuf+ip-1]);  ip = ip + 1)
	    ;
	if (Memc[stringbuf+ip-1] != EOS) {
	    LPL_XTRAN(lpl) = YES
	    call strcpy (Memc[stringbuf+ip-1], Memc[LPL_XTRSTR(lpl)], SZ_LINE)
	} else
	    LPL_XTRAN(lpl) = NO

	call clgstr ("yt", Memc[stringbuf], SZ_LINE)
	for (ip = 1;  IS_WHITE(Memc[stringbuf+ip-1]);  ip = ip + 1)
	    ;
	if (Memc[stringbuf+ip-1] != EOS) {
	    LPL_YTRAN(lpl) = YES
	    call strcpy (Memc[stringbuf+ip-1], Memc[LPL_YTRSTR(lpl)], SZ_LINE)
	} else
	    LPL_YTRAN(lpl) = NO

	# Rounding.
	LPL_XROUND(lpl) = btoi (clgetb ("xround"))
	LPL_YROUND(lpl) = btoi (clgetb ("yround"))

	# Plotter units
	call clgstr ("units", Memc[stringbuf], MAXWORDSIZE)
	for (ip = 1;  IS_WHITE(Memc[stringbuf+ip-1]);  ip = ip + 1)
	    ;
	if (strncmp (Memc[stringbuf+ip-1], "normal", 1) == 0 ||
	    strncmp (Memc[stringbuf+ip-1], "inches", 1) ==0)
	    LPL_UNIT_SCALE(lpl) = 1.0
	else if (strncmp (Memc[stringbuf+ip-1], "centimeters", 1) == 0 ||
	    strncmp (Memc[stringbuf+ip-1], "cm", 1) == 0) {
	    LPL_UNIT_SCALE(lpl) =  1.0 / INCHES_TO_CM
	    if (!IS_INDEF(xscale))
		xscale = xscale * INCHES_TO_CM
	    if (!IS_INDEF(yscale))
		yscale = yscale * INCHES_TO_CM
	}

	# Axis for image projection
	LPL_AXIS(lpl) = clgeti ("axis")

	# Now that we have made all the requests, filter invalid combinations
	# of scaling, autoexpanding, and specified mins and maxes for world
	# and plotter units.

xweedout_
	if (LPL_XAUTOEXPAND(lpl) == YES) {		# autoexpanding
	    if (!IS_INDEF(wxmin) && !IS_INDEF(wxmax)) {
		call eprintf (wx0)
		call eprintf (wx1)
		LPL_XAUTOEXPAND(lpl) = NO
		goto xweedout_
	    }

	    if (!IS_INDEF(xscale))			#     scaling
		if (!IS_INDEF(pxmin) && !IS_INDEF(pxmax)) {
		    call eprintf (wx2)
		    call eprintf (wx3)
		    pxmax = INDEFR
		    goto xweedout_
		}

	} else {					# not autoexpanding
	    if (IS_INDEF(wxmin) || IS_INDEF(wxmax)) {
		call eprintf (wx4)
		call eprintf (wx5)
		LPL_XAUTOEXPAND(lpl) = YES
		goto xweedout_
	    } 
	    if (!IS_INDEF(xscale)) {			#     scaling
		if (!IS_INDEF(wxmin) && !IS_INDEF(wxmax) &&
		    !IS_INDEF(pxmin) && !IS_INDEF(pxmax)) {

		    # We are overdetermined, but this is not an error if
		    # everything agrees with itself to within tolerance.
		    if (abs (xscale * (pxmax - pxmin)) - abs (wxmax - wxmin) >
			EPSILON) {
			call eprintf (wx6)
			call eprintf (wx7)
			pxmax = INDEFR
		    }
		}
	    }
	}

yweedout_
	if (LPL_YAUTOEXPAND(lpl) == YES) {		# autoexpanding
	    if (!IS_INDEF(wymin) && !IS_INDEF(wymax)) {
		call eprintf (wy0)
		call eprintf (wy1)
		LPL_YAUTOEXPAND(lpl) = NO
		goto yweedout_
	    }

	    if (!IS_INDEF(yscale))			#     scaling
		if (!IS_INDEF(pymin) && !IS_INDEF(pymax)) {
		    call eprintf (wy2)
		    call eprintf (wy3)
		    pymax = INDEFR
		    goto yweedout_
		}

	} else {					# not autoexpanding
	    if (IS_INDEF(wymin) || IS_INDEF(wymax)) {
		call eprintf (wy4)
		call eprintf (wy5)
		LPL_YAUTOEXPAND(lpl) = YES
		goto yweedout_
	    } 
	    if (!IS_INDEF(yscale)) {			#     scaling
		if (!IS_INDEF(wymin) && !IS_INDEF(wymax) &&
		    !IS_INDEF(pymin) && !IS_INDEF(pymax)) {

		    # We are overdetermined, but this is not an error if
		    # everything agrees with itself to within tolerance.
		    if (abs (yscale * (pymax - pymin)) - abs (wymax - wymin) >
			EPSILON) {
			call eprintf (wy6)
			call eprintf (wy7)
			pymax = INDEFR
		    }
		}
	    }
	}

	# We now have a valid combination of specified / unspecified working
	# parameters.  The unassigned values may need to be filled after calls
	# to g[ar]scale, so this is as far as we go here.  Assign to structure.

	LPL_WXMIN(lpl) = wxmin
	LPL_WXMAX(lpl) = wxmax
	LPL_WYMIN(lpl) = wymin
	LPL_WYMAX(lpl) = wymax
	LPL_PXMIN(lpl) = pxmin
	LPL_PXMAX(lpl) = pxmax
	LPL_PYMIN(lpl) = pymin
	LPL_PYMAX(lpl) = pymax

	LPL_XSCALE(lpl) = xscale
	LPL_YSCALE(lpl) = yscale

	call sfree (sp)
end
