include	<mach.h>
include	<gio.h>
include <gset.h>
include "lpl.h"
include "lpx.h"

define	LEFT_BORDER		10			# nchars at l|r edge
define	BOTTOM_BORDER		6			# nlines at bottom edge
define	MAX_SZTITLEBLOCK	0.5			# max sztitleblock, NDC
define	MIN_NTITLELINES		2			# min lines in titleblk


# LPL_GET_VIEWPORT -- Determine the final viewport size as f(title, labels),
# i.e. compute the size of the largest viewport which leaves sufficient room
# around the border for the axis labels and plot title.

procedure lpl_get_viewport (gd, lpl, lpx, curve, title, xlabel, ylabel)

pointer	gd			# graphics descriptor
pointer lpl			# longplot descriptor
pointer	lpx			# pointer to the per-curve structure
int	curve			# current curve number
char	title[ARB]		# title string
char	xlabel[ARB]		# x axis label
char	ylabel[ARB]		# y axis label


int	ntitlelines, ip
bool	draw_title, draw_xlabel, draw_ylabel, draw_xticks, draw_yticks
real	char_height,char_width
real	xwidth,ywidth, yreserve
real	ggetr(), gstatr()

#debug+
int	debug
common	/deb/ debug
#debug-
begin
	# Count the number of lines in the title block.
	ntitlelines = 0
	if (title[1] != EOS) {
	    for (ip=1;  title[ip] != EOS;  ip=ip+1)
		if (title[ip] == '\n' && title[ip+1] != EOS)
		    ntitlelines = ntitlelines + 1
	    ntitlelines = ntitlelines + 1
	}

	draw_xticks = (LPL_TICKLABEL(lpl) == YES)
	draw_yticks = draw_xticks			# may add later
	draw_title  = (ntitlelines > 0)
	draw_xlabel = (draw_xticks && xlabel[1] != EOS)
	draw_ylabel = (draw_yticks && ylabel[1] != EOS)

	char_width  = ggetr (gd, "cw")
	char_height = ggetr (gd, "ch")

	if (char_width < EPSILON)
	    char_width = DEF_CHARWIDTH
	if (char_height < EPSILON)
	    char_height = DEF_CHARHEIGHT

	# X axis.
	if (draw_yticks && draw_ylabel)
	    xwidth = max (4, LEFT_BORDER + 2)
	else if (draw_yticks)
	    xwidth = max (4, LEFT_BORDER)
	else
	    xwidth = 0
	xwidth = xwidth * char_width * gstatr (gd, G_XTICKLABELSIZE)
	    
	# Y axis.
	if (draw_xticks && draw_xlabel)
	    ywidth = BOTTOM_BORDER
	else if (draw_xticks)
	    ywidth = max (2, (BOTTOM_BORDER - 2))
	else
	    ywidth = 0
	ywidth = ywidth * char_height * gstatr (gd, G_YTICKLABELSIZE)

	# If no tick labels, axis labels, or plot title is desired, use the
	# full screen.

	if (xwidth < EPSILON && ywidth < EPSILON && !draw_title)
	    return

	# Compute amount of extra space to allow for the title block, which
	# may contain more than one line.

	if (!draw_title && !draw_xticks && !draw_yticks)
	    yreserve = 0
	else
	    yreserve = min (MAX_SZTITLEBLOCK,
		max (MIN_NTITLELINES, ntitlelines + 1) * char_height)

	# Set the viewport.  The viewport is the largest area yielding the
	# desired borders.  The viewport is centered in X and positioned just
	# below the title block in Y.

	LPX_VXMIN(lpx, curve-1) = xwidth
	LPX_VXMAX(lpx, curve-1) = 1.0 - xwidth
	LPX_VYMIN(lpx, curve-1) = ywidth
	LPX_VYMAX(lpx, curve-1) = 1.0 - yreserve
#debug+
if (debug == YES) {
call eprintf ("lpl_get_viewport: nttl %d, yres %7.4f, ywid %7.4f, ch %7.4f\n")
call pargi (ntitlelines)
call pargr (yreserve)
call pargr (ywidth)
call pargr (char_height)
call eprintf ("		draw_xtick %b, ytick %b, title %b, xlab %b, ylab %b\n")
call pargb (draw_xticks)
call pargb (draw_yticks)
call pargb (draw_title)
call pargb (draw_xlabel)
call pargb (draw_ylabel)
call eprintf ("		vxmin %7.4f, vxmax %7.4f, vymin %7.4f, vymax %7.4f\n")
call pargr (LPX_VXMIN(lpx, curve-1))
call pargr (LPX_VXMAX(lpx, curve-1))
call pargr (LPX_VYMIN(lpx, curve-1))
call pargr (LPX_VYMAX(lpx, curve-1))
}
#debug-
end
