include <gio.h>
include <gset.h>
include <mach.h>
include "lpl.h"
include "lpx.h"


# LPL_GLABAX -- Draw axes, ticks, labels, and titles.  Do gsets for line and
# markertypes to be drawn in subsequent lpl_drawdata.

procedure lpl_glabax (lpl, lpx, gd, curve, ncurves)

pointer	lpl			# longplot descriptor
pointer	lpx			# curve arrays to aid in stack plotting
pointer	gd			# graphics descriptor
int	curve			# current curve number
int	ncurves			# total curves

bool	overtrace, stack, series
int	ttl_offset
real	orig_ticlen, ticlen, char_height, csize

int	strlen()
real	ggetr(), gstatr()

begin
	# These are used heavily; convert to logicals:
	overtrace = (LPL_MULTIPLE(lpl) == MULT_OVERTRACE)
	stack     = (LPL_MULTIPLE(lpl) == MULT_STACK || LPL_MULTIPLE(lpl) ==
	    MULT_STACK_DY)
	series    = (LPL_MULTIPLE(lpl) == MULT_SERIES)

	# Titles; stack and overtrace plots do not get an image title even
	# if "title=imtitle".  Instead, a multi-line title will appear above
	# the topmost plot containing only the input specifications, one per
	# line.

	ttl_offset = Memi[LPL_TITLENDX(lpl) + curve - 1] - 1
	if (!series && curve == 1) {

	    # Draw the first curve's title only, above the topmost curve.
	    # This title may have been built up from individual inputspecs,
	    # in inverted order one per line, by lpl_rdcrvs or lpl_indirection.

	    call gsview (gd, LPX_VXMIN(lpx,0),LPX_VXMAX(lpx,0),
		LPX_VYMIN(lpx,0), LPX_VYMAX(lpx,ncurves-1))
	    call gseti (gd, G_DRAWAXES, 0)
	    call gseti (gd, G_DRAWGRID, NO)
	    call gseti (gd, G_DRAWTICKS, NO)
	    call gseti (gd, G_LABELTICKS, NO)
	    call gseti (gd, G_DRAWTITLE, YES)
#	    call lbl_glabax (gd, TITLE(LPL_TITLE(lpl), 0), "", "")
	    call glabax (gd, TITLE(LPL_TITLE(lpl), 0), "", "")
	    call gsview (gd, LPX_VXMIN(lpx,0), LPX_VXMAX(lpx,0),
		LPX_VYMIN(lpx,0), LPX_VYMAX(lpx,0))
	    call gseti (gd, G_DRAWTITLE, NO)
	    call amovkc (EOS, TITLE(LPL_TITLE(lpl), 0), 
		strlen (TITLE(LPL_TITLE(lpl), 0)))
	    call amovki (1, Memi[LPL_TITLENDX(lpl)], ncurves)

	} else if (!series)
	    call gseti (gd, G_DRAWTITLE, NO)
	else
	    call gseti (gd, G_DRAWTITLE, YES)

	# Post-transform log/elog plotting:
	if (LPL_XLOGLIN(lpl) == LOG)
	    call gseti (gd, G_XTRAN, LOG)
	else
	    call gseti (gd, G_XTRAN, LINEAR)
	if (LPL_YLOGLIN(lpl) == LOG)
	    call gseti (gd, G_YTRAN, LOG)
	else
	    call gseti (gd, G_YTRAN, LINEAR)

	# Axis and tick drawing and labelling:
	if (overtrace && curve > 1) {
	    call gseti (gd, G_DRAWAXES, 0)
	    call gseti (gd, G_XDRAWTICKS, NO)
	    call gseti (gd, G_YDRAWTICKS, NO)
	    call gseti (gd, G_XLABELTICKS, NO)
	    call gseti (gd, G_YLABELTICKS, NO)
	    call gseti (gd, G_XLABELAXIS, NO)
	    call gseti (gd, G_YLABELAXIS, NO)
	} else if (stack) {
	    if (curve == 1 && ncurves == 1) {
		call gseti (gd, G_DRAWAXES, 3)
		call gseti (gd, G_XDRAWTICKS, YES)
		call gseti (gd, G_YDRAWTICKS, YES)
		call gseti (gd, G_XLABELTICKS, LPL_TICKLABEL(lpl))
		call gseti (gd, G_YLABELTICKS, LPL_TICKLABEL(lpl))
		call gseti (gd, G_XLABELAXIS, YES)
		call gseti (gd, G_YLABELAXIS, YES)
	    } else if (curve == 1) {
		call gseti (gd, G_XDRAWAXES, 1)
		call gseti (gd, G_YDRAWAXES, 3)
		call gseti (gd, G_XDRAWTICKS, YES)
		call gseti (gd, G_YDRAWTICKS, YES)
		call gseti (gd, G_XLABELTICKS, LPL_TICKLABEL(lpl))
		call gseti (gd, G_YLABELTICKS, NO)
		call gseti (gd, G_XLABELAXIS, YES)
		call gseti (gd, G_YLABELAXIS, NO)
	    } else if (curve > 1 && curve == ncurves) {
		call gseti (gd, G_XDRAWAXES, 2)
		call gseti (gd, G_YDRAWAXES, 3)
		call gseti (gd, G_XDRAWTICKS, YES)
		call gseti (gd, G_YDRAWTICKS, YES)
		call gseti (gd, G_XLABELTICKS, NO)
		call gseti (gd, G_YLABELTICKS, NO)
		call gseti (gd, G_XLABELAXIS, NO)
		call gseti (gd, G_YLABELAXIS, NO)
	    } else if (curve > 1) {
		call gseti (gd, G_XDRAWAXES, 0)
		call gseti (gd, G_YDRAWAXES, 3)
		call gseti (gd, G_XDRAWTICKS, NO)
		call gseti (gd, G_YDRAWTICKS, YES)
		call gseti (gd, G_XLABELTICKS, NO)
		call gseti (gd, G_YLABELTICKS, NO)
		call gseti (gd, G_XLABELAXIS, NO)
		call gseti (gd, G_YLABELAXIS, NO)
	    }

	} else {

	    call gseti (gd, G_DRAWAXES, 3)
	    call gseti (gd, G_XDRAWTICKS, YES)
	    call gseti (gd, G_YDRAWTICKS, YES)
	    call gseti (gd, G_XLABELTICKS, LPL_TICKLABEL(lpl))
	    call gseti (gd, G_YLABELTICKS, LPL_TICKLABEL(lpl))
	    call gseti (gd, G_XLABELAXIS, YES)
	    call gseti (gd, G_YLABELAXIS, YES)
	}

	# Axis tick spacing; determined for each curve in lpl_calc_coords().
	call gseti (gd, G_XNMAJOR, LPX_MAJRX(lpx,curve-1))
	call gseti (gd, G_XNMINOR, LPL_MINRX(lpl))

	call gseti (gd, G_YNMAJOR, LPX_MAJRY(lpx,curve-1))
	call gseti (gd, G_YNMINOR, LPL_MINRY(lpl))

	# Override bold text and tick generation unless requested.
	if (LPL_LBOLD(lpl) == NO) {
	    call gseti (gd, G_TXFONT, GT_ROMAN)
	    call gsetr (gd, G_MAJORWIDTH, 1.0)
	    call gsetr (gd, G_MINORWIDTH, 1.0)
	    call gsetr (gd, G_AXISWIDTH, 1.0)
	} else {
	    call gseti (gd, G_TXFONT, GT_BOLD)
	    call gsetr (gd, G_MAJORWIDTH, 2.0)
	    call gsetr (gd, G_MINORWIDTH, 2.0)
	    call gsetr (gd, G_AXISWIDTH, 2.0)
	}

	# Set linetype already determined for this curve.
	call gseti (gd, G_PLTYPE, GL_SOLID)
	if (LPL_PLOTMODE(lpl) != PLOT_MARK)
	    if (LPX_LINETYPE(lpx,curve-1) != NULL)
		call gseti (gd, G_PLTYPE, LPX_LINETYPE(lpx,curve-1))
	    else if ((stack && LPL_LEGEND(lpl) == YES) || (overtrace &&
	        curve > 1))
		call gseti (gd, G_PLTYPE, mod (curve-1, NUM_GIO_LTYPE) + 1)

	if (LPL_DBOLD(lpl) == YES) {
	    call gsetr (gd, G_PLWIDTH, 2.0)
	    call gsetr (gd, G_PMWIDTH, 2.0)
	} else {
	    call gsetr (gd, G_PLWIDTH, 1.0)
	    call gsetr (gd, G_PMWIDTH, 1.0)
	}

	# Assign marker types and sizes if not plotting lines-only.
	if (LPL_PLOTMODE(lpl) != PLOT_LINE) {
	    if (LPX_MARKERTYPE(lpx,curve-1) == NULL && (LPL_LEGEND(lpl) == YES
		|| LPL_MULTIPLE(lpl) == MULT_OVERTRACE))
		if (curve == 1)
		    LPX_MARKERTYPE(lpx,curve-1) = GM_POINT
		else
		    LPX_MARKERTYPE(lpx,curve-1) = 2 ** mod (curve-1,
		        NUM_GIO_MARKER)

	    # Convert markersize in relative character units to NDC units.
	    char_height = ggetr (gd, "ch")
	    csize = gstatr (gd, G_TXSIZE)
	    if (char_height < EPSILON)
		char_height = DEF_CHARHEIGHT
	    LPX_MARKERSIZE(lpx,curve-1) = LPX_MARKERSIZE(lpx,curve-1) * csize *
		char_height
	}

	# Scale the y-axis tick marks to those of the x-axis to avoid greatly
	# stretched ones in plots with x-dimension >> y-dimension.

	orig_ticlen = 0.6 * ggetr (gd, "ch")
	ticlen = orig_ticlen * LPX_PYSCALE(lpx,curve-1) / 
	    LPX_PXSCALE(lpx,curve-1)
	call gsetr (gd, G_YMAJORLENGTH, ticlen)
	call gsetr (gd, G_YMINORLENGTH, ticlen * 0.5)
	call gsetr (gd, G_XMAJORLENGTH, orig_ticlen)
	call gsetr (gd, G_XMINORLENGTH, orig_ticlen * 0.5)
	
	# Grid?
	if (LPL_GRID(lpl) == YES)
	    call gseti (gd, G_DRAWGRID, YES)
	else
	    call gseti (gd, G_DRAWGRID, NO)

	# Finally, draw it all.
#	call lbl_glabax (gd, TITLE(LPL_TITLE(lpl),ttl_offset),
#	    XLABEL(LPL_XLABEL(lpl)), YLABEL(LPL_YLABEL(lpl)))
	call glabax (gd, TITLE(LPL_TITLE(lpl),ttl_offset),
	    XLABEL(LPL_XLABEL(lpl)), YLABEL(LPL_YLABEL(lpl)))
end
