include <chars.h>
include	<error.h>
include "lpl.h"

define	MAXCHARS_INLTYPE	3

# LPL_INDIRECTION -- Input was indirection; read all fields for each
# input specification, one per line.  Return number of curves as function value.

int procedure lpl_indirection (lpl, lpx, x, y, nelems)

pointer	lpl			# longplot descriptor
pointer	lpx			# per-curve arrays
pointer	x[MAX_CURVES]		# x data for all curves
pointer y[MAX_CURVES]		# y data for all curves
int	nelems[MAX_CURVES]	# number of points in each curve

pointer	sp, lbuf, fd, ip, fname, title_str, ltype_str, markerspec, holdtitle
int	i, lines_read, totchars, fname_chars, title_chars
int	ltype_chars, marker_chars
bool	series, onetitle, needtitle

int	open(), getline(), strlen(), ctowrd()
int	lpl_cnv_ltype(), lpl_cnv_marker(), lpl_rdcurve()
bool	streq(), strne()

errchk	open(), malloc(), getline(), lpl_rdcurve()

string	maxcrverr " (LPL_INDIR): currently max curves available=        "
string	badlnerr  " (LPL_INDIR): @file has bad line; line no. =        "

#debug+
int	debug
common /deb/ debug
#debug-
begin
	# Maximum number of (line + continuations) = 4; i.e. max of 1 line/param
	call smark (sp)
	call salloc (lbuf, SZ_LINE*4, TY_CHAR)
	call salloc (fname, SZ_FNAME, TY_CHAR)
	call salloc (title_str, SZ_LINE*4, TY_CHAR)
	call salloc (holdtitle, SZ_LINE*4, TY_CHAR)
	call salloc (ltype_str, SZ_LINE, TY_CHAR)
	call salloc (markerspec, SZ_LINE, TY_CHAR)

	iferr (fd = open (INBUF(LPL_INBUF(lpl), 1), READ_ONLY, TEXT_FILE))
	    call erract (EA_FATAL)

	# This is used heavily; convert to logical:
	series = (LPL_MULTIPLE(lpl) == MULT_SERIES)
	LPL_DIFFERENTIATE(lpl) = NO

	# For each line in the file (lines may be "\n"'d), get input
	# specification, title specification, linetype, markertype (or place
	# holders for any except input).  Strategy is to build a single line
	# buffer into lbuf from [multiple] input lines, then scan for args.

	i = 0
	lines_read = 0
	while (getline (fd, Memc[lbuf]) != EOF) {
	    lines_read = lines_read + 1
	    ip = lbuf + strlen (Memc[lbuf])
	    while (streq (Memc[ip-2], "\n")) {
		Memc[ip-2] = BLANK
		Memc[ip-1] = EOS
		ip = ip - 1
		if (getline (fd, Memc[ip]) != EOF) {
		    lines_read = lines_read + 1
		    ip = ip + strlen (Memc[ip])
		}
	    }

	    i = i + 1
	    if (i > MAX_CURVES) {
		call sprintf (maxcrverr[46], 8, "%d")
		    call pargi (MAX_CURVES)
		call error (0, maxcrverr)
	    }

	    # Must pull each arg into a string buffer to permit placeholders.
	    ip = 1
	    totchars = strlen (Memc[lbuf])

	    fname_chars = ctowrd (Memc[lbuf], ip, Memc[fname], SZ_FNAME)
#debug+
if (debug == YES) {
call eprintf ("filename = %s\n")
call pargstr (Memc[fname])
}
#debug-
	    if (fname_chars < 1) {
		call sprintf (badlnerr[45], 8, "%d")
		    call pargi (lines_read)
		call error (0, badlnerr)
	    }

	    needtitle = false
	    if (ip <= totchars) {
		title_chars = ctowrd (Memc[lbuf], ip, Memc[title_str], 
		    SZ_LINE*4)
#debug+
if (debug == YES) {
call printf ("title str = %s\n")
call pargstr (Memc[title_str])
}
#debug-
		if (title_chars > 0)
		    if (streq ("imtitle", Memc[title_str]) ||
			streq ("file", Memc[title_str]) || streq ("*",
			    Memc[title_str]))
			onetitle = false
		    else
			onetitle = true

		    if (!series) {
			needtitle = false
			if (strne ("none", Memc[title_str]))
			    if (i == 1 && !onetitle)
				call strcpy (Memc[fname], Memc[holdtitle],
				    SZ_FNAME)
			    else if (i == 1 && onetitle)
				call strcpy (Memc[title_str], Memc[holdtitle],
				    SZ_FNAME)
			    else {
				call strcat ('\n', Memc[holdtitle], SZ_LINE*4)
				call strcat (Memc[fname], Memc[holdtitle],
				    SZ_LINE*4)
			    }

		    } else if (!onetitle) {
			needtitle = true
			call strcpy (Memc[title_str], Memc[holdtitle], SZ_FNAME)

		    } else {
			needtitle = false
			if (streq ("none", Memc[title_str]))
			    call lpl_assign_title (lpl, "", i)
			else
			    call lpl_assign_title (lpl, Memc[title_str], i)
		    }
	    }

	    # Linetype.
	    if (ip <= totchars) {
		ltype_chars = ctowrd (Memc[lbuf], ip, Memc[ltype_str], 
		    MAXCHARS_INLTYPE)
#debug+
if (debug == YES) {
call printf ("linetype = %s\n")
call pargstr (Memc[ltype_str])
}
#debug-

		if (ltype_chars > 0 && LPL_PLOTMODE(lpl) != PLOT_MARK) {
		    call lpl_set_ltype (lpx, lpl_cnv_ltype (Memc[ltype_str]), i)
		    LPL_DIFFERENTIATE(lpl) = YES
		}
	    }

	    # Markertype.
	    if (ip <= totchars) {
		marker_chars = ctowrd (Memc[lbuf], ip, Memc[markerspec],
		    SZ_LINE)
#debug+
if (debug == YES) {
call printf ("markerspec = %s\n")
call pargstr (Memc[markerspec])
}
#debug-
		if (marker_chars > 0 && LPL_PLOTMODE(lpl) != PLOT_LINE) {
		    call lpl_set_marker (lpx, lpl_cnv_marker (Memc[markerspec]),
		        i)
		    LPL_DIFFERENTIATE(lpl) = YES
		}
	    }

	    iferr {
		nelems[i] = lpl_rdcurve (lpl, Memc[fname], x[i],y[i],
		    needtitle, Memc[holdtitle], i)
	    } then {
		call erract (EA_WARN)
		i = i - 1
	    }
	}
	    
	if (!series) {
	    call lpl_invert (Memc[holdtitle])
	    call lpl_assign_title (lpl, Memc[holdtitle], 1)
	}

	call close (fd)
	call sfree (sp)

	return (i)
end
