include <ctotok.h>
include <ctype.h>
include "lpl.h"

define	NUM_FUNC	12	# <--***** remove when replace w/ WIC/CTR.
define	LEFT		1
define	RIGHT		2

# LPL_PARSE_TRANSFORM -- Parse coordinate transform.  Form is either:
# t = "[c1] [*] [func] [**] [c2]", where f() is one of the SPP intrinsic
# functions of one variable, or:
# t = "p1=x1 p2=x2", where p1 refers to pixel 1, p2 to the last pixel.

procedure lpl_parse_transform (in_str, c1, func, c2, transform, linear, coordt)

char	in_str[ARB]	# string to be parsed
real	c1		# optional constant multiplier
int	func		# index of an SPP intrinsic function
real	c2		# optional constant exponent
int	transform	# is transform necessary?
int	linear		# is transform linear?
int	coordt		# is transform of form "p1=x1 p2=x2"?

int	state, ip, wip, token, i, lin_trans[NUM_FUNC]
real	bin[2]
char	word[MAXWORDSIZE], op, junk[1]
bool	needfunc

int	ctor(), ctotok(), strlen(), strdic(), strncmp()

data	lin_trans  / YES, NO, NO, NO, NO, NO, NO, NO, YES, NO, NO, NO/

include "functions.h"

begin
	# Syntax:  [c1 *] [f[(]] [old] [)] [** c2]  OR:
	#	   p1=x1 p2=x2

	for (ip=1; IS_WHITE(in_str[ip]); ip=ip+1)
	    ;
	if (strlen (in_str[ip]) <= 0) {
	    transform = NO
	    return
	}

	state = LEFT
	bin[LEFT] = INDEFR
	bin[RIGHT] = INDEFR

	# Determine the form of transform and proceed:
	if (strncmp (in_str[ip], "p1", 2) == 0) {
	    ip = ip + 2
	    coordt = YES
	    for (; IS_WHITE(in_str[ip]) || in_str[ip] == '='; ip=ip+1)
		;

	    wip = 1
	    for (; !IS_WHITE(in_str[ip]) && in_str[ip] != EOS; ip=ip+1) {
		if (wip > MAXWORDSIZE) {
		    call eprintf ("maxwordsize exceeded in transform\n")
		    call error (0, "LPL_PARSE_TRANSFORM: invalid transform\n")
		}
		word[wip] = in_str[ip]
		wip = wip + 1
	    }

	    word[wip] = EOS
	    wip = 1
	    if (ctor (word, wip, bin[state]) <= 0) {
		call eprintf ("LPL_PARSTR: cannot convert transform p1=%s\n")
		    call pargstr (word)
		call error (0, "LPL_PARSE_TRANSFORM: invalid transform\n")
	    }

	    state = RIGHT
	    for (; IS_WHITE(in_str[ip]); ip=ip+1)
		;
	    if (strncmp (in_str[ip], "p2", 2) != 0) {
		call eprintf ("LPL_PARSTR: p2=value must follow p1; found %s\n")
		    call pargstr (in_str[ip])
		call error (0, "LPL_PARSE_TRANSFORM: invalid transform\n")
	    }
	    ip = ip + 2
	    for (; IS_WHITE(in_str[ip]) || in_str[ip] == '='; ip=ip+1)
		;

	    wip = 1
	    for (; !IS_WHITE(in_str[ip]) && in_str[ip] != EOS; ip=ip+1) {
		if (wip > MAXWORDSIZE) {
		    call eprintf ("maxwordsize exceeded in transform\n")
		    call error (0, "LPL_PARSE_TRANSFORM: invalid transform\n")
		}
		word[wip] = in_str[ip]
		wip = wip + 1
	    }

	    wip = 1
	    if (ctor (word, wip, bin[state]) <= 0) {
		call eprintf ("LPL_PARSTR: cannot convert transform p2=%s\n")
		    call pargstr (word)
		call error (0, "LPL_PARSE_TRANSFORM: invalid transform\n")
	    }
	    
	} else {
	    coordt = NO
	    needfunc = true

	    repeat {
		token = ctotok (in_str, ip, word, MAXWORDSIZE)
		if (strlen (word) >= MAXWORDSIZE) {
		    call eprintf ("maxwordsize exceeded in transform\n")
		    call error (0, "LPL_PARSE_TRANSFORM: invalid transform\n")
		}
		switch (token) {
		case TOK_OPERATOR, TOK_PUNCTUATION:
		    state = RIGHT
		    op  = word[1]
		case TOK_NUMBER:
		    wip = 1
		    if (ctor (word, wip, bin[state]) <= 0)
			bin[state] = INDEFR
		case TOK_IDENTIFIER:
		    if (needfunc) {
			func = strdic (word, junk, 1, intrinsics)
			needfunc = false
			if (func == 0)
			    call error (0, 
				"LPL_PARSE_TRANSFORM: invalid transform\n")
		    }
		}
	    } until (token == TOK_EOS)
	}

	c1 = bin[LEFT]
	c2 = bin[RIGHT]

	if ((c1 != INDEFR) || (c2 != INDEFR) || (coordt == NO && func != INDEFI
	    && func != 0))
	    transform = YES
	
	# Establish whether transform is linear.
	if (coordt == NO) {
	    linear = NO
	    do i = 1, NUM_FUNC
		if (lin_trans[i] == YES) linear = YES
	} else
	    linear = YES

end
