include <gescape.h>
include	<gki.h>
include <mach.h>
include "lpl.h"
include "lpx.h"

# LPL_PLOT -- Plot routine dispatcher for task longplot; called once from main.

procedure lpl_plot (lpl, gd, x, y, nelems)

pointer	lpl			# longplot descriptor
pointer	gd			# graphics device descriptor
pointer	x[MAX_CURVES]		# x data for all curves
pointer y[MAX_CURVES]		# y data for all curves
int	nelems[MAX_CURVES]	# No. of points in each curve

pointer	lpx
int	ncurves, i

int	lpl_rdcurves()

errchk	lpl_rdcurves()

begin
	# Initialize the per-curve arrays governing sizes, scales, line/marker
	# types, etc. to MAX_CURVES; later realloc them to actual ncurves.
	call lpl_init_lpx (lpx, MAX_CURVES)

	# Read all curves into memory, along with appearance params specific
	# to individual plots if input was indirected.  Coordinate transforms
	# are handled in the appropriate read routine (list, image).

	ncurves = lpl_rdcurves (lpl, lpx, x, y, nelems)

	# Shrink down memory in the per-curve arrays.
	call lpl_reset_lpx (lpx, ncurves)

	# Set initial world ranges for all curves in lpx descriptor.
	call lpl_set_ranges (lpl, lpx, gd, x, y, nelems, ncurves)

	# Set lpl descriptor for initial (non-stacking) plotter ranges [and
	# world/plotter scales]; necessary for real ones in lpl_calc_coords.
	call lpl_set_plotp (lpl, gd)

	# Calculate expanded WCS(s) to accommodate rounding and/or non-linear
	# transformations, and shrunken viewport(s) to accommodate titles,
	# labels, etc.  This must be done here to handle many-curve plotting.

	call lpl_calc_coords (lpl, lpx, gd, ncurves)

	# Set real wcs and viewport for curve 1.
	call lpl_set_wv (lpx, gd, 1)

	# Send the plotter scales (plotter units / GKI_NDC) to any non-standard
	# kernel listening for them.
	call lpl_set_exact (lpl, gd, LPX_PXSCALE(lpx,0), LPX_PYSCALE(lpx,0))

	# Legend and scalebar (if legend, scalebar must be handled from within).
	if (LPL_TITLEPOS(lpl) == POS_LEGEND)
	    call lpl_drawlegend (lpl, lpx, gd, ncurves)
	else if (LPL_SCALEBAR(lpl) == YES)
	    call lpl_drawscalebar (lpl, gd)

	# For each curve, draw any individual-plot axes, labels, and titles,
	# plus the data.

	do i = 1, ncurves {

	    # Reset wcs & viewport, kernel scale, and do a frame clear, all
	    # depending on plotmode (lpx already initialized for all cases).
	    if (i > 1) {

		if (LPL_MULTIPLE(lpl) == MULT_SERIES) {
		    # Advance a frame on the plotter.  Gset parameters will
		    # be automatically reset in lpl_glabax before any plotting.
		    call gclear (gd)
		}

		# Reset wcs & viewport regardless of plotmode (lpx already
		# initialized for all modes); send exact scale.
		call lpl_set_wv (lpx, gd, i)
		call lpl_set_exact (lpl, gd, LPX_PXSCALE(lpx,i-1),
		    LPX_PYSCALE(lpx,i-1))
	    }

	    # Draw labels and titles specific to this curve (not already drawn).
	    call lpl_glabax (lpl, lpx, gd, i, ncurves)

	    # Finally, actually draw the data curve itself.
	    call lpl_drawdata (lpl, lpx, gd, Memr[x[i]], Memr[y[i]], nelems[i],
	        i)
	    call gflush (gd)
	}

	# Free memory in the per-curve arrays.
	call lpl_free_lpx (lpx)
end
