include <error.h>
include "lpl.h"
include "lpx.h"

# LPL_RDCURVES -- Read all data curves for task longplot from input image
# sections and/or lists, producing lists of vectors as output.  Gather and
# store each curve's title here also, depending on title keywords.  Return
# number of curves as function value.

int procedure lpl_rdcurves (lpl, lpx, x, y, nelems)

pointer	lpl			# longplot descriptor
pointer	lpx			# per-curve arrays
pointer	x[MAX_CURVES]		# x data (nelems array points to indiv. curves)
pointer	y[MAX_CURVES]		# y data
int	nelems[MAX_CURVES]	# points per curve

pointer	sp, holdtitle
char	inlist_elem[SZ_FNAME]
int	ip, ncurves, i
bool	series, onetitle, needtitle

int	lpl_indirection(), lpl_rdcurve(), lpl_nextinput()
bool	streq(), strne()

errchk	lpl_indirection()

string	maxcrverr " (LPL_RDCURVES)--longplot has max no. of curves=       "

begin
	call smark (sp)
	call salloc (holdtitle, SZ_LINE*4, TY_CHAR)
	call strcpy (TITLE(LPL_TITLE(lpl), 0), Memc[holdtitle], SZ_LINE*4)

	# This is used heavily; convert to logical:
	series = (LPL_MULTIPLE(lpl) == MULT_SERIES)

	if (streq ("imtitle", Memc[holdtitle]) ||
	    streq ("file", Memc[holdtitle]) || streq ("*", Memc[holdtitle]))
	    onetitle = false
	else
	    onetitle = true

	ip = 1
	ncurves = 0

	# If the input specification is indirection we must use a special
	# routine to read the indirected file because it may contain up to 
	# 3 plot-specific appearance parameters for each image/file 
	# specification.  Only one level of indirection is allowed, and it 
	# must be the first input specification, and any other specification 
	# in the "input" parameter will be ignored.  

	if (INBUF(LPL_INBUF(lpl), 0) == '@')
	    ncurves = lpl_indirection (lpl, lpx, x, y, nelems)
	else {

	    # Read a list of images/listfiles (curves) into memory.
	    while (lpl_nextinput (INBUF(LPL_INBUF(lpl), 0), ip, inlist_elem,
		SZ_FNAME) > 0) {

		ncurves = ncurves + 1
		if (ncurves > MAX_CURVES) {
		    call sprintf (maxcrverr[49], 7, "%d")
			call pargi (MAX_CURVES)
		    call error (0, maxcrverr)
		}
		i = ncurves

		# Titlespec; note if longplot parameter "title" was specified,
		# and was not [imtitle|file|*] then we get the same title
		# regardless of which curve we are drawing for ncurves > 1;
		# otherwise, if stacking or overtracing, construct a single
		# multi-line title consisting of each input-specification.

		if (!series) {
		    needtitle = false
		    if (i == 1 && !onetitle)
			call strcpy (inlist_elem, Memc[holdtitle], SZ_LINE*4)
		    else if (!onetitle) {
			call strcat ('\n', Memc[holdtitle], SZ_LINE*4)
			call strcat (inlist_elem, Memc[holdtitle], SZ_LINE*4)
		    }

		} else if (!onetitle && strne ("none", Memc[holdtitle]))
		    needtitle = true

		else {
		    needtitle = false
		    if (streq ("none", Memc[holdtitle]))
			call lpl_assign_title (lpl, "", i)
		    else
			call lpl_assign_title (lpl, Memc[holdtitle], i)
		}

		# Put this here for now...
		LPX_MARKERTYPE(lpx,i-1) = LPL_MARKERTYPE(lpl)
		LPX_MARKERSIZE(lpx,i-1) = LPL_MARKERSIZE(lpl)

		iferr {
		    nelems[i] = lpl_rdcurve (lpl, inlist_elem, x[i],y[i],
			needtitle, Memc[holdtitle], i)
		} then {
		    call erract (EA_WARN)
		    ncurves = ncurves - 1
		}
	    }

	    if (!series)
		if (streq ("none", Memc[holdtitle]))
		    call lpl_assign_title (lpl, "", 1)
		else {
		    call lpl_invert (Memc[holdtitle])
		    call lpl_assign_title (lpl, Memc[holdtitle], 1)
		}
	}
	
	call sfree (sp)

	if (ncurves == 0)
	    call error (0, " (LPL_RDCURVES): No curves read")
	else
	    return (ncurves)
end
