include <error.h>
include <imhdr.h>
include "lpl.h"

# LPL_RIMAGE2 -- Read an image section and optionally compute the projection
# about one dimension, producing x and y vectors as output.

int procedure lpl_rimage2 (lpl, imspec, x, y, needtitle, holdtitle, curve)

pointer	lpl			# longplot descriptor
char	imspec[ARB]		# image file name [, section]
pointer	x, y			# pointers to a single curve's data arrays
bool	needtitle		# load title from imheader?
char	holdtitle[ARB]		# original titlespec from task parameter
int	curve			# curve number for title

int	axis, npix, i
pointer	im
real	c1, c2
int	func, transform, linear, coordt

pointer	immap()
bool	streq()

errchk	immap(), lpl_improjection(), malloc()	#, xt_mk1d()

begin

	im = immap (imspec, READ_ONLY, 0)

	if (needtitle)
	    if (streq ("file", holdtitle))
		call lpl_assign_title (lpl, imspec, curve)
	    else
		call lpl_assign_title (lpl, IM_TITLE(im), curve)

	axis = LPL_AXIS(lpl)
	if (axis < 1 || axis > IM_NDIM(im)) {
	    call eprintf ("Specified projection axis %d, image dimension %d\n")
		call pargi (axis)
		call pargi (IM_NDIM(im))
	    call error (2, "Attempt to take projection over nonexistent axis")
	}
	npix = IM_LEN(im,axis)

	call malloc (y, npix, TY_REAL)
	call lpl_improjection (im, Memr[y], npix, axis)

	iferr {
	    call malloc (x, npix, TY_REAL)
	} then
	    call erract (EA_FATAL)

	do i = 1, npix
	    Memr[x+i-1] = i

#Handle the coordinate transformation here.  (CTR)
#if (ctr_map (ctr1, im, 1, Memc[LPL_XTRSTR(lpl)]) == ERR)
#etc.

#Temporary, pending CTR package:  1-19-87, inserting old method of
#coordinate transformation.
	
	if (LPL_XTRAN(lpl) == YES) {
	    call lpl_parse_transform (Memc[LPL_XTRSTR(lpl)], c1, func, c2,
		transform, linear, coordt)
	    if (transform == YES)
		call lpl_transform (x, npix, c1, func, c2, coordt)
	}

	if (LPL_YTRAN(lpl) == YES) {
	    call lpl_parse_transform (Memc[LPL_YTRSTR(lpl)], c1, func, c2,
		transform, linear, coordt)
	    if (transform == YES)
		call lpl_transform (y, npix, c1, func, c2, coordt)
	}

	call imunmap (im)
	return (npix)
end
