include <error.h>
include <ctype.h>
include "lpl.h"

# LPL_RLIST2 -- Read a list of two dimensional data pairs (if first line
# contains only one argument, x will = 1.0 ... n, and y = x) into two type
# real arrays in memory.  Return (function) count of the number of pixels
# and pointers to the arrays.

int procedure lpl_rlist2 (lpl, fname, x, y, needtitle, curve)

pointer	lpl			# longplot descriptor
char	fname[ARB]		# input file name (we know it's not an image)
pointer	x, y			# pointers to an individual curve's data array
bool	needtitle		# need to load title from fname?
int	curve			# curve number

int	buflen, n, fd, ncols, lineno
pointer	sp, lbuf, ip
real	xval, yval
real	c1, c2				#<--***** temp until WIC/CTR cleanup
int	func, transform, linear, coordt	#<--***** temp until WIC/CTR cleanup

int	getline(), nscan(), open()
errchk	open, sscan, getline, malloc

begin
	call smark (sp)
	call salloc (lbuf, SZ_LINE, TY_CHAR)

	fd = open (fname, READ_ONLY, TEXT_FILE)

	# Load title from file name; this doesn't really need to be done down
	# in here, but it parallels rdimage2, which does.
	if (needtitle)
	    call lpl_assign_title (lpl, fname, curve)

	buflen = SZ_BUF
	iferr {
	    call malloc (x, buflen, TY_REAL)
	    call malloc (y, buflen, TY_REAL)
	} then
	    call erract (EA_FATAL)

	n = 0
	ncols = 0
	lineno = 0

	while (getline (fd, Memc[lbuf]) != EOF) {
	    # Skip comment lines and blank lines.
	    lineno = lineno + 1
	    if (Memc[lbuf] == '#')
		next
	    for (ip=lbuf;  IS_WHITE(Memc[ip]);  ip=ip+1)
		;
	    if (Memc[ip] == '\n' || Memc[ip] == EOS)
		next

	    # Decode the points to be plotted.
	    call sscan (Memc[ip])
		call gargr (xval)
		call gargr (yval)

	    # The first line determines whether we have an x,y list or a
	    # y-list.  It is an error if only one value can be decoded when
	    # processing a two column list.

	    if (ncols == 0 && nscan() > 0)
		ncols = nscan()
	    
	    switch (nscan()) {
	    case 0:
		call eprintf (
		    "WARNING (LPL_RLIST2): no args; %s, line %d: %s\n")
		    call pargstr (fname)
		    call pargi (lineno)
		    call pargstr (Memc[lbuf])
		next
	    case 1:
		if (ncols == 2) {
		    call eprintf (
			"WARNING (LPL_RLIST2): only 1 arg; %s, line %d: %s\n")
			call pargstr (fname)
			call pargi (lineno)
			call pargstr (Memc[lbuf])
		    next
		} else {
		    yval = xval
		    xval = n + 1.0
		}
	    }

	    n = n + 1
	    if (n > buflen) {
		buflen = buflen + SZ_BUF
		call realloc (x, buflen, TY_REAL)
		call realloc (y, buflen, TY_REAL)
	    }

	    Memr[x+n-1] = xval
	    Memr[y+n-1] = yval
	}

	call realloc (x, n, TY_REAL)
	call realloc (y, n, TY_REAL)

#	# Perform coordinate transformation:
#	if (LPL_XTRAN(lpl) == YES)
#	    call lpl_ltrans (Memc[LPL_XTRSTR(lpl)], n, Memr[x])
#	if (LPL_YTRAN(lpl) == YES)
#	    call lpl_ltrans (Memc[LPL_YTRSTR(lpl)], n, Memr[y])

#*****--> temporary until WIC/CTR cleanup (currently [1-19-87] still using
#	  old coord transform technique).

	if (LPL_XTRAN(lpl) == YES) {
	    call lpl_parse_transform (Memc[LPL_XTRSTR(lpl)], c1, func, c2,
		transform, linear, coordt)
	    if (transform == YES)
		call lpl_transform (x, n, c1, func, c2, coordt)
	}

	if (LPL_YTRAN(lpl) == YES) {
	    call lpl_parse_transform (Memc[LPL_YTRSTR(lpl)], c1, func, c2,
		transform, linear, coordt)
	    if (transform == YES)
		call lpl_transform (y, n, c1, func, c2, coordt)
	}

	call close (fd)
	call sfree (sp)
	return (n)
end
