include	<gki.h>
include <gset.h>
include "lpl.h"

# LPL_SET_PLOTP -- Set internal plotter ranges.

procedure lpl_set_plotp (lpl, gd)

pointer	lpl			# longplot descriptor
pointer	gd			# graphics device descriptor

real	csize
real	gstatr()

#debug+
int	debug
common /deb/ debug
#debug-
begin
	# Set plotter limits [and scale] for x-axis.
	if (LPL_XSCALE_INDEF(lpl) == NO) {			# Scaling
	    # Here we are guaranteed to have only 0 or 1 of pxmin,pxmax set.
	    if (LPL_PXMIN_INDEF(lpl) == YES && LPL_PXMAX_INDEF(lpl) == YES)
		LPL_PXMIN(lpl) = DEF_PXMIN * LPL_UNIT_SCALE(lpl)

	    if (LPL_PXMAX_INDEF(lpl) == YES)
		LPL_PXMAX(lpl) = LPL_PXMIN(lpl) + 
		    (LPL_WXMAX(lpl) - LPL_WXMIN(lpl)) / LPL_XSCALE(lpl)
	    else if (LPL_PXMIN_INDEF(lpl) == YES)
		LPL_PXMIN(lpl) = min (DEF_PXMIN * LPL_UNIT_SCALE(lpl),
		    LPL_PXMAX(lpl) - (LPL_WXMAX(lpl) - LPL_WXMIN(lpl)) /
		    LPL_XSCALE(lpl))

	} else {						# Not scaling
	    if (LPL_PXMIN_INDEF(lpl) == YES)
		LPL_PXMIN(lpl) = DEF_PXMIN * LPL_UNIT_SCALE(lpl)
	    if (LPL_PXMAX_INDEF(lpl) == YES)
		LPL_PXMAX(lpl) = DEF_PXMAX * LPL_UNIT_SCALE(lpl)
	    LPL_XSCALE(lpl) = (LPL_WXMAX(lpl) - LPL_WXMIN(lpl)) /
		(LPL_PXMAX(lpl) - LPL_PXMIN(lpl))
	}

	# Set plotter limits [and scale] for y-axis.
	if (LPL_YSCALE_INDEF(lpl) == NO) {			# Scaling
	    # Here we are guaranteed to have only 0 or 1 of pymin,pymax set.
	    if (LPL_PYMIN_INDEF(lpl) == YES && LPL_PYMAX_INDEF(lpl) == YES)
		LPL_PYMIN(lpl) = DEF_PYMIN * LPL_UNIT_SCALE(lpl)

	    if (LPL_PYMAX_INDEF(lpl) == YES)
		LPL_PYMAX(lpl) = LPL_PYMIN(lpl) + 
		    (LPL_WYMAX(lpl) - LPL_WYMIN(lpl)) / LPL_YSCALE(lpl)
	    else if (LPL_PYMIN_INDEF(lpl) == YES)
		LPL_PYMIN(lpl) = min (DEF_PYMIN * LPL_UNIT_SCALE(lpl),
		    LPL_PYMAX(lpl) - (LPL_WYMAX(lpl) - LPL_WYMIN(lpl)) /
		    LPL_YSCALE(lpl))

	} else {						# Not scaling
	    if (LPL_PYMIN_INDEF(lpl) == YES)
		LPL_PYMIN(lpl) = DEF_PYMIN * LPL_UNIT_SCALE(lpl)
	    if (LPL_PYMAX_INDEF(lpl) == YES)
		LPL_PYMAX(lpl) = DEF_PYMAX * LPL_UNIT_SCALE(lpl)
	    LPL_YSCALE(lpl) = (LPL_WYMAX(lpl) - LPL_WYMIN(lpl)) / 
		(LPL_PYMAX(lpl) - LPL_PYMIN(lpl))
	}

	# Scale character sizes to plot height if requested, now that we have
	# the final values for pymin, pymax.

	csize = gstatr (gd, G_TXSIZE)
	if (LPL_FLOATCHARSIZE(lpl) == YES && !IS_INDEF(LPL_PYMIN(lpl)) &&
	    !IS_INDEF(LPL_PYMAX(lpl))) {
	    csize = csize * (LPL_PYMAX(lpl) - LPL_PYMIN(lpl)) /
	        (DEF_PYMAX - DEF_PYMIN)
	    call gsetr (gd, G_TXSIZE, csize)
	    call gsetr (gd, G_TITLESIZE, csize)
	    call gsetr (gd, G_AXISLABELSIZE, csize)
	    call gsetr (gd, G_TICKLABELSIZE, csize)
	}

#debug+
if (debug == YES) {
call eprintf ("set_plotp:\n")
call eprintf ("          lpl_pxmin(lpl)=%g, pxmax=%g, pymin=%g, pymax=%g\n")
call pargr (LPL_PXMIN(lpl)); call pargr (LPL_PXMAX(lpl))
call pargr (LPL_PYMIN(lpl)); call pargr (LPL_PYMAX(lpl))
call eprintf ("          lpl_xscale(lpl)=%7.4g, yscale=%7.4g, CSIZE=%9.4f\n")
call pargr (LPL_XSCALE(lpl)); call pargr (LPL_YSCALE(lpl)); call pargr (csize)
}
#debug-

end
