include	<gki.h>
include "lpl.h"
include "lpx.h"

# LPL_SET_RANGES -- Set wcs (gio) and internal world data ranges.

procedure lpl_set_ranges (lpl, lpx, gd, x, y, nelems, ncurves)

pointer	lpl			# longplot descriptor
pointer	lpx			# lpx descriptor; per-curve dynamic arrays
pointer	gd			# graphics device descriptor
pointer	x[MAX_CURVES]		# x data for all curves
pointer y[MAX_CURVES]		# y data for all curves
int	nelems[MAX_CURVES]	# Number of points in each curve
int	ncurves			# Number of curves

int	i, j
real	wxmin,wxmax, wymin,wymax, junkr
bool	series, overtrace

#debug+
int	debug
common /deb/ debug
#debug-
begin
	series = (LPL_MULTIPLE(lpl) == MULT_SERIES)
	overtrace = (LPL_MULTIPLE(lpl) == MULT_OVERTRACE)

	# Set wcs, related internal variables for x-axis.
	if (LPL_XAUTOEXPAND(lpl) == YES) {

	    # Lpl_coords() guaranteed that only 0 or 1 of wxmin,wxmax are set.  
	    # Determine limits of data; these may be reset later in lpl_set_wv
	    # as calculated by lpl_calc_coords if rounding is in effect.

	    call gascale (gd, Memr[x[1]], nelems[1], XCOORD)
	    call ggwind (gd, wxmin,wxmax, junkr,junkr)

	    # (Retain either limit if specified by user)
	    if (LPL_WXMIN_INDEF(lpl) == YES)
		LPX_WXMIN(lpx,0) = wxmin
	    else
		LPX_WXMIN(lpx,0) = LPL_WXMIN(lpl)

	    if (LPL_WXMAX_INDEF(lpl) == YES)
		LPX_WXMAX(lpx,0) = wxmax
	    else
		LPX_WXMAX(lpx,0) = LPL_WXMAX(lpl)

	    do i = 2, ncurves {
		if (!series)
		    call grscale (gd, Memr[x[i]], nelems[i], XCOORD)
		else
		    call gascale (gd, Memr[x[i]], nelems[i], XCOORD)

		call ggwind (gd, wxmin,wxmax, junkr,junkr)

		# (Retain either limit if specified by user)
		if (LPL_WXMIN_INDEF(lpl) == YES)
		    if (!series)
			do j = 1, i
			    LPX_WXMIN(lpx,j-1) = wxmin
		    else
			LPX_WXMIN(lpx,i-1) = wxmin
		else
		    LPX_WXMIN(lpx,i-1) = LPL_WXMIN(lpl)

		if (LPL_WXMAX_INDEF(lpl) == YES)
		    if (!series)
			do j = 1, i
			    LPX_WXMAX(lpx,j-1) = wxmax
		    else
			LPX_WXMAX(lpx,i-1) = wxmax
		else
		    LPX_WXMAX(lpx,i-1) = LPL_WXMAX(lpl)
	    }

	} else {
	    # Not autoexpanding.  Lpl_coords guaranteed wxmin,wxmax specified.
	    do i = 1, ncurves {
		LPX_WXMIN(lpx,i-1) = LPL_WXMIN(lpl)
		LPX_WXMAX(lpx,i-1) = LPL_WXMAX(lpl)
	    }
	}
	
	# Set wcs, related internal variables for y-axis.
	if (LPL_YAUTOEXPAND(lpl) == YES) {

	    # Lpl_coords() guaranteed that only 0 or 1 of wymin,wymax are set.  
	    # Determine limits of data; these may be reset later in lpl_set_wv
	    # as calculated by lpl_calc_coords if rounding is in effect.

	    call gascale (gd, Memr[y[1]], nelems[1], YCOORD)
	    call ggwind (gd, junkr,junkr, wymin,wymax)

	    # (Retain either limit if specified by user)
	    if (LPL_WYMIN_INDEF(lpl) == YES)
		LPX_WYMIN(lpx,0) = wymin
	    else
		LPX_WYMIN(lpx,0) = LPL_WYMIN(lpl)

	    if (LPL_WYMAX_INDEF(lpl) == YES)
		LPX_WYMAX(lpx,0) = wymax
	    else
		LPX_WYMAX(lpx,0) = LPL_WYMAX(lpl)

	    do i = 2, ncurves {
		if (overtrace)
		    call grscale (gd, Memr[y[i]], nelems[i], YCOORD)
		else
		    call gascale (gd, Memr[y[i]], nelems[i], YCOORD)

		call ggwind (gd, junkr,junkr, wymin,wymax)

		# (Retain either limit if specified by user)
		if (LPL_WYMIN_INDEF(lpl) == YES)
		    if (overtrace)
			do j = 1, i
			    LPX_WYMIN(lpx,j-1) = wymin
		    else
			LPX_WYMIN(lpx,i-1) = wymin
		else
		    LPX_WYMIN(lpx,i-1) = LPL_WYMIN(lpl)

		if (LPL_WYMAX_INDEF(lpl) == YES)
		    if (overtrace)
			do j = 1, i
			    LPX_WYMAX(lpx,j-1) = wymax
		    else
			LPX_WYMAX(lpx,i-1) = wymax
		else
		    LPX_WYMAX(lpx,i-1) = LPL_WYMAX(lpl)
	    }

	} else {
	    # Not autoexpanding.  Lpl_coords guaranteed wymin,wymax specified.
	    do i = 1, ncurves {
		LPX_WYMIN(lpx,i-1) = LPL_WYMIN(lpl)
		LPX_WYMAX(lpx,i-1) = LPL_WYMAX(lpl)
	    }
	}
	
#debug+
if (debug == YES) {
call eprintf ("set_ranges:\n")
do i = 1, ncurves {
call eprintf ("           wxmin(%d)=%g, wxmax=%g, wymin=%g, wymax=%g\n")
call pargi (i)
call pargr (LPX_WXMIN(lpx,i-1))
call pargr (LPX_WXMAX(lpx,i-1))
call pargr (LPX_WYMIN(lpx,i-1))
call pargr (LPX_WYMAX(lpx,i-1))
}
}
#debug-
end
