include	<math.h>
include	"lpl.h"

# LPL_TRANSFORM -- Transform input array to "new = c1 * func (old) ** c2"; or,
# if of form "p1=x1 p2=x2", to simple linear coordinate transform.

procedure lpl_transform (v, nelems, c1, func, c2, coordt)

pointer	v			# input array pointer
int	nelems			# number of elements in each curve
real	c1			# constant 1 (multiplier)
int	func			# transformation function
real	c2			# constant 2 (power)
int	coordt			# is transformation of form "p1=x1 p2=x2"?

int	i
real	factor
real	mkindefr()
extern	mkindefr()

include "functions.h"

#debug+
int	debug
common /deb/ debug
#debug-
begin
	# If transform is of form "p1=x1 p2=x2", map the input vector into
	# an output array where element 1 is c1, element n is c2.

	if (coordt == YES) {
	    if (nelems == 1)
		Memr[v] = c1
	    else {
		factor = (c2 - c1) / float (nelems - 1)
		do i = 1, nelems
		    Memr[v+i-1] = (Memr[v+i-1] - 1.0) * factor + c1
	    }

	# Else, transform each element of the input vector according to 
	# vout = c1 * func (vin) ** c2.
	} else {
	    switch (func) {
	    case F_ABS:
		call aabsr (Memr[v], Memr[v], nelems)
	    case F_ACOS:
		do i = 1, nelems
		    Memr[v+i-1] = RADTODEG (acos (Memr[v+i-1]))
	    case F_ASIN:
		do i = 1, nelems
		    Memr[v+i-1] = RADTODEG (asin (Memr[v+i-1]))
	    case F_ATAN:
		do i = 1, nelems
		    if (Memr[v+i-1] == 0.0)
			Memr[v+i-1] = INDEFR
		    else
			Memr[v+i-1] = RADTODEG (atan (Memr[v+i-1]))
	    case F_COS:
		do i = 1, nelems
		    Memr[v+i-1] = cos (DEGTORAD (Memr[v+i-1]))
	    case F_LN:
		call allnr (Memr[v], Memr[v], nelems, mkindefr)
	    case F_LOG, F_LOG10:
		call alogr (Memr[v], Memr[v], nelems, mkindefr)
	    case F_NINT:
		do i = 1, nelems
		    Memr[v+i-1] = real (nint (Memr[v+i-1]))
	    case F_SIN:
		do i = 1, nelems
		    Memr[v+i-1] = sin (DEGTORAD (Memr[v+i-1]))
	    case F_SQRT:
		call asqrr (Memr[v], Memr[v], mkindefr)
	    case F_TAN:
		do i = 1, nelems
		    Memr[v+1-1] = tan (DEGTORAD (Memr[v+i-1]))
	    }

	    # Exponent of result of requested function
	    if (c2 != 1.0 && c2 != INDEFR)
		call aexpkr (Memr[v], c2, Memr[v])

	    # Multiplier of whole expression
	    if (c1 != 1.0 && c1 != INDEFR)
		call amulkr (Memr[v], c1, Memr[v])

	}

# If any of the "log" transforms apply in the sense that we'll only use
# GIO plotting for the logs, then must set 
#	if (xneedlog) --> TR_XTRAN(tr) = LOG
# here.  Because of wcs extension, it is not possible at this point to
# determine whether it will be LOG or ELOG.

end

# MKINDEFR -- return INDEFR; used as an external reference passed to some
# of the vector op routines for impossible operations.

real procedure mkindefr (r)

real	r

begin
	return (INDEFR)
end
