include	<mach.h>
include	<gio.h>
include	"lpl.h"

# LPL_WCS_EXTEND --  Compute tick sizes and round the WCS out to the next
# tick outside the boundary on either end of the axis.  The job is to
# position the major ticks in world coordinates at round numbers, e.g.,
# 10, 20, 30 for a linear scale or 10, 100, 1000 for a log scale.  
# If a scale was specified, we force the step between ticks, and therefore
# the boundary extension, to equal it; otherwise, we use the requested
# number of major ticks to determine the step size.

procedure lpl_wcs_extend (scaling, nmajor, wstep, wmin, wmax)

int	scaling			# graphics output transform? (e.g. LOG)
int	nmajor			# number of major ticks or use-scale flag
real	wstep			# user-requested world scale (= step size)
real	wmin,wmax		# axis limits in world units

int	logflag 
real	p1,p2, tick1, step, length, tp1, tp2
bool	fp_equalr()
real	elogr(), aelogr()

#debug+
int	debug
common /deb/ debug
#debug-
begin
	# Call the tick placement algorithm to determine where to put the major
	# tick marks.  The output of this block are the variables P1 and P2,
	# the world coords of the ends of the axis, and TICK1 and STEP, the
	# world coords of the first tick and separation in world coords between
	# major ticks.

	p1 = wmin
	p2 = wmax

	if (scaling == LOG)
	    if (p1 <= 0.0 || p2 <= 0.0)
		scaling = ELOG
	if (scaling == LOG) {
	    p1 = log10 (p1)
	    p2 = log10 (p2)
	    logflag = YES
	} else if (scaling == ELOG) {
	    p1 = elogr (p1)
	    p2 = elogr (p2)
	    logflag = YES
	} else
	    logflag = NO

#debug+
if (debug == YES) {
call eprintf ("lpl_wcs_extend:	nmajor %d, logflag %d, scaling %d\n")
call pargi (nmajor)
call pargi (logflag)
call pargi (scaling)
}
#debug-
	# If a world scale was specified, and the user wants a tick every
	# scale units, we force the step size to equal it.
	if (nmajor < 0) {
	    step = wstep
	    tick1 = int (p1 / step) * step + step
	    if (logflag == YES) {
		step = log10 (step)
		tick1 = log10 (tick1)
	    }
	} else
	    call gtickr (p1, p2, nmajor, logflag, tick1, step)
#debug+
if (debug == YES) {
call eprintf ("		p1 %7.4f, p2 %7.4f, tick1 %7.4f, step %7.4f\n")
call pargr (p1)
call pargr (p2)
call pargr (tick1)
call pargr (step)
}
#debug-

	if (!fp_equalr (p1, tick1)) {
	    tick1 = tick1 - step
	    p1 = tick1
	}

	length = (p2 - p1) / step
	if (!fp_equalr (p1 + int(length) * step, p2)) {
	    p2 = p1 + (int(length) + 1) * step
#debug+
if (debug == YES) {
call eprintf ("		p1 %7.4f, p2 %7.4f, tick1 %7.4f, length %7.4f\n")
call pargr (p1)
call pargr (p2)
call pargr (tick1)
call pargr (length)
}
#debug-
}

	if (scaling == ELOG) {
	    tp1 = aelogr (p1)
	    tp2 = aelogr (p2)
	} else if (scaling == LOG) {
	    tp1 = 10.0 ** p1
	    tp2 = 10.0 ** p2
	} else {
	    tp1 = p1
	    tp2 = p2
	}

	wmin = tp1
	wmax = tp2
end
