include <ctype.h>
include <fset.h>
include "lpl.h"

# LONGPLOT -- Produce "long" x-y plots to an exact scale.
# Input may be an image/file template, where file means a one-or-2d list.
# Multidimensional image sections are reduced to a vector by computing
# the projection about the indicated axis.  See the manual page for a
# description of the many options for personalizing the plot. 

procedure t_longplot()

char	device[SZ_FNAME], spoolfile[SZ_FNAME]
pointer	lpl, gd, x[MAX_CURVES], y[MAX_CURVES]
int	nelems[MAX_CURVES], i, g_stream, ip, spool

pointer	gopen()
int	fstati(), open(), strlen()

errchk	lpl_coords(), lpl_plot()

#debug+
int	debug
bool	clgetb()
common /deb/ debug
#debug-

begin

	# Allocate the longplot descriptor and input template buffer.
	call calloc (lpl, LEN_LPL, TY_STRUCT)
	call malloc (LPL_INBUF(lpl), SZ_LINE, TY_CHAR)

	# Initialize data array pointers and title buffer to NULL in order
	# to mfree successfully if lpl_plot aborts without allocating them.
	do i = 1, MAX_CURVES {
	    x[i] = NULL
	    y[i] = NULL
	    nelems[i] = NULL
	}
	LPL_TITLE(lpl) = NULL
	LPL_TITLENDX(lpl) = NULL
	LPL_XLABEL(lpl) = NULL
	LPL_YLABEL(lpl) = NULL

	# Get input template; if standard input is redirected, load dummy
	# input buffer containing the text file "STDIN".

	if (fstati (STDIN, F_REDIR) == YES)
	    call strcpy ("STDIN", INBUF(LPL_INBUF(lpl), 0), SZ_LINE)
	else
	    call clgstr ("input", INBUF(LPL_INBUF(lpl), 0), SZ_LINE)

	# Open graphics device on standard plot stream, or, if spoolfile is
	# specified, on a new binary output metacode file.

	call clgstr ("device", device, SZ_FNAME)
	call clgstr ("spoolfile", spoolfile, SZ_FNAME)
#debug+
if (clgetb ("debug"))
    debug = YES
else
    debug = NO
#debug-

	for (ip=1; IS_WHITE(spoolfile[ip]); ip=ip+1)
	    ;

	if (strlen (spoolfile[ip]) <= 0) {
	    g_stream = STDPLOT
	    spool = NO
	} else {
	    g_stream = open (spoolfile, NEW_FILE, BINARY_FILE)
	    spool = YES
	}

	gd = gopen (device, NEW_FILE, g_stream)

	# Get all data/axis related parameters and set in lpl descriptor.
	iferr (call lpl_coords (lpl))
	    call fatal (0, "LONGPLOT:  task aborted")

	# Get appearance-related parameters not embedded in input indirection.
	call lpl_appearance (lpl)

	# Initialize static parameters describing the initial state of the
	# 10 world & plotter ranges and the specified world/plotter scales.
	call lpl_init_original (lpl)

	# Draw the plot for all input sets.
	iferr (call lpl_plot (lpl, gd, x,y,nelems))
	    call fatal (0, "LONGPLOT:  task aborted")

#debug+
if (debug == YES) 
call eprintf ("--> in t_longplot.x return from lpl_plot ")

	# Return data, input buffer, and descriptor space.
	do i = 1, MAX_CURVES {
	    call mfree (x[i], TY_REAL)
	    call mfree (y[i], TY_REAL)
	} 
	call mfree (LPL_INBUF(lpl), TY_CHAR)
	call mfree (LPL_TITLE(lpl), TY_CHAR)
	call mfree (LPL_TITLENDX(lpl), TY_INT)
	call mfree (LPL_XLABEL(lpl), TY_CHAR)
	call mfree (LPL_XTRSTR(lpl), TY_CHAR)
	call mfree (LPL_YLABEL(lpl), TY_CHAR)
	call mfree (LPL_YTRSTR(lpl), TY_CHAR)
	call mfree (lpl, TY_STRUCT)

	# Close graphics device and stream.
	call gclose (gd)
	if (spool == YES)
		call close (g_stream)
end
