include <imhdr.h>
include <mach.h>
include "rccd.h"

# RCD_READ_IMAGE -- Read CCD image pixels to IRAF image file

procedure rcd_read_image (ccd_fd, im)

int	ccd_fd		# CCD file descriptor
pointer im		# IRAF image descriptor

int 	i, npix, npix_record, mxoff, bbxo, incr, yend, byend, row, bias 
long	v[IM_MAXDIM] 
pointer tempbuf, buf, fixed_buf
real	linemax, linemin, rbias
char	tstring[40]

long 	clktime()
int	rcd_init_read_pixels(), rcd_read_pixels(), rcd_cal_bias()
errchk	malloc, mfree, rcd_init_read_pixels, rcd_read_pixels
errchk  rcd_put_image_line, rcd_pix_limits, rcd_change_pix
data	tempbuf /NULL/, fixed_buf /NULL/

include "rccd.com"

begin

	# intialize
	call rcd_set_image_header (im)
	bias = 0

	# Calculate effective image and bias offsets and loop increment
	if (flip < 1) {  # If the data has not been mirrorred, read from
			 # the right
	    if (nostrip == 1)  # No strip
		mxoff = dim1
	    else mxoff = xoff + NAXISN(im, 1)
	    incr = -1
	    bbxo = bxo + 1
	}
	else {
		if (nostrip == 1)
		    mxoff = 0
		else mxoff = dim2 - xoff - NAXISN(im, 1) + 1
		incr = 1
		bbxo = dim1 - bxo - bxsize + 1
	}
	yend = yoff + NAXISN(im, 2) - 1
	byend = byo + bysize - 1 

	IRAFMAX(im) = -MAX_REAL
	IRAFMIN(im) = MAX_REAL

	npix_record = dim1 * lpr 
	npix = dim1 

	if (tempbuf != NULL)
	    call mfree (tempbuf, TY_LONG)
	call malloc (tempbuf, npix, TY_LONG)

	if (fixed_buf != NULL)
	    call mfree (fixed_buf, TY_LONG)
	call malloc (fixed_buf, NAXISN(im, 1), TY_LONG)

	i = rcd_init_read_pixels (npix_record, CCD_SHORT, LSBF, TY_LONG)
	call amovkl (long(1), v, IM_MAXDIM)

	row = 0

	repeat {
		# Read in image line
		if (rcd_read_pixels (ccd_fd, Meml[tempbuf], npix) != npix)
		    call error (4, "RCD_READ_IMAGE: Error reading CCD data")

		# Fix data for output
		if (row >= yoff && row <= yend) {

		    # Write next image line
		    call rcd_put_image_line (im, buf, v, PIXTYPE(im))

		    call rcd_fix_pix (Meml[tempbuf], Meml[fixed_buf], mxoff, 
				      NAXISN(im, 1), incr)
		    call rcd_change_pix (Meml[fixed_buf], buf, NAXISN(im, 1),
					 PIXTYPE(im))

	  	    # Calculate image maximum and minimum
		    call rcd_pix_limits (buf, NAXISN(im, 1), PIXTYPE(im),
		  	                 linemin, linemax)
	   	    IRAFMAX(im) = max (IRAFMAX(im), linemax)
	   	    IRAFMIN(im) = min (IRAFMIN(im), linemin)
		 }

		if (row >= byo && row <= byend) 
		    bias = bias + rcd_cal_bias (Meml[tempbuf], bbxo, bxsize)

		row = row + 1
	} until (row == dim2)

	rbias = bias
	rbias = rbias/bysize/bxsize

	call printf("Extended Register Value = %.3f\n")
	call pargr(rbias)
	call sprintf(tstring, 40, "EXTENDED-REG= %.3f\n") 
		call pargr(rbias)
	call strcat (tstring, UNKNOWN(im), (LEN_USER_AREA - 1) * SZ_HISTORY)

	LIMTIME(im) = clktime (long(0))

end


# RCD_SET_IMAGE_HEADER -- Set remaining header fields not set in
# rcd_read_header.

procedure rcd_set_image_header (im)

pointer im		#IRAF image pointer

int	precision
include "rccd.com"

begin
	# Set data_type to short if user data type not specified.

	if (data_type == ERR) {
	    precision = CCDS_PREC
	    PIXTYPE(im) = TY_SHORT
	}
	else PIXTYPE(im) = data_type
end

# RCD_FIX_PIX -- Procedure to convert an IRAF image line from type short
# to the requested output data type with optional mirror option.

procedure rcd_fix_pix (inbuf, outbuf, offset, size, incr) 

long inbuf[ARB], outbuf[ARB]
int offset, size, incr

int i
int os

begin
	os = offset
	for ( i = 1; i <= size; i = i + 1) {
	      if ( inbuf[os] < 0 )
		   outbuf[i] = 65536 + inbuf[os]
	      else
	      	   outbuf[i] = inbuf[os]
	      os = os + incr
	}	
end


# RCD_CAL_BIAS --

int procedure rcd_cal_bias (inbuf, offset, bxsize)

long inbuf[ARB]
int offset, bxsize, 

int i, tbias, os

begin
	os = offset - 1
	tbias = 0
	do i = 1, bxsize
		tbias = tbias + inbuf[os + i]
        return (tbias)
end


# RCD_CCD_PUT_IMAGE_LINE -- Procedure to output an image line to and IRAF FILE

procedure rcd_put_image_line(im, buf, v, data_type)

pointer im		# IRAF image descriptor
pointer buf		# Pointer to output image line
long    v[ARB]	      	# imio pointer
int	data_type	# output pixel type

int	impnll(), impnlr(), impnld(), impnlx()
errchk	impnll, impnlr, impnld, impnlx

begin
	switch (data_type) {
	case TY_SHORT, TY_INT, TY_USHORT, TY_LONG:
	    if (impnll (im, buf, v) == EOF)
		call error (3, "RCD_PUT_IMAGE_LINE: Error writing CCD data")
	case TY_REAL:
	    if (impnlr (im, buf, v) == EOF)
		call error (3, "RCD_PUT_IMAGE_LINE: Error writing CCD data")
	case TY_DOUBLE:
	    if (impnld (im, buf, v) == EOF)
		call error (3, "RCD_PUT_IMAGE_LINE: Error writing CCD data")
	case TY_COMPLEX:
	    if (impnlx (im, buf, v) == EOF)
		call error (3, "RCD_PUT_IMAGE_LINE: Error writing CCD data")
	default:
	    call error (10, "RCD_PUT_IMAGE_LINE: Unsupported IRAF image type")
	}
end


# RCD_CHANGE_PIX -- Procedure to change a line of long integers to the
# IRAF image type.

procedure rcd_change_pix (inbuf, outbuf, npix, data_type)

long inbuf[ARB]		# array of FITS integers
pointer	outbuf			# pointer to IRAF image line
int	npix			# number of pixels
int	data_type		# IRAF pixel type

begin
	switch (data_type) {
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
	    call achtll (inbuf, Meml[outbuf], npix)
	case TY_REAL:
	    call achtlr (inbuf, Memr[outbuf], npix)
	case TY_DOUBLE:
	    call achtld (inbuf, Memd[outbuf], npix)
	case TY_COMPLEX:
	    call achtlx (inbuf, Memx[outbuf], npix)
	default:
	    call error (10, "RCD_CHANGE_LINE: Illegal IRAF image type")
	}
end


# RFT_PIX_LIMITS -- Procedure to determine to maxmimum and minimum values in a
# line.

procedure rcd_pix_limits (buf, npix, pixtype, linemin, linemax)

pointer	buf				# pointer to IRAF image line
int	npix				# number of pixels
int	pixtype				# output data type
real	linemax, linemin		# min and max pixel values

short   lmax, lmin
real	rmax, rmin
double	dmax, dmin
complex	xmax, xmin

begin
	switch (pixtype) {
	case TY_SHORT, TY_INT, TY_USHORT, TY_LONG:
	    call aliml (Meml[buf], npix, lmin, lmax)
	    linemax = lmax
	    linemin = lmin
	case TY_REAL:
	    call alimr (Memr[buf], npix, rmin, rmax)
	    linemax = rmax
	    linemin = rmin
	case TY_DOUBLE:
	    call alimd (Memd[buf], npix, dmin, dmax)
	    linemax = dmax
	    linemin = dmin
	case TY_COMPLEX:
	    call alimx (Memx[buf], npix, xmin, xmax)
	    linemax = xmax
	    linemin = xmin
	default:
	    call error (30, "RCD_PIX_LIMITS: Unknown IRAF type")
	}

end
