procedure quad (raw, mosaic)

string	raw		{prompt="Raw multiplexed image"}
string	mosaic		{prompt="Dequaded mosaic"}

int	bias1 = 1500	{prompt="Rough bias offset for chip 1"}
int	bias2 =  880	{prompt="Rough bias offset for chip 2"}
int	bias3 =  430	{prompt="Rough bias offset for chip 3"}
int	bias4 =  660	{prompt="Rough bias offset for chip 4"}

int	xgap = 40	{prompt="X pixel gap", min=0}
int	ygap = 40	{prompt="Y pixel gap", min=0}

int	blank = 0	{prompt="Pixel value for gaps"}

bool	verbose = yes	{prompt="Verbose commentary"}

begin
	string	lraw, lmos, rs[8], ms[8], tmp
	int	quad, nx, ny, nx1, ny1, xs[8], xe[8], ys[8], ye[8], bias[8]

	lraw = raw
	lmos = mosaic

	tmp = mktemp ("tmp$mos")

	hselect (lraw, "naxis1,naxis2", "yes") | scan (nx, ny)
	if (mod(nx,4) != 0 || mod(ny,4) != 0)
	    print ("WARNING: input image axes aren't divisible by 4!")

	rs[1] = lraw // "[65:"//nx//":4,1:"//ny//":2]"
	rs[2] = lraw // "[1:"//nx-64//":4,2:"//ny//":2]"
	rs[3] = lraw // "[2:"//nx//":4,1:"//ny//":2]"
	rs[4] = lraw // "[2:"//nx-128//":4,2:"//ny//":2]"
	rs[5] = lraw // "[3:"//nx//":4,1:"//ny//":2]"
	rs[6] = lraw // "[3:"//nx-128//":4,2:"//ny//":2]"
	rs[7] = lraw // "[68:"//nx//":4,1:"//ny//":2]"
	rs[8] = lraw // "[4:"//nx-64//":4,2:"//ny//":2]"

	nx1 = nx/2 - 32
	ny1 = ny/2

    xs[1]=1; xe[1]=nx1/2; ye[1]=ny1+ygap+1; ys[1]=2*ny1+ygap
    xs[2]=nx1/2+1; xe[2]=nx1; ye[2]=ny1+ygap+1; ys[2]=2*ny1+ygap
    xe[4]=1; xs[4]=nx1/2; ys[4]=1; ye[4]=ny1
    xe[3]=nx1/2+1; xs[3]=nx1; ys[3]=1; ye[3]=ny1
    xs[5]=nx1+xgap+1; xe[5]=3*nx1/2+xgap; ye[5]=ny1+ygap+1; ys[5]=2*ny1+ygap
    xs[6]=3*nx1/2+xgap+1; xe[6]=2*nx1+xgap; ye[6]=ny1+ygap+1; ys[6]=2*ny1+ygap
    xe[8]=nx1+xgap+1; xs[8]=3*nx1/2+xgap; ys[8]=1; ye[8]=ny1
    xe[7]=3*nx1/2+xgap+1; xs[7]=2*nx1+xgap; ys[7]=1; ye[7]=ny1

	ms = lmos // "["// xs//":"//xe //","// ys//":"//ye //"]"

	bias[1] = bias1; bias[2] = bias1
	bias[3] = bias2; bias[4] = bias2
	bias[5] = bias3; bias[6] = bias3
	bias[7] = bias4; bias[8] = bias4

	nx = nx + xgap - 64
	ny = ny + ygap

	if (verbose) {
	    printf ("\nDequading %s:\n", lraw)
	    printf ("  ")
	    time
	    printf ("  ")
	    $mkpattern (lmos, pattern="constant", option="replace", v1=blank,
		pixtype="short", ndim=2, ncols=nx, nlines=ny, header=lraw)
	    printf ("\n")
	} else {
	    mkpattern (lmos, pattern="constant", option="replace", v1=blank,
		pixtype="short", ndim=2, ncols=nx, nlines=ny, header=lraw)
	}

	for (i=1; i<=8; i+=1) {
	    if (verbose) {
		quad = (i+1)/2
		printf ("Trimming and bias correcting quadrant %d", quad)
		if (mod(i,2) == 1)
		    printf ("A:\n")
		else
		    printf ("B:\n")

		printf ("  ")
		$imarith (rs[i], "-", bias[i], tmp,
		    pixtype="short", calctype="short", verb-)

		printf ("  ")
		$imcopy (tmp, ms[i], verb-)

		if (quad == 2 && mod(i,2) == 0) {
		    printf ("Transposing quadrant 2:\n")
		    imdelete (tmp, ver-, >& "dev$null")
		    printf ("  ")
		    $imtranspose (lmos//"[1:"//nx1//",1:"//ny1//"]", tmp)
		    printf ("  ")
		    $imcopy (tmp, lmos//"[1:"//nx1//",1:"//ny1//"]", verb-)
		}

		if (mod(i,2) == 0)
		    printf ("\n")

	    } else {
		imarith (rs[i], "-", bias[i], tmp,
		    pixtype="short", calctype="short", verb-)

		imcopy (tmp, ms[i], verb-)

		if (quad == 2 && mod(i,2) == 0) {
		    imdelete (tmp, ver-, >& "dev$null")
		    imtranspose (lmos//"[1:"//nx1//",1:"//ny1//"]", tmp)
		    imcopy (tmp, lmos//"[1:"//nx1//",1:"//ny1//"]", verb-)
		}

	    }

	    imdelete (tmp, ver-, >& "dev$null")
	}

	if (verbose) {
	    printf ("  ")
	    time
	}
end
