include	<error.h>
include	<ctotok.h>
 
 
# T_LCODE - Print statistics for SPP and Fortran programs.
 
procedure t_lcode()
 
char	name[SZ_FNAME]		# current file name
int	list			# input file list
int	fd			# file descriptor
int	nline			# line counter
int	nblank			# blank lines counter
int	ncomm			# comment lines counter
int	nproc			# procedure counter
int	nfunc			# function counter
int	nentry			# entry points counter
int	ninc			# include counter
int	ndef			# define counter
int	ntype			# type declarations counter
int	totline			# line total
int	totblank		# blank lines total
int	totcomm			# comment lines total
int	totproc			# procedure total
int	totfunc			# function total
int	totentry		# entry points total
int	totinc			# include total
int	totdef			# define total
int	tottype			# type declarations total
bool	verbose			# verbose output
 
bool	clgetb()
int	clpopnu(), clgfil(), open()
int	strmatch()
 
begin
	# Open input list
	list = clpopnu ("input")
 
	# Get verbose parameter
	verbose = clgetb ("verbose")
 
	# Clear totals
	totline = 0
	totblank = 0
	totcomm = 0
	totproc = 0
	totfunc = 0
	totentry = 0
	totinc = 0
	totdef = 0
	tottype	= 0
 
	# Print title
	call printf ("File         Lines  Blank   Comm   Proc   Func  Entry    Inc    Def   Type\n")
 
	# Loop over input list
	while (clgfil (list, name, SZ_FNAME) != EOF) {
 
	    # Open next file and skip it if it can't be opened
	    iferr (fd = open (name, READ_ONLY, TEXT_FILE)) {
		if (verbose)
		    call erract (EA_WARN)
		next
	    }
		
	    # Clear counters
	    nline = 0
	    nblank = 0
	    ncomm = 0
	    nproc = 0
	    nfunc = 0
	    nentry = 0
	    ninc = 0
	    ndef = 0
	    ntype = 0
 
	    # Process next file according to the file
	    # extension (SPP or FORTRAN)
	    if (strmatch (name, ".x$") != 0)
		call spp_lcode (fd, nline, nblank, ncomm, nproc, nfunc, nentry,
			      ninc, ndef, ntype)
	    else if (strmatch (name, ".f$") != 0)
		call fortran_lcode (fd, nline, nblank, ncomm, nproc, nfunc)
	    else {
		if (verbose) {
		    call eprintf ("Warning: unknown file (%s) skipped\n")
		    call pargstr (name)
		}
		next
	    }
 
	    # Add counters to totals
	    totline = totline + nline
	    totblank = totblank + nblank
	    totcomm = totcomm + ncomm
	    totproc = totproc + nproc
	    totfunc = totfunc + nfunc
	    totentry = totentry + nentry
	    totinc = totinc + ninc
	    totdef = totdef + ndef
	    tottype = tottype + ntype
 
	    # Output file name and counters
	    call output_data (name, nline, nblank, ncomm, nproc, nfunc,
			      nentry, ninc, ndef, ntype)
 
	    # Close file
	    call close (fd)
	}
 
	# Output totals
	call output_data ("Totals", totline, totblank, totcomm, totproc,
			  totfunc, totentry, totinc, totdef, tottype)
 
	# Close input list
	call clpcls (list)
end
 
 
# SPP_LCODE - Count total lines of code, blank lines, comment lines, number
# of procedures, function procedures, entry points, include statements,
# define statements, variable declarations in an SPP input file.
 
define	TYPES	"|bool|char|string|short|int|long|real|double|complex|pointer|"
 
procedure spp_lcode (fd, nline, nblank, ncomm, nproc, nfunc, nentry,
		   ninc, ndef, ntype)
 
int	fd			# file descriptor
int	nline			# line counter
int	nblank			# blank lines counter
int	ncomm			# comment lines counter
int	nproc			# procedure counter
int	nfunc			# function counter
int	nentry			# entry points counter
int	ninc			# include counter
int	ndef			# define counter
int	ntype			# type declarations counter
 
int	ip			# input string (line) pointer
int	token			# next line token
char	line[SZ_LINE]		# input line
char	outstr[SZ_LINE]		# output string
 
bool	streq()
int	fscan(), ctotok()
int	strsearch()
 
begin
	# Loop over lines
	while (fscan (fd) != EOF) {
 
	    # Read line
	    call gargstr (line, SZ_LINE)
 
	    # Get first token of line
	    ip = 1
	    token = ctotok (line, ip, outstr, SZ_LINE)
 
	    # Count blank lines, comment lines, procedure declarations, 
	    # function declarations and variable declarations.
	    if (token == TOK_EOS)
		nblank = nblank + 1
	    else if (token == TOK_OPERATOR && streq (outstr, "#"))
		ncomm = ncomm + 1
	    else if (token == TOK_IDENTIFIER) {
	    	if (streq (outstr, "procedure"))
		    nproc = nproc + 1
		else if (streq (outstr, "entry"))
		    nentry = nentry + 1
		else if (streq (outstr, "include"))
		    ninc = ninc + 1
		else if (streq (outstr, "define"))
		    ndef = ndef + 1
	        else if (strsearch (TYPES, outstr) != 0) {
		    token = ctotok (line, ip, outstr, SZ_LINE)
		    if (token == TOK_IDENTIFIER && streq (outstr, "procedure"))
		        nfunc = nfunc + 1
		    else
		        ntype = ntype + 1
		}
	    }
 
	    # Count lines
	    nline = nline + 1
	}
end
 
 
# FORTRAN_LCODE - Count total lines of code, blank lines, comment lines, number
# of procedures, and functions in a Fortran input file.
 
procedure fortran_lcode (fd, nline, nblank, ncomm, nproc, nfunc)
 
int	fd			# file descriptor
int	nline			# line counter
int	nblank			# blank lines counter
int	ncomm			# comment lines counter
int	nproc			# procedure counter
int	nfunc			# function counter
int	ip			# input string (line) pointer
int	token			# next line token
char	line[SZ_LINE]		# input line
char	outstr[SZ_LINE]		# output string
 
bool	streq()
int	fscan(), ctotok()
 
begin
	# Loop over lines
	while (fscan (fd) != EOF) {
 
	    # Read line and convert it to upper case
	    call gargstr (line, SZ_LINE)
	    call strupr (line)
 
	    # Get first token of line
	    ip = 1
	    token = ctotok (line, ip, outstr, SZ_LINE)
 
	    # Count blank lines, comment lines, sunroutine
	    # and function declarations
	    if (token == TOK_EOS)
		nblank = nblank + 1
	    else if (token == TOK_IDENTIFIER) {
	    	if (streq (outstr, "C"))
		    ncomm = ncomm + 1
	    	if (streq (outstr, "SUBROUTINE"))
		    nproc = nproc + 1
		else if (streq (outstr, "FUNCTION"))
		    nfunc = nfunc + 1
	    }
 
	    # Count lines
	    nline = nline + 1
	}
end
 
 
# OUTPUT_DATA - Print the counters or totals in one line.
 
procedure output_data (name, nline, nblank, ncomm, nproc, nfunc, nentry,
		       ninc, ndef, ntype)
 
char	name[ARB]		# name
int	nline			# line 
int	nblank			# blank lines 
int	ncomm			# comment lines 
int	nproc			# procedure 
int	nfunc			# function 
int	nentry			# entry points 
int	ninc			# include 
int	ndef			# define 
int	ntype			# type declarations 
 
begin
	call printf ("%-11.11s %6.d %6.d %6.d %6.d %6.d %6.d %6.d %6.d %6.d\n")
	    call pargstr (name)
	    call pargi (nline)
	    call pargi (nblank)
	    call pargi (ncomm)
	    call pargi (nproc)
	    call pargi (nfunc)
	    call pargi (nentry)
	    call pargi (ninc)
	    call pargi (ndef)
	    call pargi (ntype)
end

