include	<chars.h>
include	<ctype.h>
include	<error.h>
include	<fset.h>
include	<lexnum.h>
include <pattern.h>
include	<printf.h>

define	MAX_RANGES	100

define	INDEX_TYPES	"|none|prepend|append|"

define	NONE		1
define	PREPEND		2
define	APPEND		3

define	LEN_PASTE	32

# switches and scalars
define	DEF_IS_NUM	Memi[($1)]
define	INDEX		Memi[($1)+1]
define	ISCALE		Memr[P2R(($1)+2)]
define	IOFFSET		Memr[P2R(($1)+3)]
define	ADD_NEWLINE	Memi[($1)+4]
define	QUIT		Memi[($1)+5]
define	SKIP		Memi[($1)+6]
define	WHITESPACE	Memi[($1)+7]
define	NOPEN		Memi[($1)+8]
# this space for rent

# vectors and lengths
define	NFILES		Memi[($1)+16]		# file descriptors
define	FD_PT		Memi[($1)+17]
define	FD		Memi[FD_PT($1)+($2)-1]

define	IP_PT		Memi[($1)+18]		# format index for each file
define	IP		Memi[IP_PT($1)+($2)-1]

define	TYPE_PT		Memi[($1)+19]		# datatype for each file
define	TYPE		Memi[TYPE_PT($1)+($2)-1]

define	WIDTH_PT	Memi[($1)+20]		# field width for each file
define	WIDTH		Memi[WIDTH_PT($1)+($2)-1]

define	SZ_BUF		Memi[($1)+21]		# multi-purpose string buffer
define	BUF_PT		Memi[($1)+22]
define	BUF		Memc[BUF_PT($1)]

define	SZ_FORM		Memi[($1)+23]		# format string for the files
define	FORM_PT		Memi[($1)+24]
define	FORM		Memc[FORM_PT($1)+IP($1,$2)-1]

define	SZ_IFORM	Memi[($1)+25]		# format string for the index
define	IFORM_PT	Memi[($1)+26]
define	IFORM		Memc[IFORM_PT($1)]

define	SZ_DEF		Memi[($1)+27]		# default value string
define	DEF_PT		Memi[($1)+28]
define	DEF		Memc[DEF_PT($1)]
# this space for rent


# PASTE -- paste files side-by-side.

procedure t_paste ()

int	filelist, current_line, line
int	nfiles, nranges, ranges[3,MAX_RANGES], i
pointer	pe

pointer	pe_init()
int	pe_clpopen(), clplen(), clgfil(), clgeti(), btoi()
int	open(), pe_line(), decode_ranges(), get_next_number()

errchk	open, pe_clpopen, pe_init, pe_line

begin
	iferr {
	    filelist = pe_clpopen ("files", NO, btoi (clgeti ("$nargs") >= 1))
	    nfiles = clplen (filelist)
	    if (nfiles <= 0) {
		call clpcls (filelist)
		return
	    }

	    pe = pe_init (nfiles)

	    # Lines will be accessed in ascending order
	    call clgstr ("lines",  BUF(pe), SZ_BUF(pe))
	    if (decode_ranges (BUF(pe), ranges, MAX_RANGES, nranges) == ERR)
		call error (1, "problem with line specification")

	    for (i=1; clgfil (filelist, BUF(pe), SZ_BUF(pe)) != EOF; i=i+1)
		FD(pe,i) = open (BUF(pe), READ_ONLY, TEXT_FILE)

	    NOPEN(pe) = nfiles

	    line = 0
	    current_line = 0
	    while (get_next_number (ranges, line) != EOF)
		if (pe_line (pe, current_line, line) == EOF)
		    break

	} then {
	    do i = 1, NFILES(pe)
		if (FD(pe,i) != NULL)
		    call close (FD(pe,i))

	    call pe_close (pe)
	    call clpcls (filelist)
	    call erract (EA_ERROR)
	}

	# close all the files
	do i = 1, NFILES(pe)
	    if (FD(pe,i) != NULL)
		call close (FD(pe,i))

	call pe_close (pe)
	call clpcls (filelist)
end


# PE_INIT -- initialize the paste data structure.

pointer procedure pe_init (nfiles)

int	nfiles		#I number of files in template

pointer	pe
int	ip, junk

bool	clgetb()
int	clgwrd(), btoi(), lexnum()
real	clgetr()
char	pe_next_fmt()

begin
	# error recovery assumes that NULL == 0
	call calloc (pe, LEN_PASTE, TY_STRUCT)

	# Note that FMTIO limits formats to SZ_LINE.  SZ_FORM needs room
	# for the format and for enough NULLs to separate the fields.
	NFILES(pe) = nfiles
	SZ_BUF(pe) = SZ_LINE
	SZ_DEF(pe) = SZ_LINE
	SZ_FORM(pe) = 2 * SZ_LINE
	SZ_IFORM(pe) = SZ_LINE

	call calloc (FD_PT(pe), NFILES(pe), TY_INT)
	call malloc (IP_PT(pe), NFILES(pe), TY_INT)
	call malloc (TYPE_PT(pe), NFILES(pe), TY_INT)
	call malloc (WIDTH_PT(pe), NFILES(pe), TY_INT)
	call malloc (BUF_PT(pe), SZ_BUF(pe), TY_CHAR)
	call malloc (DEF_PT(pe), SZ_DEF(pe), TY_CHAR)
	call malloc (FORM_PT(pe), SZ_FORM(pe), TY_CHAR)
	call malloc (IFORM_PT(pe), SZ_IFORM(pe), TY_CHAR)

	ADD_NEWLINE(pe) = btoi (clgetb ("newline"))
	QUIT(pe) = btoi (clgetb ("quit_on_eof"))
	SKIP(pe) = btoi (clgetb ("skip_if_missing"))
	WHITESPACE(pe) = btoi (clgetb ("whitespace"))

	# read and interpret the default string
	call clgstr ("default", DEF(pe), SZ_DEF(pe))

	# this should require trailing characters to be whitespace
	# doesn't work with complex numbers
	DEF_IS_NUM(pe) = btoi (lexnum (DEF(pe), ip, junk) != LEX_NONNUM)

	# read and parse the format string
	call clgstr ("format", BUF(pe), SZ_BUF(pe))

	ip = 0
	if (pe_next_fmt (BUF(pe), ip, junk) == NUL)
	    # assumes that the buffer length is >= 5
	    call strcpy ("%s\t%s", BUF(pe), 5)
	else
	    # the difference between user entered and compiled formats
	    # remove this (and similar call below) when fpradv() is fixed.
	    call pe_convert_fmt (BUF(pe))

	IP(pe,1) = 1		# this enables the FORM reference below

	call pe_decode_fmt (BUF(pe), FORM(pe,1), IP(pe,1), TYPE(pe,1),
	    WIDTH(pe,1), NFILES(pe))

	# read the indexing parameters
	INDEX(pe) = clgwrd ("index", BUF(pe), SZ_BUF(pe), INDEX_TYPES)
	if (INDEX(pe) != NONE) {
	    ISCALE(pe) = clgetr ("scale")
	    IOFFSET(pe) = clgetr ("offset")

	    call clgstr ("iformat", IFORM(pe), SZ_IFORM(pe))

	    ip = 0
	    if (pe_next_fmt (IFORM(pe), ip) != NUL)
		call pe_convert_fmt (IFORM(pe))
	    else
		switch (INDEX(pe)) {
		case PREPEND:
		    call strcpy ("%4d\t", IFORM(pe), 5)
		case APPEND:
		    call strcpy ("\t%4d", IFORM(pe), 5)
		default:
		    call error (1, "bad switch in PE_INIT")
		}
	}

	return (pe)
end


#PE_CLOSE -- clean up, assumes that the files have been closed.

procedure pe_close (pe)

pointer	pe		#U paste structure descriptor

begin
	if (pe == NULL)
	    return

	if (IFORM_PT(pe) != NULL)
	    call mfree (IFORM_PT(pe), TY_CHAR)

	if (FORM_PT(pe) != NULL)
	    call mfree (FORM_PT(pe), TY_CHAR)

	if (BUF_PT(pe) != NULL)
	    call mfree (BUF_PT(pe), TY_CHAR)

	if (WIDTH_PT(pe) != NULL)
	    call mfree (WIDTH_PT(pe), TY_INT)

	if (TYPE_PT(pe) != NULL)
	    call mfree (TYPE_PT(pe), TY_INT)

	if (IP_PT(pe) != NULL)
	    call mfree (IP_PT(pe), TY_INT)

	if (FD_PT(pe) != NULL)
	    call mfree (FD_PT(pe), TY_INT)

	call mfree (pe, TY_STRUCT)
	pe = NULL			# just in case
end


# PE_LINE -- paste the specific line number (in order) in a list of files.

int procedure pe_line (pe, current, desired)

pointer	pe		#I paste structure descriptor
int	current		#U current line number
int	desired		#I line number desired

int	nf, nl, nl_max, len
pointer	format, sp
real	index
bool	first

long	lval
double	dval
bool	bval
complex	xval

int	getline(), nscan(), strmatch()

define	end_	13

begin
	if (current >= desired)
	    call error (1, "line numbers are out of sync")

	call smark (sp)
	call salloc (format, SZ_LINE, TY_CHAR)

	if (INDEX(pe) != NONE)
	    index = ISCALE(pe) * desired + IOFFSET(pe)

	first = true
	nl_max = current
	do nf = 1, NFILES(pe) {

	    if (FD(pe,nf) != NULL) {
		for (nl=current+1; nl <= desired; nl=nl+1) {
		    len = getline (FD(pe,nf), BUF(pe))

		    if (len == EOF) {
			call close (FD(pe,nf))
			FD(pe,nf) = NULL	# just to be sure...

			# exit on first EOF
			if (QUIT(pe) == YES)
			    goto end_

			NOPEN(pe) = NOPEN(pe) - 1
			break
		    }

		    call pe_nldel (BUF(pe), len)
		}

		nl = nl - 1
		nl_max = max (nl, nl_max)
	    }

	    if (FD(pe,nf) == NULL)
		BUF(pe) = EOS
	    else if (WHITESPACE(pe) == NO)
		# Is the entire line whitespace?
		if (strmatch (BUF(pe), "^#$") != 0)
		    BUF(pe) = EOS

	    if (BUF(pe) == EOS && SKIP(pe) == YES)
		next

	    if (first) {
		first = false
		if (INDEX(pe) == PREPEND) {
		    call printf (IFORM(pe))
			call pargr (index)
		}
	    }

	    switch (TYPE(pe,nf)) {
	    case TY_CHAR:
		if (BUF(pe) == EOS) {
		    call printf (FORM(pe,nf))
			call pargstr (DEF(pe))
		} else {
		    call printf (FORM(pe,nf))
			call pargstr (BUF(pe))
		}

	    case TY_LONG:
		call sscan (BUF(pe))
		    call gargl (lval)

		if (nscan() == 1) {
		    call printf (FORM(pe,nf))
			call pargl (lval)

		} else if (SKIP(pe) == YES) {
		    next

		} else if (DEF_IS_NUM(pe) == YES) {
		    # should scan this once in PE_INIT
		    call sscan (DEF(pe))
			call gargl (lval)

		    call printf (FORM(pe,nf))
			call pargl (lval)

		} else {
		    call printf ("%*s")
			call pargi (WIDTH(pe,nf))
			call pargstr (DEF(pe))
		}

	    case TY_DOUBLE:
		call sscan (BUF(pe))
		    call gargd (dval)

		if (nscan() == 1) {
		    call printf (FORM(pe,nf))
			call pargd (dval)

		} else if (SKIP(pe) == YES) {
		    next

		} else if (DEF_IS_NUM(pe) == YES) {
		    # should scan this once in PE_INIT
		    call sscan (DEF(pe))
			call gargd (dval)

		    call printf (FORM(pe,nf))
			call pargd (dval)

		} else {
		    call printf ("%*s")
			call pargi (WIDTH(pe,nf))
			call pargstr (DEF(pe))
		}

	    case TY_BOOL:
		call sscan (BUF(pe))
		    call gargb (bval)

		if (nscan() == 1) {
		    call printf (FORM(pe,nf))
			call pargb (bval)

		} else if (SKIP(pe) == YES) {
		    next

		} else {
		    # should scan this once in PE_INIT
		    call sscan (DEF(pe))
			call gargb (bval)

		    if (nscan() == 1) {
			call printf (FORM(pe,nf))
			    call pargb (bval)
		    } else {
			call printf ("%*s")
			    call pargi (WIDTH(pe,nf))
			    call pargstr (DEF(pe))
		    }
		}

	    # lexnum doesn't handle complex numbers
	    case TY_COMPLEX:
		call sscan (BUF(pe))
		    call gargx (xval)

		if (nscan() == 1) {
		    call printf (FORM(pe,nf))
			call pargx (xval)

		} else if (SKIP(pe) == YES) {
		    next

		} else {
		    # should scan this once in PE_INIT
		    call sscan (DEF(pe))
			call gargx (xval)

		    if (nscan() == 1) {
			call printf (FORM(pe,nf))
			    call pargx (xval)
		    } else {
			call printf ("%*s")
			    call pargi (WIDTH(pe,nf))
			    call pargstr (DEF(pe))
		    }
		}

	    default:
		call sfree (sp)
		call error (1, "bad switch in PE_LINE")
	    }
	}

	current = nl_max

end_	if (! first) {
	    if (INDEX(pe) == APPEND) {
		call printf (IFORM(pe))
		    call pargr (index)
	    }
	    if (ADD_NEWLINE(pe) == YES)
		call printf ("\n")
	}

	call flush (STDOUT)
	call sfree (sp)

	# exit on last EOF
	if (NOPEN(pe) == 0 || current != desired)
	    return (EOF)
	else
	    return (current)
end


# PE_DECODE_FMT -- index the individual fields in a format string.
# The final available field is propagated to all remaining entries.

# out_fmt should be at least twice the size of in_fmt

procedure pe_decode_fmt (in_fmt, out_fmt, index, type, width, nfields)

char	in_fmt[ARB]		#I input format string to be decoded
char	out_fmt[ARB]		#O decoded format
int	index[nfields]		#O array of indices
int	type[nfields]		#O array of types
int	width[nfields]		#O array of widths
int	nfields			#I size of the arrays

int	size1, type1, width1, new_ip, new_width, next_ip, i, ip, op
char	format_char
bool	init

char	pe_next_fmt()

begin
	init = false

	ip = 1
	op = 1
	new_ip = 0

	# this assumes that there is at least one good format specification
	do i = 1, nfields {
	    format_char = pe_next_fmt (in_fmt, new_ip, new_width)

	    if (format_char == GET_FIELD)
		call error (1, "deferred value formats not supported")

	    else if (format_char != NUL) {
		size1 = new_ip - ip
		width1 = new_width

		switch (format_char) {
		case FMT_CHAR, FMT_STRING:
		    type1 = TY_CHAR
		case FMT_DECIMAL, FMT_OCTAL, FMT_HEX, FMT_RADIX, FMT_UNSIGNED:
		    type1 = TY_LONG
		case FMT_EXPON, FMT_FIXED, FMT_GENERAL, FMT_HMS, FMT_MINSEC:
		    type1 = TY_DOUBLE
		case FMT_BOOL:
		    type1 = TY_BOOL
		case FMT_COMPLEX:
		    type1 = TY_COMPLEX
		default:
		    call error (1, "bad format code")
		}

		next_ip = new_ip
		init = true
	    }

	    if (! init)
		call error (1, "bad format specification")

	    index[i] = op
	    type[i] = type1
	    width[i] = width1

	    call strcpy (in_fmt[ip], out_fmt[op], size1)

	    ip = next_ip
	    op = op + size1 + 1
	}
end


# PE_NEXT_FMT -- find the index of the character following the next
# "%w.dC" format sequence and return the format specification code.
# If the next format is the last, sets the index to the EOS.
# Returns EOF if no formats remain or if the next format is invalid.
# Returns GET_FIELD (see <printf.h>) if a deferred value field is found.

# This uses routines modified from fpradv.x and fprfmt.x, which are
# internal to FMTIO.

# The overhead of checking each format twice keeps the caller simpler.
# This routine is only called once per invocation so the overhead is ok.

char procedure pe_next_fmt (format, ip, width)

char	format[ARB]		#I format string
int	ip			#U pointer to next char in format string
int	width			#O field width

int	ip2, ip_save, junk
char	chtype

int	pe_fpradv()
char	pe_fprfmt()

begin
	if (pe_fpradv (format, ip) == EOF)
	    return (NUL)

	chtype = pe_fprfmt (format, ip, width)
	if (chtype == NUL)
	    return (NUL)

	# Is this the last (legal) format?  We point to the character
	# immediately following the current format unless this is the
	# last legal format, in which case we point to either the end of
	# the format string or to the last `good' character in the format.

	ip2 = ip
	if (pe_fpradv (format, ip2) == EOF)
	    # end of the format string
	    ip = ip2
	else {
	    ip_save = ip2
	    if (pe_fprfmt (format, ip2, junk) == NUL)
		# next format is bad
		ip = ip_save
	}

	return (chtype)
end


# PE_FPRADV -- find the index of the next format.  Return OK if there
# another format to follow, EOF if not.

# Note that the `w' and `t' formats must be lowercase and can't be deferred.

int procedure pe_fpradv (format, ip)

char	format[ARB]		#I format string
int	ip			#U pointer to next char in format string
int	ip_save, junk, ival

int	ctoi()

begin
	if (format[ip] == EOS)
	    return (EOF)

	# I removed the `inappropriate' escape character handling.
	# This causes problems with fields like:  `\\<escape sequence>'.

	# position to the start of the next format
	for (ip=ip+1;  format[ip] != EOS;  )
	    if (format[ip] == START_OF_FORMAT) {
		if (format[ip+1] == START_OF_FORMAT)	# '%%' --> '%'
		    ip = ip + 2

		else if (IS_DIGIT (format[ip+1])) {	# %Nw or %Nt
		    ip_save = ip
		    ip = ip + 1
		    junk = ctoi (format, ip, ival)

		    switch (format[ip]) {
		    case FMT_WHITESPACE, FMT_TOCOLUMN:
			ip = ip + 1			# eat "t" or "w"
			next

		    default:
			ip = ip_save			# regular format spec
			break				# now what is it?
		    }

		} else
		    break				# regular format spec
	    } else
		ip = ip + 1

	if (format[ip] == EOS)
	    return (EOF)
	else
	    return (OK)
end


# PE_FPRFMT -- verify the next format string is valid and return the
# character code.  Leave the index positioned after the end of the format.

char procedure pe_fprfmt (format, ip, width)

char	format[ARB]		#I format string
int	ip			#U pointer to next char in format string
int	width			#O field width

int	junk, ival, deferred
char	chtype, ch

int	ctoi(), stridx()
char	chrlwr()

begin
	ip = ip + 1
	deferred = 0

	if (format[ip] == '-')
	    ip = ip + 1
	if (format[ip] == '0')
	    ip = ip + 1

	if (format[ip] == GET_FIELD) {			# "%*.dC"
	    deferred = deferred + 1
	    width = INDEFI
	} else
	    junk = ctoi (format, ip, width)

	if (format[ip] == '.') {
	    ip = ip + 1
	    if (format[ip] == GET_FIELD)		# "%w.*C"
		deferred = deferred + 1
	    else
		junk = ctoi (format, ip, ival)
	}

	chtype = chrlwr (format[ip])			# "%w.dC"

	if (chtype == GET_FIELD) 			# "%w.d*"
	    deferred = deferred + 1

	else if (stridx (chtype, "bcdefghmorsuxz") <= 0)
	    # this includes EOS and other bad specifications
	    # note that `w' and `t' formats should never appear here
	    return (NUL)

	else if (chtype == FMT_RADIX) {			# "%w.dr*"
	    ip = ip + 1
	    ch = chrlwr (format[ip])

	    if (ch == GET_FIELD)
		deferred = deferred + 1
	    else if (! IS_DIGIT(ch) && ! IS_LOWER(ch))
		ip = ip - 1
	}

	# position index one place _past_ the current format
	ip = ip + 1

	# handle this better
	if (deferred != 0)
	    return (GET_FIELD)
	else
	    return (chtype)
end


# PE_CONVERT_FMT -- convert escape sequences into their ascii equivalents.
# This should be handled by FMTIO but there's a bug in fpradv.x.

procedure pe_convert_fmt (a)

char	a[ARB]

int	ip, op, junk, cctoc()

begin
	ip = 1
	for (op=1; a[ip] != EOS; op=op+1)
	    if (a[ip] == ESCAPE)
		junk = cctoc (a, ip, a[op])
	    else {
		a[op] = a[ip]
		ip = ip + 1
	    }

	a[op] = EOS
end


# PE_NLDEL -- remove a newline from the end of a string (if there is one).

procedure pe_nldel (str, len)

char	str[ARB]	#U character string, perhaps ending in newline
int	len		#I length of the string, i.e., char before EOS

begin
	if (str[len] == '\n')
	    str[len] = EOS
end


# PE_CLPOPEN -- open a template, doing something reasonable if the
# standard input is redirected.  May be sorted or unsorted.

int procedure pe_clpopen (param, sort, force)

char	param[ARB]		#I CL filename template parameter
int	sort			#I sort the list?
int	force			#I expand the template even if redirected?

int	list
pointer	buf, sp
bool	stdin_seen

int	fntopnb(), fntgfnb(), fstati(), strcmp()

begin
	call smark (sp)
	call salloc (buf, SZ_LINE, TY_CHAR)

	if (fstati (STDIN, F_REDIR) == NO) {
	    call clgstr (param, Memc[buf], SZ_LINE)
	    list = fntopnb (Memc[buf], sort)

	} else if (force == NO) {
	    list = fntopnb ("STDIN", sort)

	} else {
	    call clgstr (param, Memc[buf], SZ_LINE)
	    list = fntopnb (Memc[buf], sort)

	    stdin_seen = false
	    while (fntgfnb (list, Memc[buf], SZ_LINE) != EOF)
		if (strcmp (Memc[buf], "STDIN") == 0)
		    stdin_seen = true

	    if (! stdin_seen) {
		call eprintf ( "Warning: redirected STDIN isn't in file list.")
		call flush (STDERR)
	    }

	    call fntrewb (list)
	}

	call sfree (sp)
	return (list)
end
