# Copyright(c) 1986 Association of Universities for Research in Astronomy Inc.

# various of the SZ_LINE's should probably be SZ_PATHNAME's...

include	<error.h>
include	<imhdr.h>
include	<time.h>
include <fset.h>
include <oif.h>

define	SZ_DIMSTR	(IM_MAXDIM*4)

# values for the header access status
define	NOT_FOUND	0
define	OK		1
define	INFERRED	2
define	WRONG_HEADER	3

define	MAX_BLANKS	SZ_FNAME


# T_PIXHEADER -- Read contents of a pixel "header" and print on STDOUT.

procedure t_pixheader()

int	list, count, maxlen
pointer	sp, pixfile
bool	verbose, orphan_only

int	clpopnu(), clgfil(), max_length()
bool	clgetb()

begin
	call smark (sp)
	call salloc (pixfile, SZ_LINE, TY_CHAR)

	list = clpopnu ("pixels")
	maxlen = max_length (list)

	verbose = clgetb ("longheader")
	orphan_only = clgetb ("orphan_only")

	if (clgetb ("left_justify"))
	    maxlen = -maxlen

	count = 0
	while (clgfil (list, Memc[pixfile], SZ_LINE) != EOF)
	    iferr (call pixphdr (Memc[pixfile], verbose, orphan_only,
		maxlen, count)) {

		call flush (STDOUT)	# to synch the error message
		call erract (EA_WARN)
	    }

	call clputi ("nfiles", count)

	call clpcls (list)
	call sfree (sp)
end


# PIXPHDR -- Print the contents of a pixel file internal header.

procedure pixphdr (pixfile, verbose, orphan_only, maxlen, count)

char	pixfile[ARB]		#I pixel file name
bool	verbose			#I verbose output?
bool	orphan_only		#I only list "orphans"?
int	maxlen			#I max length of pixel name for format
int	count			#U running count of files listed

char	ldim[SZ_DIMSTR], pdim[SZ_DIMSTR], ctime[SZ_TIME]
pointer	pix, hdr, sp
int	hdr_status

pointer	pixmap()
int	access_hdr()

errchk	pixmap, access_hdr, pix_fmt_dim

begin
	pix = pixmap (pixfile)

	call smark (sp)
	call salloc (hdr, SZ_LINE, TY_CHAR)

	# the header filename is stored in IM_PIXFILE in the pixel file
	hdr_status = access_hdr (IM_PIXFILE(pix), pixfile, pix,
	    Memc[hdr], SZ_LINE)
	if (orphan_only && hdr_status != NOT_FOUND) {
	    call sfree (sp)
	    call pixunmap (pix)
	    return
	}

	count = count + 1
	if (verbose && count > 1)
	    call printf ("\n")

	call printf ("%*s %s %s\n")
	    call pargi (maxlen)		# L/R justification is implicit
	    call pargstr (pixfile)

	    switch (hdr_status) {
	    case NOT_FOUND:
		call pargstr ("?")
	    case WRONG_HEADER:
		call pargstr ("!")
	    default:
		call pargstr (" ")
	    }

	    call pargstr (Memc[hdr])

	if (verbose) {
	    switch (hdr_status) {
	    case NOT_FOUND:
		call print_blanks (abs(maxlen)+3)
		call printf ("HEADER FILE NOT FOUND\n")
	    case WRONG_HEADER:
		call print_blanks (abs(maxlen)+3)
		call printf ("WRONG HEADER WAS FOUND\n")
	    case INFERRED:
		call print_blanks (abs(maxlen)+2)	# line up pathnames
		call printf ("[%s]\n")
		    call pargstr (IM_PIXFILE(pix))
	    }

	    # Format the subscript and date strings
	    call pix_fmt_dim (pix, ldim, SZ_DIMSTR, IM_LEN(pix,1))
	    call pix_fmt_dim (pix, pdim, SZ_DIMSTR, IM_PHYSLEN(pix,1))
	    call cnvtime (IM_CTIME(pix), ctime, SZ_TIME)

	    call print_blanks (abs(maxlen)+3)
	    call printf ("%s[%s], physdim %s\n")
		call pargstr (ldim)
		call ph_pargtype (IM_PIXTYPE(pix))
		call pargstr (pdim)

	    call print_blanks (abs(maxlen)+3)
	    call printf ("created %s\n")
		call pargstr (ctime)
	}

	call sfree (sp)
	call pixunmap (pix)
end


# PIXMAP -- "Map" a pixel file's internal header.  We could play some
# games with tricking immap to work, but the IM_MAGIC string is
# hardwired to "imhdr" (instead of "impix") throughout much of the
# IMIO and the OIF IKI.

pointer procedure pixmap (pixfile)

char	pixfile[ARB]		#I pixel file pathname

pointer	pix, buf, sp
int	fd, n

int	open(), read()
bool	strne()

errchk	open, seek, read

begin
	fd = open (pixfile, READ_ONLY, BINARY_FILE)

	call malloc (pix, LEN_IMDES + LEN_PIXHDR, TY_STRUCT)

	iferr {
	    sp = NULL

	    call seek (fd, BOFL)
	    n = read (fd, IM_MAGIC(pix), LEN_PIXHDR * SZ_STRUCT)

	    if (n < LEN_PIXHDR * SZ_STRUCT || strne (IM_MAGIC(pix), "impix")) {
		call smark (sp)		# the stack is freed by the handler
		call salloc (buf, SZ_LINE, TY_CHAR)

		call sprintf (Memc[buf], SZ_LINE, "Pixel file `%s' unreadable.")
		    call pargstr (pixfile)

		call error (1, Memc[buf])
	    }
	} then {
	    if (sp != NULL)
		call sfree (sp)
	    call close (fd)
	    call mfree (pix, TY_STRUCT)
	    call erract (EA_ERROR)
	}

	call close (fd)
	return (pix)
end


# PIXUNMAP -- "Unmap" a pixel file's internal header.

procedure pixunmap (pix)

pointer	pix			#I pixel file descriptor

begin
	if (pix != NULL)
	    call mfree (pix, TY_STRUCT)

	# just to be sure
	pix = NULL
end


# ACCESS_HDR -- Is the indicated header file accessable?  If not, can
# a better pathname be inferred from the given header directory and the
# pixel file root name?

int procedure access_hdr (hdrfile, pixfile, pix, newhdr, maxch)

char	hdrfile[ARB]		#I header file pathname to test
char	pixfile[ARB]		#I pixel file for reconstruction
pointer	pix			#I pixel descriptor for comparison
char	newhdr[ARB]		#O header pathname if found
int	maxch			#I maximum length of newhdr

pointer	im, hdr, path, root, sp, pix2
int	status
int	junk

pointer	immap(), pixmap()
int	fnldir(), fnroot()
bool	aveqi()

errchk	immap, pixmap

begin
	call flush (STDOUT)	# to synch immap KS prompts

	call smark (sp)
	call salloc (hdr, SZ_LINE, TY_CHAR)
	call salloc (path, SZ_LINE, TY_CHAR)
	call salloc (root, SZ_LINE, TY_CHAR)

	# convert VMS style names
	call hokey_osfn2vfn (hdrfile, Memc[hdr], SZ_LINE)

	# and extract the directory (sometimes needed below)
	junk = fnldir (Memc[hdr], Memc[path], SZ_LINE)

	ifnoerr (im = immap (Memc[hdr], READ_ONLY, 0)) {
	    call strcpy (hdrfile, newhdr, maxch)
	    status = OK

	} else {
	    # another guess based on the pixel file name
	    junk = fnroot (pixfile, Memc[root], SZ_LINE)
	    call strcpy (Memc[path], Memc[hdr], SZ_LINE)
	    call strcat (Memc[root], Memc[hdr], SZ_LINE)

	    # for neatness - but can fail if oif image lacks an extension
	    call strcat (".imh", Memc[hdr], SZ_LINE)

	    ifnoerr (im = immap (Memc[hdr], READ_ONLY, 0)) {
		call strcpy (Memc[hdr], newhdr, maxch)
		status = INFERRED

	    } else {
		call sfree (sp)
		call strcpy (hdrfile, newhdr, maxch)
		return (NOT_FOUND)
	    }
	}

	# the header exists - but is it the right header?

	# make the header's pointer to the pixel file usable
	call pu_gpixfname (IM_PIXFILE(im), Memc[path], SZ_LINE)

	# compare the new pixel file to the old one
	pix2 = pixmap (Memc[path])

	# assumes SZ_STRUCT == SZ_INT
	if (! aveqi (IM_MAGIC(pix), IM_MAGIC(pix2), LEN_PIXHDR))
	    status = WRONG_HEADER

	call pixunmap (pix2)
	call imunmap (im)
	call sfree (sp)
	return (status)
end


# HOKEY_OSFN2VFN -- escape the first '[' in a header pathname to get us
# past immap.  There shouldn't be any conflict with the section notation.

procedure hokey_osfn2vfn (osfile, vfile, maxch)

char	osfile[ARB]		#I os dependent pathname
char	vfile[ARB]		#O acceptable VOS pathname
int	maxch			#I maximum length of vfile

int	i, stridxs()

begin
	i = stridxs ("[", osfile)

	if (i > 0) {
	    call strcpy (osfile, vfile, i-1)
	    call strcpy ("\\", vfile[i], 1)
	    call strcpy (osfile[i], vfile[i+1], maxch-i)
	} else
	    call strcpy (osfile, vfile, maxch)
end


# PU_GPIXFNAME -- expand an image headers copy of the pathname to
# the pixel file into an exceptable host/vos pathname.

procedure pu_gpixfname (pixpath, realpath, maxch)

char	pixpath[ARB]		#I pixel path as known to the header
char	realpath[ARB]		#U header directory [I] / pixel path [O]
int	maxch			#I max length of realpath

int	strsearch()

begin
	if (strsearch (pixpath, "HDR$") == 5)
	    call strcat (pixpath[5], realpath, maxch)
	else
	    call strcpy (pixpath, realpath, maxch)
end


# MAX_LENGTH -- find the maximum length of a file name from a template.
# This is coded using clgfil, but the fnt package could be used directly.

int procedure max_length (list)

int	list		#I file template descriptor

int	maxlen
pointer	filename, sp

int	clgfil(), strlen()

begin
	call smark (sp)
	call salloc (filename, SZ_LINE, TY_CHAR)

	maxlen = 0
	while (clgfil (list, Memc[filename], SZ_LINE) != EOF)
	    maxlen = max (strlen (Memc[filename]), maxlen)

	call clprew (list)

	call sfree (sp)
	return (maxlen)
end


# PIX_FMT_DIM -- Format the image dimensions in the form
# of a subscript, i.e., "[nx,ny,nz,...]".

procedure pix_fmt_dim (pix, outstr, maxch, len_axes)

pointer	pix			#I pixel file descriptor
char	outstr[ARB]		#O formatted dimension string
int	maxch			#I maximum length of outstr
long	len_axes[ARB]		#I array of axis lengths (for convenience)

int	i, fd

int	stropen()
errchk	stropen, fprintf, pargl

begin
	fd = stropen (outstr, maxch, NEW_FILE)

	call fprintf (fd, "[%d")
	    call pargl (len_axes[1])

	do i = 2, IM_NDIM(pix) {
	    call fprintf (fd, ",%d")
		call pargl (len_axes[i])
	}

	call fprintf (fd, "]")
	call close (fd)
end


# PH_PARGTYPE -- Convert an integer type code into a string,
# and output the string with PARGSTR to FMTIO.

procedure ph_pargtype (dtype)

int	dtype			#I data type code

begin
	switch (dtype) {
	case TY_UBYTE:
	    call pargstr ("ubyte")
	case TY_BOOL:
	    call pargstr ("bool")
	case TY_CHAR:
	    call pargstr ("char")
	case TY_SHORT:
	    call pargstr ("short")
	case TY_USHORT:
	    call pargstr ("ushort")
	case TY_INT:
	    call pargstr ("int")
	case TY_LONG:
	    call pargstr ("long")
	case TY_REAL:
	    call pargstr ("real")
	case TY_DOUBLE:
	    call pargstr ("double")
	case TY_COMPLEX:
	    call pargstr ("complex")
	case TY_POINTER:
	    call pargstr ("pointer")
	case TY_STRUCT:
	    call pargstr ("struct")
	default:
	    call pargstr ("unknown datatype")
	}
end


# PH_PARGSTAT -- Convert an integer type code into a string,
# and output the string with PARGSTR to FMTIO.

procedure ph_pargstat (dtype)

int	dtype			#I data type code

begin
	switch (dtype) {
	case NOT_FOUND:
	    call pargstr ("[NOT FOUND]")
	case OK:
	    call pargstr ("[ok]")
	case INFERRED:
	    call pargstr ("[inferred]")
	default:
	    call pargstr ("[UNKNOWN]")
	}
end


# PRINT_BLANKS -- output a number of blanks on the stdout.  Only need
# this since the %t and %w formats won't take deferred value fields.

procedure print_blanks (number)

int	number		#I number of blanks to print

int	i

begin
	do i = 1, min (number, MAX_BLANKS)
	    call printf (" ")
end


# T_PIXFIX -- regenerate a header for a wayward pixel file.

procedure t_pixfix ()

int	pixlist, junk
pointer	pix, sp, pixfile, hdrfile, path, root
bool	firsttime, old_directory, old_name, update

bool	clgetb()
int	clpopnu(), clgfil(), fnldir(), fnroot()
pointer	pixmap()

errchk	pixmap, pu_init_header

begin
	firsttime = true

	call smark (sp)
	call salloc (pixfile, SZ_LINE, TY_CHAR)
	call salloc (hdrfile, SZ_LINE, TY_CHAR)
	call salloc (path, SZ_LINE, TY_CHAR)
	call salloc (root, SZ_LINE, TY_CHAR)

	pixlist = clpopnu ("pixels")

	old_directory = clgetb ("old_directory")
	if (! old_directory) {
	    call clgstr ("directory", Memc[root], SZ_LINE)
	    junk = fnldir (Memc[root], Memc[path], SZ_LINE)
	}

	old_name = clgetb ("old_name")
	update = clgetb ("update")

	while (clgfil (pixlist, Memc[pixfile], SZ_LINE) != EOF) {
	    iferr (pix = pixmap (Memc[pixfile])) {
		call flush (STDOUT)	# to synch the error message
		call erract (EA_WARN)
		next
	    }
		
	    if (old_directory)
		# header name is stored in IM_PIXFILE in the pixel file
		junk = fnldir (IM_PIXFILE(pix), Memc[hdrfile], SZ_LINE)
	    else
		call strcpy (Memc[path], Memc[hdrfile], SZ_LINE)

	    if (old_name)
		junk = fnroot (IM_PIXFILE(pix), Memc[root], SZ_LINE)
	    else
		junk = fnroot (Memc[pixfile], Memc[root], SZ_LINE)

	    call strcat (Memc[root], Memc[hdrfile], SZ_LINE)
	    call strcat (".imh", Memc[hdrfile], SZ_LINE)	# for neatness

	    iferr (call pu_init_header (Memc[hdrfile], Memc[pixfile], pix)) {
		call pixunmap (pix)
		call flush (STDOUT)
		call erract (EA_WARN)
		next
	    }

	    if (update)
		call pu_update_pix (Memc[hdrfile], Memc[pixfile], pix)

	    if (! firsttime)
		call printf ("\n")

	    call printf ("Created header `%s'\nfor pixel file `%s'\n")
		call pargstr (Memc[hdrfile])
		call pargstr (Memc[pixfile])

	    call pixunmap (pix)

	    firsttime = false
	}

	call clpcls (pixlist)
	call sfree (sp)
end


# PU_INIT_HEADER -- initialize a blank header to point to a pixel file.

procedure pu_init_header (hdrfile, pixfile, pix)

char	hdrfile[ARB]		#I name of header to create
char	pixfile[ARB]		#I name of pixel file
pointer	pix			#I pixel file descriptor

int	i
pointer	sp, pixpath, hdr

pointer	immap()
errchk	immap

begin
	call smark (sp)
	call salloc (pixpath, SZ_LINE, TY_CHAR)

	hdr = immap (hdrfile, NEW_IMAGE, 0)

	IM_PIXTYPE(hdr)	= IM_PIXTYPE(pix)
	IM_NDIM(hdr)	= IM_NDIM(pix)

	do i = 1, IM_MAXDIM {
	    IM_LEN(hdr,i)	= IM_LEN(pix,i)
	    IM_PHYSLEN(hdr,i)	= IM_PHYSLEN(pix,i)
	}

	# some of these aren't needed (some others might be)
	IM_LUTOFF(hdr)	= IM_LUTOFF(pix)
	IM_PIXOFF(hdr)	= IM_PIXOFF(pix)
	IM_HGMOFF(hdr)	= IM_HGMOFF(pix)
	IM_BLIST(hdr)	= IM_BLIST(pix)
	IM_SZBLIST(hdr)	= IM_SZBLIST(pix)
	IM_NBPIX(hdr)	= IM_NBPIX(pix)

	call strcpy ("Reconstructed Header", IM_TITLE(hdr), SZ_IMTITLE)

	call fpathname (pixfile, Memc[pixpath], SZ_LINE)
	call strcpy (Memc[pixpath], IM_PIXFILE(hdr), SZ_IMPIXFILE)

	call imunmap (hdr)
	call sfree (sp)
end


# PU_UPDATE_PIX -- modify the internal pixel file pointer to reference
# the newly created header file.

procedure pu_update_pix (hdrfile, pixfile, pix)

char	hdrfile[ARB]		#I pathname to the new header
char	pixfile[ARB]		#I pathname to the pixel file
pointer	pix			#I pixel file descriptor

int	fd

int	open()

errchk	open, seek, write, close

begin
	# header filename is stored in IM_PIXFILE in the pixel header
	call strcpy (hdrfile, IM_PIXFILE(pix), SZ_IMPIXFILE)

	# only write over the internal header
	# could limit this to the IM_PIXFILE string itself...
	fd = open (pixfile, WRITE_ONLY, BINARY_FILE)
	call seek (fd, BOFL)
	call fseti (fd, F_BUFSIZE, LEN_PIXHDR * SZ_STRUCT)
	call write (fd, IM_MAGIC(pix), LEN_PIXHDR * SZ_STRUCT)
	call close (fd)
end


# T_PIXUPDATE -- correct the internal pixel file pointers to a
# list of image headers.

procedure t_pixupdate ()

pointer	hdrlist, hdrpath, hdrfile, hdr, pixfile, pix, sp
int	junk

pointer	imtopenp(), immap(), pixmap()
int	imtgetim(), fnldir()

errchk	immap, pixmap

begin
	call smark (sp)
	call salloc (hdrpath, SZ_LINE, TY_CHAR)
	call salloc (hdrfile, SZ_LINE, TY_CHAR)
	call salloc (pixfile, SZ_LINE, TY_CHAR)

	hdrlist = imtopenp ("headers")

	while (imtgetim (hdrlist, Memc[hdrfile], SZ_LINE) != EOF) {
	    iferr (hdr = immap (Memc[hdrfile], READ_ONLY, 0)) {
		call erract (EA_WARN)
		next
	    }

	    junk = fnldir (Memc[hdrfile], Memc[pixfile], SZ_LINE)
	    call pu_gpixfname (IM_PIXFILE(hdr), Memc[pixfile], SZ_LINE)
	    call imunmap (hdr)

	    ifnoerr (pix = pixmap (Memc[pixfile])) {
		call fpathname (Memc[hdrfile], Memc[hdrpath], SZ_LINE)
		call pu_update_pix (Memc[hdrpath], Memc[pixfile], pix)
		call pixunmap (pix)
	    } else
		call erract (EA_WARN)
	}

	call imtclose (hdrlist)
	call sfree (sp)
end
