C     SUBROUTINE SCALE1
C
C     PURPOSE
C        TO FIND THE SCALE PARAMETERS OF AN ARRAY OF DATA
C
C     USAGE
C        CALL SCALE1(ARRAY,DIST,NPTS,K,IC)
C          ARRAY  - NAME OF ARRAY CONTAINING VALUES TO BE SCALED
C          DIST   - DISTANCE IN INCHES OVER WHICH DATA IS TO BE SCALED
C          NPTS   - NUMBER OF DATA ELEMENTS TO BE SCANNED
C          K      - REPEAT CYCLE OF A MIXED ARRAY
C                     K<0, INDICATES USER WISHES VALUES TO DECREASE WITH DIST
C          IC     - =0 FOR LINEAR SCALING
C                   =1 FOR LOGARITHMIC SCALING
C
C     SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C        NONE
C
C     COMMENTS
C        SCALE PARAMETERS ARE STORED AT THE END OF THE ARRAY.  THUS ARRAY
C          MUST BE LARGER THAN THE NUMBER OF POINT AT LEAST BY 2.
C
CALCOMP  PRODUCT NUMBER  411010*2-G              EBCDIC PUNCH
CALCOMP HCBS FOR CDC-6000     906/EM RS-232 ONLINE          JUL, 1979 *
CALCOMP  COPYRIGHT 1979 CALIFORNIA COMPUTER PRODUCTS INC.
C
C     MODIFIED FOR LOGARITHMIC SCALING BY: Ed Anderson, NOAO, May 1985.
C
      SUBROUTINE SCALE1(ARRAY,DIST,NPTS,K,IC)
      DIMENSION  ARRAY(1),SAVE(6)
      LOGICAL INC
      SAVE(1)=1.0
      SAVE(2)=2.0
      SAVE(3)=4.0
      SAVE(4)=5.0
      SAVE(5)=8.0
      SAVE(6)=10.0
      FAD=0.01
      INC=.FALSE.
C
C..............................................DETERMINE DIRECTION OF INCREMENT
      IF (K.LT.0) THEN
         INC=.TRUE.
         K=IABS(K)
      ENDIF
C
C.............................................FIND THE MAX AND MIN OF THE ARRAY
      N=NPTS*K
      YMIN=ARRAY(1)
      YMAX=YMIN
      DO 10 I=1,N,K
         YMIN=AMIN1(YMIN,ARRAY(I))
         YMAX=AMAX1(YMAX,ARRAY(I))
   10 CONTINUE
C
C.......................IF THE VALUES ARE A CONST (DELTAV=0) THEN FUDGE A SCALE
      IF (YMIN.EQ.YMAX) THEN
         IF (IC.EQ.0) THEN
            FIRSTV=AINT(YMIN-1.)
            DELTAV=2./DIST
         ELSE IF (IC.EQ.1) THEN
            IF (YMIN.LE.0.) THEN
               CALL ERR(' SCALE1 ERROR:  LOG(-VE VALUE)')
               RETURN
            ENDIF
            AI=10.
   20       IF (AI.LT.YMIN) THEN
               AI=AI*10.
               GOTO 20
            ENDIF
            FIRSTV=AINT(ALOG10(AI-1))
            DELTAV=DIST
         ELSE
            CALL ERR(' SCALE1 ERROR:  INVALID VALUE OF IC')
            RETURN
         ENDIF
      GOTO 3       
      ENDIF
C
C...........................IF LOGARITHMIC SCALING REQUIRED GO TO THAT SECTION
      IF (IC.EQ.1) GOTO 2
 
      FIRSTV=YMIN
      IF (YMIN.LT.0.) FAD=FAD-1.0
      DELTAV=(YMAX-FIRSTV)/DIST
      I=ALOG10(DELTAV)+1000.0
      P=10.0**(I-1000)
      DELTAV=DELTAV/P-0.01
      DO 30 I=1,7
         IS=I
         IF  (SAVE(I).GE.DELTAV) GOTO 1
   30 CONTINUE

    1 DELTAV=SAVE(IS)*P
      FIRSTV=DELTAV*AINT(YMIN/DELTAV+FAD)
      T=FIRSTV+(DIST+0.01)*DELTAV
 
      IF (T.LT.YMAX) THEN
         FIRSTV=P*AINT(YMIN/P+FAD)
         T=FIRSTV+(DIST+.01)*DELTAV
         IF (T.LT.YMAX) THEN
            IS=IS+1
            GOTO 1
         ENDIF
      ENDIF
      FIRSTV=FIRSTV-AINT((DIST+(FIRSTV-YMAX)/DELTAV)/2.0)*DELTAV
 
      IF ((YMIN*FIRSTV).LE.0.) FIRSTV=0.
      IF (INC) THEN
         FIRSTV=FIRSTV+AINT(DIST+.5)*DELTAV
         DELTAV=-DELTAV
      ENDIF
      GOTO 3
C
C...........................................................LOGARITHMIC SCALING
    2 IF (IC.NE.1) THEN
         CALL ERR(' SCALE1 ERROR:  INVALID VALUE OF IC')
         RETURN
      ENDIF
      IF ((YMIN.LE.0.).OR.(YMAX.LE.0.)) THEN
         CALL ERR(' SCALE1 ERROR:  LOG(-VE VALUE)')
         RETURN
      ENDIF
      AI=10.
   40 IF (AI.LT.YMIN) THEN
         AI=AI*10.
         GOTO 40
      ENDIF
      FIRSTV=AINT(ALOG10(AI-1))
      AI=10.
   50 IF (YMAX.GE.AI) THEN
         AI=AI*10.
         GOTO 50
      ENDIF
      CYC=ANINT(ALOG10(AI))-FIRSTV
      DELTAV=DIST/CYC
C
C..............................................PUT SCALE FACTORS INTO THE ARRAY
    3 N=N+1
      ARRAY(N)=FIRSTV
      N=N+K
      ARRAY(N)=DELTAV
      RETURN
      END
